
local Apollo = require "Apollo"

local tAddon = Apollo.GetAddon("MischhBossModules")
local ConfigEnv = {}
tAddon.ConfigEnv = ConfigEnv

local tConfigENV;
function ConfigEnv.Apply(tConfig, fConfig)
  if not fConfig then return tConfig end

  tConfigENV.tConfig = tConfig
  tConfigENV.nPosition = 100
  debug.setfenv(fConfig, tConfigENV)
  fConfig(tConfig)
  return tConfig
end

local tDefaults = {
  units = {enable=true, label="ERR_DEFAULT_LABEL"},
  lines = {enable=true, label="ERR_DEFAULT_LABEL"},
  timers = {enable=true, label="ERR_DEFAULT_LABEL"},
  casts = {enable=true, label="ERR_DEFAULT_LABEL"},
  alerts = {enable=true, label="ERR_DEFAULT_LABEL"},
  sounds = {enable=true, label="ERR_DEFAULT_LABEL", file="alert"},
  icons = {enable=true, label="ERR_DEFAULT_LABEL"},
  texts = {enable=true, label="ERR_DEFAULT_LABEL"},
  auras = {enable=true, label="ERR_DEFAULT_LABEL"},
}


tConfigENV = setmetatable({
  _G = _G, --we actually dont want to restrict the access. But we dont expect it to be used.
  print = function(...) print(...) end, --for debug purposes. typing _G.print would get really old, really fast.

  tConfig = nil,

  unit = function(t) t[0] = "units"; return t end,
  line = function(t) t[0] = "lines"; return t end,
  timer = function(t) t[0] = "timers"; return t end,
  cast = function(t) t[0] = "casts"; return t end,
  alert = function(t) t[0] = "alerts"; return t end,
  sound = function(t) t[0] = "sounds"; return t end,
  icon = function(t) t[0] = "icons"; return t end,
  text = function(t) t[0] = "texts"; return t end,
  aura = function(t) t[0] = "auras"; return t end,
},{__index = function(t, strKey)
  local tConfig = rawget(t, "tConfig")
  local nPosition = rawget(t, "nPosition")
  rawset(t, "nPosition", nPosition+1)

  return function(tAdditions)

    --find all predefined values
    local tPrototype = {}
    for strField, oValue in pairs(tAdditions) do
      if type(strField) == "string" then
        tPrototype[strField] = oValue
      end
    end

    --apply these predefenitions, if not already defined.
    for _, tOptions in ipairs(tAdditions) do
      for strField, oValue in pairs(tPrototype) do
        if tOptions[strField] == nil then
          tOptions[strField] = oValue
        end
      end
    end

    --apply constant defaults, if not already defined.
    for _, tOptions in ipairs(tAdditions) do
      if tDefaults[tOptions[0]] then
        for strField, oValue in pairs(tDefaults[tOptions[0]]) do
          if tOptions[strField] == nil then
            tOptions[strField] = oValue
          end
        end
      end
      tOptions.position = tOptions.position or nPosition
    end

    --move the comleted configs into the correct configType
    for _, tOptions in ipairs(tAdditions) do
      local configType = tOptions[0]
      tOptions[0] = nil
      if configType then
        tConfig[configType] = tConfig[configType] or {}
        tConfig[configType][strKey] = tOptions
      end
    end
  end
end})
