local Apollo = require "Apollo"

local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


-- nUnitId, strUnitName, tUnit,
-- nSpellId, strSpellName, -- for Buffs/Casts (Casts have no SpellId)
-- tBuff, nStack, --only for Buffs
-- tCast --only for Casts
-- strMessage, strMessageSource, --only for Datachron (OnNPCSay OnNPCYell OnNPCWhisper OnDatachron)
-- nHealthPercent, --for Health Changed/Reached
-- nHealthReached --for HealthReached
-- nDuration, --Buff/Cast
-- bInCombat, --UnitCreated


function Encounter:OnUnitCreated(nUnitId, tUnit, strUnitName, bInCombat, ...)
	if not self.run then return end

	--Okay. This is one of the most annoying things for bossmods in this game:
		--Targeting a unit 'Creates' it again. - We want to prevent this mechanic!
	if self.tCreatedUnits and nUnitId then --i dont know... i got some very weird errors...
		if self.tCreatedUnits[nUnitId] and self.tCreatedUnits[nUnitId].tUnit == tUnit then return end
		self.tCreatedUnits[nUnitId] = {
			nHelathReached = 101, --we want updates for 'HealthReached: 100%'
			tUnit = tUnit,
		}
	end

	self:FireCallbacks("UnitCreated", {
		nUnitId, tUnit, strUnitName, bInCombat, ...,
		nUnitId = nUnitId,
		tUnit = tUnit,
		strUnitName = strUnitName,
		bInCombat = bInCombat,
	}, nUnitId, tUnit, strUnitName, bInCombat)
end

function Encounter:OnUnitDestroyed(nUnitId, tUnit, strUnitName, ...)
	if not self.run then return end

	if self.tCreatedUnits[nUnitId] and self.tCreatedUnits[nUnitId].tUnit == tUnit then
		self.tCreatedUnits[nUnitId] = nil
	end

	self:FireCallbacks("UnitDestroyed", {
		nUnitId, tUnit, strUnitName, ...,
		nUnitId = nUnitId,
		tUnit = tUnit,
		strUnitName = strUnitName,
	}, nUnitId, tUnit, strUnitName)
end

function Encounter:OnBuffAdded(nUnitId, nSpellId, strSpellName, tBuff, strUnitName, nStack, nDuration, ...)
	if not self.run then return end

	self:FireCallbacks("BuffAdded", {
		nUnitId, strUnitName, tBuff.tUnit, nSpellId, strSpellName, tBuff, nStack, nDuration, ...,
		nUnitId = nUnitId,
		strUnitName = strUnitName,
		tUnit = tBuff.tUnit,
		nSpellId = nSpellId,
		strSpellName = strSpellName,
		tBuff = tBuff,
		nStack = nStack,
		nDuration = nDuration,
	}, nUnitId, strUnitName, tBuff.tUnit, nSpellId, strSpellName, tBuff, nStack, nDuration)
end

function Encounter:OnBuffUpdated(nUnitId, nSpellId, strSpellName, tBuff, strUnitName, nStack, nDuration, ...)
	if not self.run then return end

	self:FireCallbacks("BuffUpdated", {
		nUnitId, strUnitName, tBuff.tUnit, nSpellId, strSpellName, tBuff, nStack, nDuration, ...,
		nUnitId = nUnitId,
		strUnitName = strUnitName,
		tUnit = tBuff.tUnit,
		nSpellId = nSpellId,
		strSpellName = strSpellName,
		tBuff = tBuff,
		nStack = nStack,
		nDuration = nDuration,
	}, nUnitId, strUnitName, tBuff.tUnit, nSpellId, strSpellName, tBuff, nStack, nDuration)
end

function Encounter:OnBuffRemoved(nUnitId, nSpellId, strSpellName, tBuff, strUnitName, ...)
	if not self.run then return end

	self:FireCallbacks("BuffRemoved", {
		nUnitId, strUnitName, tBuff.tUnit, nSpellId, strSpellName, tBuff, ...,
		nUnitId = nUnitId,
		strUnitName = strUnitName,
		tUnit = tBuff.tUnit,
		nSpellId = nSpellId,
		strSpellName = strSpellName,
		tBuff = tBuff,
	}, nUnitId, strUnitName, tBuff.tUnit, nSpellId, strSpellName, tBuff)
end

function Encounter:OnCastStart(nUnitId, strSpellName, tCast, strUnitName, nDuration, ...)
	if not self.run then return end

	self:FireCallbacks("CastStart", {
		nUnitId, strUnitName, tCast.tUnit, strSpellName, tCast, nDuration, ...,
		nUnitId = nUnitId,
		strUnitName = strUnitName,
		tUnit = tCast.tUnit,
		strSpellName = strSpellName,
		tCast = tCast,
		nDuration = nDuration,
	}, nUnitId, strUnitName, tCast.tUnit, strSpellName, tCast, nDuration)
end

function Encounter:OnCastEnd(nUnitId, strSpellName, tCast, strUnitName, ...)
	if not self.run then return end

	self:FireCallbacks("CastEnd", {
		nUnitId, strUnitName, tCast.tUnit, strSpellName, tCast, tCast.nDuration, ...,
		nUnitId = nUnitId,
		strUnitName = strUnitName,
		tUnit = tCast.tUnit,
		strSpellName = strSpellName,
		tCast = tCast,
		nDuration = tCast.nDuration,
	}, nUnitId, strUnitName, tCast.tUnit, strSpellName, tCast, tCast.nDuration)
end

function Encounter:OnDatachron(strMessage, strUnitName, strMessageSource, ...)
	if not self.run then return end

	self:FireCallbacks("Datachron", {
		strUnitName, strMessage, strMessageSource, ...,
		strUnitName = strUnitName,
		strMessage = strMessage,
		strMessageSource = strMessageSource,
	}, strUnitName, strMessage, strMessageSource)
end

function Encounter:OnHealthChanged(nUnitId, nHealthPercent, strUnitName, tUnit, ...)
	if not self.run then return end

	self:FireCallbacks("HealthChanged", {
		nUnitId, strUnitName, tUnit, nHealthPercent, ...,
		nUnitId = nUnitId,
		strUnitName = strUnitName,
		tUnit = tUnit,
		nHealthPercent = nHealthPercent,
	}, nUnitId, strUnitName, tUnit, nHealthPercent)



	local tCreatedUnit = self.tCreatedUnits[nUnitId]
	if not tCreatedUnit then return end

	local nPrevHealth = tCreatedUnit.nHealthReached or 101
	local nCurrHealth = math.ceil(nHealthPercent)
	tCreatedUnit.nHealthReached = nCurrHealth

	if nCurrHealth < nPrevHealth then
		--if we've gone from 101% to 96% in one frame, we want to inform about 100%, 99%, 98%, 97% and 96%.
		for nHealthReached = nPrevHealth-1, nCurrHealth, -1 do
			self:FireCallbacks("HealthReached", {
				nUnitId, strUnitName, tUnit, nHealthPercent, nHealthReached,
				nUnitId = nUnitId,
				strUnitName = strUnitName,
				tUnit = tUnit,
				nHealthPercent = nHealthPercent,
				nHealthReached = nHealthReached,
			}, nUnitId, strUnitName, tUnit, nHealthPercent, nHealthReached)
		end
	end
end

function Encounter:OnFrame()
	if not self.run then return end

	self.tick = Apollo.GetTickCount()

	if not self.lastCheck then
		self.lastCheck = self.tick
	end

	if (self.tick - self.lastCheck) > (self.config.interval or 0) then
		self:FireCallbacks("OnFrame", {})

		self.lastCheck = self.tick
	end
end

function Encounter:OnCommMessage(strUnitName, strReason, tData)
	if not self.run then return end
	self:FireCallbacks("CommMessage", {
		strUnitName, strReason, tData,
		strUnitName = strUnitName,
		strReason = strReason,
		tData = tData,
	}, strUnitName, strReason, tData)
end

function Encounter:OnEnable()
	self.run = true
	self:RemoveEvents()
	self:RegisterEvents()


	self:FireCallbacks("Enable", {})
end

function Encounter:OnDisable()
	self.run = false
	self:RemoveEvents()

	self:FireCallbacks("Disable", {})
end

-- Register Event Handler for the module
function Encounter:RegisterEvents()
	for strEvent, strMethod in pairs(self.tEvents) do
		Apollo.RegisterEventHandler(strEvent, strMethod, self)
	end
end

-- Unregister Event Handler for the module
function Encounter:RemoveEvents()
	self.tVars = {}
	self.tTrackedUnits = {}
	self.tCreatedUnits = {}
	for _, tTimer in pairs(self.tRunningTimers) do
		tTimer:Stop()
	end
	self.tRunningTimers = {}
	for strEvent in pairs(self.tEvents) do
		Apollo.RemoveEventHandler(strEvent, self)
	end
end

--arArgs can be strMethod aswell. Method wont be constructed then.
function Encounter:AutoRegisterEvent(strEvent, arArgs)
	local strMethod; --either "OnEvent_"..strEvent or tostring(arArgs)
	if type(arArgs) == "table" then
		strMethod = "OnEvent_"..strEvent

		self[strMethod] = function(this, ...)
			if not this.run then return end

			local tArgs = {...}
			for i, strName in ipairs(arArgs) do
				tArgs[strName] = select(i, ...)
			end

			this:FireCallbacks(strMethod, tArgs, ...)
		end
	elseif type(arArgs) == "string" then
		strMethod = arArgs
	end

	if strMethod then
		self.tEvents[strEvent] = strMethod
	end

	return strMethod
end

function Encounter:FireCallbacks(strType, tArgs, ...)
	tArgs = tArgs or {}

	tArgs["self"] = self

	if self.tCallbacks[strType] then
		for _, fCallback in ipairs(self.tCallbacks[strType]) do
			fCallback(self, tArgs, ...)
		end
	end
end

function Encounter:RegisterCallback(strType, fCallback)
	self.tCallbacks[strType] = self.tCallbacks[strType] or {}
	table.insert(self.tCallbacks[strType], fCallback)
end