local Apollo = require "Apollo"

local tAddon = Apollo.GetAddon("MischhBossModules")
local MethodEnv = {}
tAddon.MethodEnv = MethodEnv

local tMethodENV = setmetatable({
  tEncounter = nil,
}, {__index = _G})
local _arg;
local	_env;
local _var;

function MethodEnv.Apply(tEncounter, fMethodgen)
  if not fMethodgen then return end

  tMethodENV.tEncounter = tEncounter
  debug.setfenv(fMethodgen, tMethodENV)

  rawset(_var, "tEncounter", tEncounter)
  fMethodgen(_arg,_env,_var)
  rawset(_var, "tEncounter", nil)
end

local _lookup = setmetatable({
  --[_arg] = function...
  --[_env] = function... (defined right below)
},{__mode='k'})

local function buildRef()
  return setmetatable({},{
    __index = function(t,k)
      local tRef = buildRef()

      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)

        local tbl = t
        if rawget(_lookup, t) then tbl = _lookup[t](tArgs, tGlobals) end
        local key = k
        if rawget(_lookup, k) then key = _lookup[k](tArgs, tGlobals) end

        -- if type(tbl) ~= "table" and type(tbl) ~= "userdata" then
        -- 	error(("Tried to Index a (%s)-value, key: %s"):format(type(tbl), tostring(key)))
        -- end
        return tbl[key]
      end

      return tRef
    end,
    __call = function(t, ...)
      local tRef = buildRef()
      local tK = {...}
      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)
        local f = t
        if  rawget(_lookup, t) then f = _lookup[t](tArgs, tGlobals) end

        local copy = {unpack(tK)}
        for i,v in pairs(copy) do
          if rawget(_lookup, v) then copy[i] = _lookup[v](tArgs, tGlobals) end
        end

        return f(unpack(copy))
      end

      return tRef
    end,
    __unm = function(t)
      local tRef = buildRef()
      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)
        local val = t
        if rawget(_lookup, t) then val = _lookup[t](tArgs, tGlobals) end

        return -val
      end

      return tRef
    end,
    __add = function(a, b)
      local tRef = buildRef()
      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)
        local A = a
        if rawget(_lookup, a) then A = _lookup[a](tArgs, tGlobals) end
        local B = b
        if rawget(_lookup, b) then B = _lookup[b](tArgs, tGlobals) end

        return A+B
      end

      return tRef
    end,
    __sub = function(a, b)
      local tRef = buildRef()
      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)
        local A = a
        if rawget(_lookup, a) then A = _lookup[a](tArgs, tGlobals) end
        local B = b
        if rawget(_lookup, b) then B = _lookup[b](tArgs, tGlobals) end

        return A-B
      end

      return tRef
    end,
    __mul = function(a, b)
      local tRef = buildRef()
      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)
        local A = a
        if rawget(_lookup, a) then A = _lookup[a](tArgs, tGlobals) end
        local B = b
        if rawget(_lookup, b) then B = _lookup[b](tArgs, tGlobals) end

        return A*B
      end

      return tRef
    end,
    __div = function(a, b)
      local tRef = buildRef()
      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)
        local A = a
        if rawget(_lookup, a) then A = _lookup[a](tArgs, tGlobals) end
        local B = b
        if rawget(_lookup, b) then B = _lookup[b](tArgs, tGlobals) end

        return A/B
      end

      return tRef
    end,
    __mod = function(a, b)
      local tRef = buildRef()
      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)
        local A = a
        if rawget(_lookup, a) then A = _lookup[a](tArgs, tGlobals) end
        local B = b
        if rawget(_lookup, b) then B = _lookup[b](tArgs, tGlobals) end

        return A%B
      end

      return tRef
    end,
    __pow = function(a, b)
      local tRef = buildRef()
      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)
        local A = a
        if rawget(_lookup, a) then A = _lookup[a](tArgs, tGlobals) end
        local B = b
        if rawget(_lookup, b) then B = _lookup[b](tArgs, tGlobals) end

        return A^B
      end

      return tRef
    end,
    __concat = function(a, b)
      local tRef = buildRef()
      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)
        local A = a
        if rawget(_lookup, a) then A = _lookup[a](tArgs, tGlobals) end
        local B = b
        if rawget(_lookup, b) then B = _lookup[b](tArgs, tGlobals) end

        return A..B
      end

      return tRef
    end,
  })
end


local tArgEnvMeta = {
  __index = function(tArg, strVar)
    local tRef = buildRef()

    if tArg == _arg then
      _lookup[tRef] = function(tArgs, tGlobals) --luacheck: no unused args
        return tArgs[strVar]
      end
    elseif tArg == _env then
      _lookup[tRef] = function(tArgs, tGlobals) --luacheck: no unused args
        return tGlobals[strVar]
      end
    end
    return tRef
  end,
  __call = function(tArg, ...) --luacheck: no unused args
    local tRef = buildRef()

    local arArgs = {...}

    _lookup[tRef] = function(tArgs, tGlobals)
      local arCopy = {unpack(arArgs)}

      for i,v in pairs(arCopy) do
        if rawget(_lookup, v) then
          arCopy[i] = _lookup[v](tArgs, tGlobals)
        else
          arCopy[i] = v
        end
      end

      return unpack(arCopy)
    end

    return tRef
  end
}
_arg = setmetatable({}, tArgEnvMeta)
_env = setmetatable({}, tArgEnvMeta)
_lookup[_arg] = function(tArgs, tGlobals) return tArgs end --luacheck: no unused args
_lookup[_env] = function(tArgs, tGlobals) return tGlobals end --luacheck: no unused args


local function buildVar(strVar)
  local function set(tArgs, oVal)
    tArgs.self.tVars[strVar] = oVal
  end


  return setmetatable({},{
    __index = function(t,k)
      local tRef = buildRef()

      --overwrite the function
      _lookup[tRef] = function(tArgs, tGlobals)

        local tbl = t
        if rawget(_lookup, t) then tbl = _lookup[t](tArgs, tGlobals) end
        local key = k
        if rawget(_lookup, k) then key = _lookup[k](tArgs, tGlobals) end

        return tbl[key]
      end

      return tRef
    end,
    __call = function(_, val)
      local tRef = {}

      _lookup[tRef] = function(tArgs, tGlobals)
        return function()
          local oVal = val
          if rawget(_lookup, oVal) then oVal = _lookup[oVal](tArgs, tGlobals) end

          set(tArgs, oVal)

          return oVal
        end
      end

      return tRef
    end,
    __unm = function(t)
      local tRef = {}

      _lookup[tRef] = function(tArgs, tGlobals)
        return function()
          local oVal = - _lookup[t](tArgs, tGlobals)

          set(tArgs, oVal)

          return oVal
        end
      end

      return tRef
    end,
    __add = function(t, val)
      local tRef = {}

      _lookup[tRef] = function(tArgs, tGlobals)
        return function()
          local oValA = _lookup[t](tArgs, tGlobals)
          local oValB = val
          if rawget(_lookup, oValB) then oValB = _lookup[oValB](tArgs, tGlobals) end

          local oVal = oValA+oValB

          set(tArgs, oVal)

          return oVal
        end
      end

      return tRef
    end,
    __sub = function(t, val)
      local tRef = {}

      _lookup[tRef] = function(tArgs, tGlobals)
        return function()
          local oValA = _lookup[t](tArgs, tGlobals)
          local oValB = val
          if rawget(_lookup, oValB) then oValB = _lookup[oValB](tArgs, tGlobals) end

          local oVal = oValA-oValB

          set(tArgs, oVal)

          return oVal
        end
      end

      return tRef
    end,
    __mul = function(t, val)
      local tRef = {}

      _lookup[tRef] = function(tArgs, tGlobals)
        return function()
          local oValA = _lookup[t](tArgs, tGlobals)
          local oValB = val
          if rawget(_lookup, oValB) then oValB = _lookup[oValB](tArgs, tGlobals) end

          local oVal = oValA*oValB

          set(tArgs, oVal)

          return oVal
        end
      end

      return tRef
    end,
    __div = function(t, val)
      local tRef = {}

      _lookup[tRef] = function(tArgs, tGlobals)
        return function()
          local oValA = _lookup[t](tArgs, tGlobals)
          local oValB = val
          if rawget(_lookup, oValB) then oValB = _lookup[oValB](tArgs, tGlobals) end

          local oVal = oValA/oValB

          set(tArgs, oVal)

          return oVal
        end
      end

      return tRef
    end,
    __mod = function(t, val)
      local tRef = {}

      _lookup[tRef] = function(tArgs, tGlobals)
        return function()
          local oValA = _lookup[t](tArgs, tGlobals)
          local oValB = val
          if rawget(_lookup, oValB) then oValB = _lookup[oValB](tArgs, tGlobals) end

          local oVal = oValA%oValB

          set(tArgs, oVal)

          return oVal
        end
      end

      return tRef
    end,
    __pow = function(t, val)
      local tRef = {}

      _lookup[tRef] = function(tArgs, tGlobals)
        return function()
          local oValA = _lookup[t](tArgs, tGlobals)
          local oValB = val
          if rawget(_lookup, oValB) then oValB = _lookup[oValB](tArgs, tGlobals) end

          local oVal = oValA^oValB

          set(tArgs, oVal)

          return oVal
        end
      end

      return tRef
    end,
    __concat = function(t, val)
      local tRef = {}

      _lookup[tRef] = function(tArgs, tGlobals)
        return function()
          local oValA = _lookup[t](tArgs, tGlobals)
          local oValB = val
          if rawget(_lookup, oValB) then oValB = _lookup[oValB](tArgs, tGlobals) end

          local oVal = oValA..oValB

          set(tArgs, oVal)

          return oVal
        end
      end

      return tRef
    end,
  })
end

local tVarMeta = {
  __index = function(tVars, strVar) --luacheck: no unused args
    local tVar = buildVar(strVar)

    _lookup[tVar] = function(tArgs, tGlobals) --luacheck: no unused args
      return tArgs.self.tVars[strVar]
    end

    return tVar
  end,
  __call = function(tVars, oVar) --luacheck: no unused args
    local tRef = buildRef()

    _lookup[tRef] = function(tArgs, tGlobals)
      local oCopy = oVar
      if rawget(_lookup, oCopy) then oCopy = _lookup[oCopy](tArgs, tGlobals) end
      if type(oCopy) == "table" then
        local tCopy = {}
        for idx, val in pairs(oCopy) do
          if rawget(_lookup, idx) then idx = _lookup[idx](tArgs, tGlobals) end
          if rawget(_lookup, val) then val = _lookup[val](tArgs, tGlobals) end

          if idx ~= nil then
            tCopy[idx] = val
          end
        end
        oCopy = tCopy
      end

      return oCopy
    end

    return tRef
  end,
  __newindex = function(tVars, strVar, oValue)
    local fCallback = function(self)
      local oVal = oValue
      if rawget(_lookup, oVal) then oVal = _lookup[oVal]({self=self}, _G) end
      self.tVars[strVar] = oVal
    end
    local tEncounter = rawget(tVars, "tEncounter")
    if tEncounter then
      tEncounter:RegisterCallback("Enable", fCallback)
    end
  end
}
_var = setmetatable({}, tVarMeta)


local tModes = {
  ["match"]=function(self,a,b, tArgs)
    if type(a) == "string" and type(b) == "string" then
      local strMatch = self.L[a]:match(self.L[b])
      if strMatch then
        tArgs.strMatch = strMatch
        return true
      end
      return false
    else
      return false
    end
  end,
  ["=="]=function(self,a,b)
    if type(a) == "string" and type(b) == "string" then
      if self.L[a]:find(self.L[b], nil, true) then
        return true
      end
    else
      return a==b
    end
  end,
  ["==="]=function(_,a,b)
    return a==b
  end,
  ["<"]=function(_,a,b)
    return a<b
  end,
  [">"]=function(_,a,b)
    return a>b
  end,
  ["~="]="==",["~=="]="===",["<="]=">",[">="]="<",}
local function CheckTableFilter(self, oCompare, tTable, tArgs)
  local mode = unpack(tTable)
  if mode and rawget(_lookup, mode) then mode = _lookup[mode](tArgs, _G) end
  local ignoreIndex = 1

  if not mode or not tModes[mode] then
    mode = "=="
    ignoreIndex = nil
  end

  local fComp = tModes[mode] --either a function, or "==", "<", ">"
  local bComp = true

  if tModes[fComp] then
    fComp = tModes[fComp]
    bComp = false
  end

  for oIdx, oVal in pairs(tTable) do
    if oIdx ~= ignoreIndex then
      if rawget(_lookup, oVal) then oVal = _lookup[oVal](tArgs, _G) end
      if fComp(self, oCompare, oVal, tArgs) then return bComp == true end
    end
  end

  return bComp == false
end

local function CreateFilter(tFilter)
  --the Filter should be structured like this:

  -- tFilter[key] = val  where
    -- key is _arg or _env or
    -- key references tArgs[key] (aka. _arg[key])

    -- val is _arg or _env or
    -- val is an array of values to compare to (if 1 matches, this pair succeeds)
    -- val is the value to compare to
      -- if both key and val are strings, stearch the val inside the key.


  return function(self, tArgs)
    for oKey, oVal in pairs(tFilter) do

      local oValueCompare;

      if rawget(_lookup, oKey) then
        oValueCompare = _lookup[oKey](tArgs, _G)
      else
        oValueCompare = tArgs[oKey]
      end

      if type(oVal) == "table" and not rawget(_lookup, oVal) then
        if not CheckTableFilter(self, oValueCompare, oVal, tArgs) then
          return false --Failed this part of the Filter!
        end
      else
        local oValueCompare2 = oVal
        if rawget(_lookup, oVal) then
          oValueCompare2 = _lookup[oVal](tArgs, _G)
        end

        if type(oValueCompare) == "string" and type(oValueCompare2) == "string" then
          if not self.L[oValueCompare]:find(self.L[oValueCompare2], nil, true) then
            return false --Failed this part of the Filter!
          end
        elseif oValueCompare ~= oValueCompare2 then
          return false --Failed this part of the Filter!
        end
      end
    end --passed the filter

    return true
  end
end

local function DeepLookup(tCheck, tArgs, tGlobals, tChecked)
  tChecked = tChecked or {}

  if tChecked[tCheck] then
    return tChecked[tCheck], tCheck~=tChecked[tCheck]
  end

  local tCopy = {}
  tChecked[tCheck] = tCopy

  local bModified = false --if we didnt lookup anything, we return the original!
  --scince this would make this ALOT more complicated, we also copy, if there is a recursive structure.
  -- example: local a = {}; a[a] = a

  for oKey,oVal in pairs(tCheck) do
    if rawget(_lookup, oKey) then
      oKey = _lookup[oKey](tArgs, tGlobals)
      bModified = true
    elseif type(oKey) == "table" then
      local bMod;
      oKey, bMod = DeepLookup(oKey, tArgs, tGlobals, tChecked)

      bModified = bModified or bMod
    end

    if rawget(_lookup, oVal) then
      oVal = _lookup[oVal](tArgs, tGlobals)
      bModified = true
    elseif type(oVal) == "table" then
      local bMod;
      oVal, bMod = DeepLookup(oVal, tArgs, tGlobals, tChecked)

      bModified = bModified or bMod
    end

    tCopy[oKey] = oVal
  end

  if not bModified then
    tChecked[tCheck] = tCheck
    --we do this, so that things like this table doesnt end up classifying 'a' as modified/recusive
      -- local a = {x = 1, y = 2}
      -- {a, a}
  end

  if bModified then
    return tCopy, true
  else
    return tCheck, false
  end
end

local function CreateCallback(oCallbackArg)

  if type(oCallbackArg) == "function" then
    return function(self, tArgs, ...) --luacheck: no unused args
      oCallbackArg(self,...)
    end
  elseif type(oCallbackArg) ~= "table" then
    return function(self, tArgs, ...) --luacheck: no unused args
      self[oCallbackArg](self, ...)
    end
  elseif rawget(_lookup, oCallbackArg) then
    return function(self,tArgs, ...)
      local oCallback = _lookup[oCallbackArg](tArgs, _G)
      if type(oCallback) == "function" then
        oCallback(self, ...)
      elseif type(oCallback) == "string" then
        self[oCallback](self, ...)
      end
    end
  else
    return function(self, tArgs, ...) --luacheck: no unused args
      local arCallback = DeepLookup(oCallbackArg, tArgs, _G)

      local oMethod = arCallback[1]
      if type(oMethod) == "function" then
        oMethod(select(2,unpack(arCallback)))
      else
        self[oMethod](self, select(2,unpack(arCallback)))
      end
    end
  end
end

local function PrepAndRegister(tEncounter, strType, tData)
  local tFilter = {}
  local arCallbacks = {}

  for i, v in pairs(tData) do
    if type(i) == "number" then --v is a callback!
      table.insert(arCallbacks, CreateCallback(v))
    else --this is a Filter!
      tFilter[i] = v
    end
  end

  local fFilter = CreateFilter(tFilter)

  --create the callback.
  local function fCallback(self, tArgs, ...)
    if fFilter(self, tArgs) then
      for _, fMethod in ipairs(arCallbacks) do
        fMethod(self, tArgs, ...)
      end
      return true --callback executed
    else
      return false --callback filtered
    end
  end

  if strType then
    tEncounter:RegisterCallback(strType, fCallback)
  end
  return fCallback
end

tMethodENV.Enable = function(t) PrepAndRegister(tMethodENV.tEncounter, "Enable", t) end
tMethodENV.Disable = function(t) PrepAndRegister(tMethodENV.tEncounter, "Disable", t) end
tMethodENV.UnitCreated = function(t) PrepAndRegister(tMethodENV.tEncounter, "UnitCreated", t) end
tMethodENV.UnitDestroyed = function(t) PrepAndRegister(tMethodENV.tEncounter, "UnitDestroyed", t) end
tMethodENV.BuffAdded = function(t) PrepAndRegister(tMethodENV.tEncounter, "BuffAdded", t) end
tMethodENV.BuffUpdated = function(t) PrepAndRegister(tMethodENV.tEncounter, "BuffUpdated", t) end
tMethodENV.BuffRemoved = function(t) PrepAndRegister(tMethodENV.tEncounter, "BuffRemoved", t) end
tMethodENV.CastStart = function(t) PrepAndRegister(tMethodENV.tEncounter, "CastStart", t) end
tMethodENV.CastEnd = function(t) PrepAndRegister(tMethodENV.tEncounter, "CastEnd", t) end
tMethodENV.Datachron = function(t) PrepAndRegister(tMethodENV.tEncounter, "Datachron", t) end
tMethodENV.HealthChanged = function(t) PrepAndRegister(tMethodENV.tEncounter, "HealthChanged", t) end
tMethodENV.HealthReached = function(t) PrepAndRegister(tMethodENV.tEncounter, "HealthReached", t) end
tMethodENV.CommMessage = function(t) PrepAndRegister(tMethodENV.tEncounter, "CommMessage", t) end
tMethodENV.Callback = function(strCallback, t) PrepAndRegister(tMethodENV.tEncounter, strCallback, t) end

tMethodENV.OnFrame = function(t)
  tMethodENV.tEncounter:AutoRegisterEvent("NextFrame", "OnFrame")
  PrepAndRegister(tMethodENV.tEncounter, "OnFrame", t)
end

tMethodENV.Event = function(strEvent, ...)
  local arArgs = {...}
  local t = select(-1, ...) --the last entry of ... is equal to t of the other functions.
  arArgs[#arArgs] = nil --this works even, if #arArgs == 0

  local strMethod = tMethodENV.tEncounter:AutoRegisterEvent(strEvent, arArgs)
  PrepAndRegister(tMethodENV.tEncounter, strMethod, t)
end

local IfMeta = {
  __div = function(If, t)
    local fCallback = PrepAndRegister(tMethodENV.tEncounter, nil, t)--Prep without Register.
    table.insert(If, fCallback) --we insert the callback at the end of the existing 'If'-Statement
    return If
  end
}
local IfFunction = function(tArgs, ...)
  for i = 1, select('#', ...), 1 do
    local fCallback = select(i, ...)
    local bExecuted = fCallback(tArgs.self, tArgs, unpack(tArgs))
    if bExecuted then break end
  end
end
tMethodENV.If = function(...)
  local tCheck = setmetatable({IfFunction, _arg}, IfMeta)

  for i = 1, select('#', ...), 1 do
    tCheck = tCheck / (select(i,...))
  end
  return tCheck
end

local prepTbl = {false, false, false, false, false, false, false, false}

local function TableConverter(strMethod, nArgs, strInverted, nInvArgs, ...)
  local tArgs = {...}

  local tTable = {strMethod, unpack(prepTbl)}
  for i, v in pairs(tArgs) do
    tTable[i+1] = v
  end

  return setmetatable(tTable, {
    __unm = function()
      return TableConverter(strInverted, nInvArgs, strInverted, nInvArgs, unpack(tArgs))
    end,
    __add = function(self, tConfig)
      self[nArgs+4] = tConfig
      return self
    end,
    __mod = function(self, oMod)
      self[nArgs+3] = oMod
      return self
    end,
  })

end

tMethodENV.Unit = 					function(...) return TableConverter("Unit", 					1, "RemoveUnit", 				0, ...) end
tMethodENV.TrackUnit = 			function(...) return TableConverter("TrackUnit",			1, "RemoveUnit", 				0, ...) end
tMethodENV.Timer = 					function(...) return TableConverter("Timer", 					2, "RemoveTimer", 			0, ...) end
tMethodENV.Cast = 					function(...) return TableConverter("Cast", 					1, "RemoveCast", 				0, ...) end
tMethodENV.Aura = 					function(...) return TableConverter("Aura", 					1, "RemoveAura", 				0, ...) end
tMethodENV.Alert = 					function(...) return TableConverter("Alert", 					1, "RemoveAlert", 			0, ...) end
tMethodENV.Sound = 					function(...) return TableConverter("Sound", 					0, "RemoveSound", 			0, ...) end
tMethodENV.Text = 					function(...) return TableConverter("Text", 					2, "RemoveText", 				0, ...) end
tMethodENV.TimerText = 			function(...) return TableConverter("TimerText", 			2, "RemoveTimerText", 	0, ...) end
tMethodENV.HealthText = 		function(...) return TableConverter("HealthText", 		2, "RemoveHealthText", 	0, ...) end
tMethodENV.Icon = 					function(...) return TableConverter("Icon", 					1, "RemoveIcon", 				0, ...) end
tMethodENV.Polygon = 				function(...) return TableConverter("Polygon", 				3, "RemovePolygon", 		0, ...) end
tMethodENV.Line = 					function(...) return TableConverter("Line", 					2, "RemoveLine", 				0, ...) end
tMethodENV.LineBetween = 		function(...) return TableConverter("LineBetween", 		2, "RemoveLineBetween", 0, ...) end
tMethodENV.Print = 					function(...) return TableConverter("Print", 					0, "Print", 						0, ...) end
tMethodENV.SendCommMessage =function(...) return TableConverter("SendCommMessage",2, "SendCommMessage", 	2, ...) end
tMethodENV.Delay = 					function(...) return TableConverter("Delay", 					0, "RemoveDelay",				0, ...) end
tMethodENV.FireCallbacks =	function(...) return TableConverter("FireCallbacks",	0, "FireCallbacks",			0, ...) end
tMethodENV.Count = 					function(...) return _arg.self:Count(...)																							end
tMethodENV.GetCount = 			function(...) return _arg.self:GetCount(...)																					end
tMethodENV.SetCount = 			function(...) return _arg.self:SetCount(...)																					end
tMethodENV.InRange = 				function(...) return _arg.self:InRange(...) 																					end
tMethodENV.GetDistance = 		function(...) return _arg.self:GetDistance(...) 																			end

tMethodENV.GameTime = function(...) return	_env.GameLib.GetGameTime(...) end