local XmlDoc = require "XmlDoc"
local ActionSetLib = require "ActionSetLib"
local GameLib = require "GameLib"

local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter

local Mod = Encounter.new("Avatus", {
  instance = "Datascape",
  displayName = "Avatus",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.avatus"},
    tZones = {
      [1] = {
        continentId = 52,
        parentZoneId = 98,
        mapId = 104,
      },
    },
  },
},{
  ["enUS"] = {
    -- Unit names.
    ["unit.avatus"] = "Avatus",
    ["unit.hand"] = "Holo Hand",
    ["unit.yellow_room"] = "Mobius Physics Constructor",
    ["unit.green_room"] = "Unstoppable Object Simulation",
    ["unit.blue_room"] = "Infinite Logic Loop",
    ["unit.red_room"] = "Excessive Force Protocol",
    ["unit.holo_cannon"] = "Holo Cannon",
    ["unit.shock_sphere"] = "Shock Sphere",
    ["unit.support_cannon"] = "Support Cannon",
    ["unit.green_room_platform"] = "Tower Platform",
    ["unit.rosdower"] = "Augmented Rowsdower",
    ["unit.hand_last_phase"] = "Fragmented Data Chunk",
    -- Datachron messages.
    ["datachron.portals_opened"] = "Avatus' power begins to surge! Portals have opened!",
    ["datachron.gun_grid"] = "SECURITY PROTOCOL: Gun Grid Activated.",
    ["datachron.red_room_boss_activation"] = "The Excessive Force Protocol's protective barrier has fallen.",
    ["datachron.red_room_boss_dead"] = "The Excessive Force Protocol has been terminated.",
    ["datachron.blue_room_purge_increase"] = "Escalating defense matrix system to level",
    -- Cast.
    ["cast.hand_crush"] = "Crushing Blow",
    ["cast.yellow_room_blind"] = "Data Flare",
    ["cast.obliteration_beam"] = "Obliteration Beam",
    -- BuffName with many ID.
    ["buff.blue_room_red_buff"] = "Red Empowerment Matrix",
    ["buff.blue_room_blue_buff"] = "Blue Disruption Matrix",
    ["buff.blue_room_green_buff"] = "Green Reconstitution Matrix",
    -- Timer bars.
    ["message.next_gun_grid"] = "Next gun grid",
    ["message.next_obliteration_beam"] = "Next obliteration beam",
    ["message.next_purge_increase"] = "Next increase of number of purge",
    ["message.next_purge_cycle"] = "Next purge cycle",
    ["message.next_support_cannon"] = "Next support cannon",
    ["message.next_hands"] = "Next holo: Hands",
    ["message.next_cannons"] = "Next holo: Cannons",
    ["message.next_blind"] = "Next blind",
    -- Message bars.
    ["alert.hand_spawned"] = "HOLO HAND SPAWNED",
    ["alert.phase2_soon"] = "P2 SOON !",
    ["alert.phase3_soon"] = "P3 SOON !",
    ["alert.kill_avatus"] = "KILL AVATUS !!!",
    ["alert.interrupt_hand_crush"] = "INTERRUPT CRUSHING BLOW!",
    ["alert.blind"] = "BLIND! TURN AWAY FROM BOSS",
    ["alert.gun_grid"] = "GUN GRID NOW!",
    ["alert.yellow_room_start"] = "Yellow Room: Combat started",
    ["alert.yellow_room_health"] = "Yellow Health: %d%%",

    ["alert.purge_blue"] = "%s. PURGE BLUE (%s: %s)",
    ["alert.purge_red"] = "%s. PURGE RED (%s: %s)",
    ["alert.purge_green"] = "%s. PURGE GREEN (%s: %s)",
    ["alert.purge_you"] = "YOU !!!",

    ["marker.north"] = "North",
    ["marker.south"] = "South",
    ["marker.east"] = "East",
    ["marker.west"] = "West",

    ["label.gun_grid"] = "Gun Grid",
    ["label.room_lines"] = "Colored Room Lines",
    ["label.yellow_room_boss"] = "Yellow Room: Boss",
    ["label.green_room_boss"] = "Green Room: Boss",
    ["label.blue_room_boss"] = "Blue Room: Boss",
    ["label.red_room_boss"] = "Red Room: Boss",
    ["label.shock_sphere"] = "Yellow Room: Shock Sphere",
    ["label.support_cannon"] = "Last Phase: Support Cannon",
    ["label.phase_switch"] = "Phase Switch",
    ["label.cardinal_markers"] = "Cardinal Markers",
    ["label.green_room_markers"] = "Green Room: Markers",
    ["label.hand_spawn"] = "Hand Spawn Point",
  },
  ["deDE"] = {
    -- Unit names.
    ["unit.avatus"] = "Avatus",
    ["unit.hand"] = "Holohand",
    ["unit.yellow_room"] = "Mobius"..string.char(226,128,153).." Physikkonstrukteur",
    ["unit.green_room"] = "Unaufhaltbare Objektsimulation",
    ["unit.blue_room"] = "Unendliche Logikschleife",
    ["unit.red_room"] = "Überzogene Gewaltprotokolle",
    ["unit.holo_cannon"] = "Holokanone",
    ["unit.shock_sphere"] = "Schocksphäre",
    ["unit.support_cannon"] = "Hilfskanone",
    ["unit.green_room_platform"] = "Turm Plattform", -- ["unit.green_room_platform"] = "Tower Platform",
    ["unit.rosdower"] = "Augmentierter Rowsdower", -- ["unit.rosdower"] = "Augmented Rowsdower",
    ["unit.hand_last_phase"] = "Fragmentiertes Daten Segment", -- ["unit.hand_last_phase"] = "Fragmented Data Chunk",
    -- Datachron messages.
    -- ["datachron.portals_opened"] = "Avatus' power begins to surge! Portals have opened!",
    ["datachron.portals_opened"] = "Portale haben sich geöffnet!",
    ["datachron.gun_grid"] = "SICHERHEITSPROTOKOLL: Waffenraster aktiviert.",
    ["datachron.red_room_boss_activation"] = "Die Barriere des Überzogenen Gewaltprotokolls ist gefallen.",
    ["datachron.red_room_boss_dead"] = "Das Überzogene Gewaltprotokoll wurde terminiert.",
    -- ["datachron.blue_room_purge_increase"] = "Escalating defense matrix system to level",
    -- Cast.
    ["cast.hand_crush"] = "Vernichtender Schlag",
    ["cast.yellow_room_blind"] = "Daten-Leuchtsignal",
    ["cast.obliteration_beam"] = "Vernichtungsstrahl",
    -- BuffName with many ID.
    ["buff.blue_room_red_buff"] = "Rote Ermächtigungsmatrix",
    ["buff.blue_room_blue_buff"] = "Blaue Unterbrechungsmatrix",
    ["buff.blue_room_green_buff"] = "Grüne Wiederherstellungsmatrix",
    -- Timer bars.
    ["message.next_gun_grid"] = "Nächstes Gun Grid",
    ["message.next_obliteration_beam"] = "Nächster Obliteration Beam",
    ["message.next_purge_increase"] = "Nächste Erhöhung der Purge Anzahl",
    ["message.next_purge_cycle"] = "Nächste Purges",
    ["message.next_support_cannon"] = "Nächste Support Cannon",
    ["message.next_hands"] = "Nächste Hände",
    ["message.next_cannons"] = "Nächste Cannons",
    ["message.next_blind"] = "Nächstes Blenden",
    -- Message bars.
    ["alert.hand_spawned"] = "NEUE HOLO HÄNDE!",
    ["alert.phase2_soon"] = "P2 BALD !",
    ["alert.phase3_soon"] = "P3 BALD !",
    ["alert.kill_avatus"] = "TÖTE AVATUS !!!",
    ["alert.interrupt_hand_crush"] = "INTERRUPT!",
    ["alert.blind"] = "BLENDEN! DREH DICH WEG VOM BOSS!",
    ["alert.gun_grid"] = "GUN GRID!",
    ["alert.yellow_room_start"] = "Gelber Raum: Kampfbeginn",
    ["alert.yellow_room_health"] = "Gelb Leben: %d%%",

    ["alert.purge_blue"] = "%s. PURGE BLAU (%s: %s)",
    ["alert.purge_red"] = "%s. PURGE ROT (%s: %s)",
    ["alert.purge_green"] = "%s. PURGE GRÜN (%s: %s)",
    ["alert.purge_you"] = "DU !!!",

    ["marker.north"] = "Nord",
    ["marker.south"] = "Süd",
    ["marker.east"] = "Ost",
    ["marker.west"] = "West",

    ["label.gun_grid"] = "Gun Grid",
    ["label.room_lines"] = "Farbraum Linien",
    ["label.yellow_room_boss"] = "Gelber Raum: Boss",
    ["label.green_room_boss"] = "Grüner Raum: Boss",
    ["label.blue_room_boss"] = "Blauer Raum: Boss",
    ["label.red_room_boss"] = "Roter Raum: Boss",
    ["label.shock_sphere"] = "Gelber Raum: Schocksphäre",
    ["label.support_cannon"] = "Letzte Phase: Hilfskanone",
    ["label.phase_switch"] = "Phasenwechsel",
    ["label.cardinal_markers"] = "Himmelsrichtungen",
    ["label.green_room_markers"] = "Grüner Raum: Markierungen",
    ["label.hand_spawn"] = "Hand Spawn Punkt",
  },
  ["frFR"] = {
    -- Unit names.
    ["unit.avatus"] = "Avatus",
    ["unit.hand"] = "Holo-main",
    ["unit.yellow_room"] = "Constructeur de physique de Möbius",
    ["unit.green_room"] = "Simulacre invincible",
    ["unit.blue_room"] = "Boucle de logique infinie",
    ["unit.red_room"] = "Protocole de force excessive",
    ["unit.holo_cannon"] = "Holocanon",
    ["unit.shock_sphere"] = "Sphère de choc",
    ["unit.support_cannon"] = "Canon d'appui",
    ["unit.green_room_platform"] = "Plateforme de la tour",
    ["unit.rosdower"] = "Tamarou augmenté",
    ["unit.hand_last_phase"] = "Données fragmentées",
    -- Datachron messages.
    ["datachron.portals_opened"] = "L'énergie d'Avatus commence à déferler ! Des portails se sont ouverts !",
    ["datachron.gun_grid"] = "PROTOCOLE DE SÉCURITÉ : pétoires activées.",
    -- ["datachron.red_room_boss_activation"] = "The Excessive Force Protocol's protective barrier has fallen.",
    ["datachron.red_room_boss_dead"] = "Le Protocole de force excessive a été abandonné.",
    ["datachron.blue_room_purge_increase"] = "Passage de la matrice de défense aux protocoles de niveau",
    -- Cast.
    ["cast.hand_crush"] = "Coup écrasant",
    ["cast.yellow_room_blind"] = "Signal de données",
    ["cast.obliteration_beam"] = "Rayon de suppression",
    -- BuffName with many ID.
    ["buff.blue_room_red_buff"] = "Matrice de renforcement rouge",
    ["buff.blue_room_blue_buff"] =  "Matrice disruptive bleue",
    ["buff.blue_room_green_buff"] = "Matrice de reconstitution verte",
    -- Timer bars.
    ["message.next_gun_grid"] = "Prochaine pétoires grille",
    ["message.next_obliteration_beam"] = "Prochain rayon de suppression",
    ["message.next_purge_increase"] = "Prochaine augmentation du nombre de purge",
    ["message.next_purge_cycle"] = "Prochain cycle de purge",
    ["message.next_support_cannon"] = "Prochain canon d'appui",
    ["message.next_hands"] = "Prochain holo: Mains",
    ["message.next_cannons"] = "Prochain holo: Cannons",
    ["message.next_blind"] = "Prochain aveuglement",
    -- Message bars.
    ["alert.hand_spawned"] = "HOLO-MAIN APPARITION",
    ["alert.phase2_soon"] = "P2 BIENTÔT !",
    ["alert.phase3_soon"] = "P3 BIENTÔT !",
    ["alert.kill_avatus"] = "TUEZ AVATUS !!!",
    ["alert.interrupt_hand_crush"] = "INTERROMPRE COUP ÉCRASANT!",
    ["alert.blind"] = "AVEUGLER! DOS AU BOSS",
    ["alert.gun_grid"] = "PÉTOIRES MAINTENANT!",
    ["alert.yellow_room_start"] = "Salle Jaune: Combat démarré",
    ["alert.yellow_room_health"] = "Mobius santé: %d%%",

    ["alert.purge_blue"] = "%s. PURGE BLEUE (%s: %s)",
    ["alert.purge_red"] = "%s. PURGE ROUGE (%s: %s)",
    ["alert.purge_green"] = "%s. PURGE VERT (%s: %s)",
    ["alert.purge_you"] = "TOI !!!",

    ["marker.north"] = "Nord",
    ["marker.south"] = "Sud",
    ["marker.east"] = "Est",
    ["marker.west"] = "Ouest",

    ["label.gun_grid"] = "Gun Grid",
    ["label.room_lines"] = "Colored Room Lines",
    ["label.yellow_room_boss"] = "Yellow Room: Boss",
    ["label.green_room_boss"] = "Green Room: Boss",
    ["label.blue_room_boss"] = "Blue Room: Boss",
    ["label.red_room_boss"] = "Red Room: Boss",
    ["label.shock_sphere"] = "Yellow Room: Shock Sphere",
    ["label.support_cannon"] = "Last Phase: Support Cannon",
    ["label.phase_switch"] = "Phase Switch",
    ["label.cardinal_markers"] = "Cardinal Markers",
    ["label.green_room_markers"] = "Green Room: Markers",
    ["label.hand_spawn"] = "Hand Spawn Point",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Avatus{
    label = "unit.avatus",
    unit{},
    line{color = "ff14eb14", thickness = 10},
    alert{label = "label.phase_switch"},
    cast{label = "alert.kill_avatus"},
    sound{label = "label.phase_switch", file = "info"},
  }

  GunGrid{
    label = "label.gun_grid",
    timer{color = "xkcdBlue"},
    line{color = "ff0000ff"},
    alert{},
    sound{file = "beware"},
  }
  ObliterationBeam{
    label = "cast.obliteration_beam",
    timer{color = "xkcdBloodRed"},
  }

  RoomLines{
    label = "label.room_lines",
    line{color = false},
  }

  YellowRoomBoss{
    label = "label.yellow_room_boss",
    color = "ffffff00",
    unit{},
    line{color = "ffff0000"},


    alert{label = "alert.blind"},
    timer{label = "message.next_blind", color = "xkcdBurntYellow"},
    cast{label = "alert.blind"},
    sound{label = "alert.blind", file = "inferno"},
  }
  GreenRoomBoss{
    label = "label.green_room_boss",
    color = "ff00ff00",
    unit{},
    text{label = "label.green_room_markers"},
    alert{label = "alert.yellow_room_health"}
  }
  BlueRoomBoss{
    label = "label.blue_room_boss",
    color = "ff0000ff",
    unit{},
  }
  RedRoomBoss{
    label = "label.red_room_boss",
    color = "ffff0000",
    unit{},
  }
  Hand{
    label = "unit.hand",
    unit{},
    alert{},
    sound{file = "info"},
    line{color = "ff0000ff"},
    timer{color = "xkcdOrangeyRed"},
  }
  HandSpawn{
    label = "label.hand_spawn",
    line{thickness = 5},
  }
  Cannon{
    label = "unit.holo_cannon",
    unit{},
    timer{color = "xkcdOrangeyRed"},
  }
  ShockSphere{
    label = "label.shock_sphere",
    line{color = "ff0000ff", thickness = 5},
  }
  SupportCannon{
    label = "label.support_cannon",
    unit{},
    timer{color = "xkcdBrightLilac"},
  }

  PurgeCycle{
    label = "message.next_purge_cycle",
    timer{color = "xkcdBluishGreen"},
    alert{color = false},
  }
  PurgeIncrease{
    label = "message.next_purge_increase",
    timer{color = "xkcdDeepOrange"},
  }

  Cardinal{
    label = "label.cardinal_markers",
    text{},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global
  _var.bShowHandIcons = false -- for self:RefresHandMarkers


  _var.strCurrentPhase = "MAIN_PHASE" --changed in :ApplyWorldMarkersByPhase(strPhase)
  UnitCreated{ strUnitName = "unit.avatus", bInCombat = true,
    Unit("Avatus", _arg.tUnit),
    Line("Avatus", _arg.tUnit, 22),
    {"ApplyWorldMarkersByPhase", "MAIN_PHASE"},
    {"RefresHandMarkers"},
  }

  Enable{
    Timer("GunGrid", 21)
  }

  UnitCreated{ strUnitName = "unit.rosdower",
    {"ApplyWorldMarkersByPhase", "LABYRINTH_PHASE"},
  }

  UnitCreated{ strUnitName = "unit.yellow_room",
    If{ [_arg.tUnit:GetHealth()] = {"~=", _arg(nil)}, --the room portals have the same name, but 'nil' Health
      {"ApplyWorldMarkersByPhase", "YELLOW_PHASE"},
      Unit("YellowRoomBoss", _arg.tUnit),
      Line("YellowRoomBoss", _arg.tUnit, 35),
      If{ [_arg.bInCombat] = true,
        SendCommMessage("MOBIUS_IN_COMBAT", _arg.tUnit:GetId())
      }
    }/{ --Else
      LineBetween("RoomLines", _arg.tUnit) % ("Yellow") + {color = "ffffff00"}
    }
  }
  UnitCreated{ strUnitName = "unit.green_room",
    If{ [_arg.tUnit:GetHealth()] = {"~=", _arg(nil)}, --the room portals have the same name, but 'nil' Health
      {"ApplyWorldMarkersByPhase", "GREEN_PHASE"},
      Unit("GreenRoomBoss", _arg.tUnit)
    }/{ --Else
      LineBetween("RoomLines", _arg.tUnit) % ("Green") + {color = "ff00ff00"}
    }
  }
  UnitCreated{ strUnitName = "unit.blue_room",
    If{ [_arg.tUnit:GetHealth()] = {"~=", _arg(nil)}, --the room portals have the same name, but 'nil' Health
      {"ApplyWorldMarkersByPhase", "BLUE_PHASE"},
      Unit("BlueRoomBoss", _arg.tUnit),
      {_env.GameLib.SetTargetUnit, _arg.tUnit}, --this ESURES, we see the Purge Debuffs.
      If{ [_arg.bInCombat] = true,
        {"DisplayPurgeList"}
      }
    }/{ --Else
      LineBetween("RoomLines", _arg.tUnit) % ("Blue") + {color = "ff0000ff"}
    }
  }
  UnitCreated{ strUnitName = "unit.red_room",
    If{ [_arg.tUnit:GetHealth()] = {"~=", _arg(nil)}, --the room portals have the same name, but 'nil' Health
      {"ApplyWorldMarkersByPhase", "RED_PHASE"},
      Unit("RedRoomBoss", _arg.tUnit)
    }/{ --Else
      LineBetween("RoomLines", _arg.tUnit) % ("Red") + {color = "ffff0000"}
    }
  }

  _var.tHands = _arg(function() return {} end)() --a new table each :Enable()
  UnitCreated{ strUnitName = "unit.hand",
    {"RefresHandMarkers", false},
    Unit("Hand", _arg.tUnit) % (_arg.nUnitId),
    Alert("Hand", "alert.hand_spawned"),
    Sound("Hand"),
    Line("Hand", _arg.tUnit, 25) % (_arg.nUnitId .. "Straight"),
    Line("Hand", _arg.tUnit,  7) % (_arg.nUnitId .. "Cleave1") + {color = "xkcdBluegrey", nRotation =  60},
    Line("Hand", _arg.tUnit,  7) % (_arg.nUnitId .. "Cleave2") + {color = "xkcdBluegrey", nRotation = -60},

    {function(self, nUnitId, tUnit)
      self.tVars.tHands[nUnitId] = tUnit
    end, _arg.self, _arg.nUnitId, _arg.tUnit}
  }
  UnitDestroyed{ strUnitName = "unit.hand",
    {function(self, nUnitId)
      self.tVars.tHands[nUnitId] = nil
    end, _arg.self, _arg.nUnitId}
  }

  UnitCreated{ strUnitName = "unit.holo_cannon",
    Line("GunGrid", _arg.tUnit, 100) % (_arg.nUnitId),

    If{ [_arg.tUnit:GetHealth()] = {"~=", _arg(nil)},
      Unit("Cannon", _arg.tUnit)
    }
  }

  UnitCreated{ strUnitName = "unit.shock_sphere",
    Line("ShockSphere", _arg.tUnit, -7),
  }

  _var.nLastSupportCannon = 0
  UnitCreated{ strUnitName = "unit.support_cannon", [_var.nLastSupportCannon] = {"<", _env.GameLib.GetGameTime()+20},
    _var.nLastSupportCannon(_env.GameLib.GetGameTime()),
    Unit("SupportCannon", _arg.tUnit),
    Timer("SupportCannon", 23, "message.next_support_cannon"),
  }

  UnitCreated{ strUnitName = "unit.hand_last_phase", -- we leave everything out, but the lines. As RaidCore does.
    Line("Hand", _arg.tUnit, 25) % (_arg.nUnitId .. "Straight"),
    Line("Hand", _arg.tUnit,  7) % (_arg.nUnitId .. "Cleave1") + {color = "xkcdBluegrey", nRotation =  60},
    Line("Hand", _arg.tUnit,  7) % (_arg.nUnitId .. "Cleave2") + {color = "xkcdBluegrey", nRotation = -60},
  }

  UnitDestroyed{ strUnitName = "unit.yellow_room",
    SendCommMessage("MOBIUS_DEATH"),
  }

  _var.bIsPurgeSync = false
  UnitDestroyed{ strUnitName = "unit.blue_room",
    -Timer("PurgeCycle"),
    -Timer("PurgeIncrease"),
    _var.bIsPurgeSync(false), --somehow leaving the room (or voidslip) invalidates the order of buffs / its not sync
  }

  local e = {
    PURGE_BLUE = 1,
    PURGE_RED = 2,
    PURGE_GREEN = 3,
  }
  local tPurgeBuffIcons = {
    ["Icon_ItemMisc_UI_Item_Crafting_Special_2"] = e.PURGE_GREEN,
    ["Icon_ItemMisc_UI_Item_Crafting_Special_3"] = e.PURGE_RED,
    ["Icon_ItemMisc_UI_Item_Crafting_PowerCore_Blue"] = e.PURGE_BLUE,
  }

  _var.nPurgeCount = 0 --a whole total (not sure why we keep this...)
  _var.nLastPurgeDrop = 0 --GameTime, when the last purge dropped (for determining, whether there is a new cycle)
  _var.nPurgeCycleCount = 0 --Purges this cycle
  _var.nPurgeCounters = _arg(function() return {
    [e.PURGE_GREEN] = 0, --the number of times this cycle, this color buff has been applied
    [e.PURGE_RED] = 0,
    [e.PURGE_BLUE] = 0,
  } end)()
  _var.nPurgeOffsets = _arg(function() return {
    [e.PURGE_GREEN] = 0, --the number of times this cycle, we skipped someone in the orders
    [e.PURGE_RED] = 0,
    [e.PURGE_BLUE] = 0,
  } end)()
  _var.arPurgeOrders = _arg(function() return {
    [e.PURGE_GREEN] = {}, --these are arrays of names.
    [e.PURGE_RED] = {},
    [e.PURGE_BLUE] = {},
  } end)()
  _var.tPurgeList = _arg(function() return {
    [e.PURGE_GREEN] = {}, --these are tables like ["strUnitName"] = GameLib.GetGameTime()
    [e.PURGE_RED] = {},
    [e.PURGE_BLUE] = {},
  } end)()
  BuffAdded{ strUnitName = "unit.blue_room",
    {function(self, tSpell)
      if not tSpell or not tSpell.splEffect then return end
      local strIcon = tSpell.splEffect:GetIcon()
      if not strIcon then return end
      local ePurgeVar = tPurgeBuffIcons[strIcon]
      if not ePurgeVar then return end

      self.tVars.nPurgeCount = self.tVars.nPurgeCount+1
      if self.tVars.nLastPurgeDrop + 1 <= GameLib.GetGameTime() then
        --we had 1 sec between the last buff dropped and a new applied: NEW CYCLE - WOOHOO
        --scince we are in a new cycle, we are in sync again.
        self.tVars.bIsPurgeSync = true
        --we are in a new cycle, so we reset everything to its 'default' state
        self.tVars.nPurgeCycleCount = 0
        self.tVars.nPurgeCounters[e.PURGE_GREEN] = 0
        self.tVars.nPurgeCounters[e.PURGE_RED] = 0
        self.tVars.nPurgeCounters[e.PURGE_BLUE] = 0
        self.tVars.nPurgeOffsets[e.PURGE_GREEN] = 0
        self.tVars.nPurgeOffsets[e.PURGE_RED] = 0
        self.tVars.nPurgeOffsets[e.PURGE_BLUE] = 0
        -- a new cycle started, so we also know when the upcoming starts:
        self:Timer("PurgeCycle", 20)
        -- there is a new Cycle, so we display how it should be handled
        self:DisplayPurgeList()
      end

      do --Okay - a new Purgebuff appeared. So we advanced in our Purgecycle.
        self.tVars.nPurgeCycleCount = self.tVars.nPurgeCycleCount + 1
        self.tVars.nPurgeCounters[ePurgeVar] = self.tVars.nPurgeCounters[ePurgeVar] + 1
      end

      do --Okay - who should purge this debuff tho? (this part should be in line with RaidCore)
        -- we will display 'nPurgeCycleCount'. PURGE 'strColor' ('PURGE_color_count': 'strUnitPurge')
        -- alternatives when error/not sync: nPurgeCycleCount = nPurgeCount;

        local nCount = nPurgeCount; --the here initialized variables are the defaults,
        local nColorIndex = "?"; -- if for some reason no value can be displayed
        local strUnitPurge = "N/A"; -- or the info is highly misleading (out of sync)
        local bIsPlayer = false;

        if self.tVars.bIsPurgeSync then
          nCount = self.tVars.nPurgeCycleCount; --scince we are sync, we know exactly which #Purge we are on.
          nColorIndex = self.tVars.nPurgeCounters[ePurgeVar]; -- how many times this color appeared already.

          --now, we know the index of the guy who should purge - find him, validate him.
          --if the guy himself is not available, increment the offset to the next in order.
          local nOffset = self.tVars.nPurgeOffsets[ePurgeVar]
          repeat
            local nGuyIndex = nColorIndex + nOffset

            local strPurgeGuy = self.tVars.arPurgeOrders[ePurgeVar][nGuyIndex]
            local tPurgeGuy = GameLib.GetPlayerUnitByName(strPurgeGuy)

            if tPurgeGuy and tPurgeGuy:IsValid() and not tPurgeGuy:IsDead() or false then
              bIsPlayer = tPurgeGuy:IsThePlayer()
              strUnitPurge = bIsThePlayer and self.L["alert.purge_you"] or strPurgeGuy
              self.tVars.nPurgeOffsets[ePurgeVar] = nOffset
              break;
            end
            nOffset = nOffset+1
          until(not strPurgeGuy)
        end


        local strMsg;
        local strCol;
        if ePurgeVar == e.PURGE_GREEN then
          strMsg = self.L["alert.purge_green"]:format(nCount, nColorIndex, strUnitPurge)
          strCol = "ff00ff00"
        elseif ePurgeVar == e.PURGE_RED then
          strMsg = self.L["alert.purge_red"]:format(nCount, nColorIndex, strUnitPurge)
          strCol = "ffff0000"
        elseif ePurgeVar == e.PURGE_BLUE then
          strMsg = self.L["alert.purge_blue"]:format(nCount, nColorIndex, strUnitPurge)
          strCol = "ff0000ff"
        end

        if bIsPlayer then
          self:Sound("PurgeCycle")
        end
        if strMsg then
          self:Alert("PurgeCycle", strMsg, nil, {color = strCol})
        end
      end
    end, _arg.self, _arg.tData.tSpell}
  }
  BuffRemoved{ strUnitName = "unit.blue_room",
    If{ [_arg(tPurgeBuffIcons)[_arg.tData.tSpell.splEffect:GetIcon()]] = {"~=", _arg(nil)},
      _var.nLastPurgeDrop(_env.GameLib.GetGameTime())
    }
  }

  local BUFFID_HOLO_CANNONS_ACTIVE = 44756
  BuffAdded{ strUnitName = "unit.avatus", nSpellId = BUFFID_HOLO_CANNONS_ACTIVE,
    Timer("ObliterationBeam", 26),
  }

  local BUFFID_PROTECTIVE_BARRIER = 45304
  BuffAdded{ strUnitName = "unit.avatus", nSpellId = BUFFID_PROTECTIVE_BARRIER,
    -Timer("GunGrid"),
    -Timer("ObliterationBeam"),
    -Timer("Hand"),
    -Timer("Cannon"),
    {"RefresHandMarkers", false},
  }


  CastStart{ strUnitName = "unit.avatus", strSpellName = "MOO",
    -Timer("SupportCannon"),
    Alert("Avatus", "alert.kill_avatus"),
    Cast("Avatus", _arg.tCast),
    Sound("Avatus"),
  }


  HealthReached{ strUnitName = "unit.avatus", nHealthReached = {78, 53, 28},
    nHealthPercent = {">", _arg.nHealthReached-3},
    Sound("Avatus"),

    If{ [_arg.nHealthReached] = {">", 50},
      Alert("Avatus", "alert.phase2_soon")
    }/{ --only on 25%
      Alert("Avatus", "alert.phase3_soon")
    }
  }

  HealthReached{ strUnitName = "unit.yellow_room", nHealthReached = {90, 80, 70, 60, 50, 40, 30, 20, 10},
    nHealthPercent = {">", _arg.nHealthReached-3}, --we want to have this be quite accurate (in 3% error).
    SendCommMessage("MOBIUS_HEALTH_UPDATE", _arg.nHealthReached),
  }

  CastStart{ strSpellName = "cast.obliteration_beam",
    Timer("ObliterationBeam", 37),
  }

  CastStart{ strSpellName = "cast.hand_crush",
    {function(self, nHandId, tHands)
      local bClosest = false
      local nDistanceClose = 35 --want the hand in at least 35m range.
      for nUnitId, tUnit in pairs(tHands) do
        local nDistance = self:GetDistance(tUnit)
        if nDistance < nDistanceClose then
          nDistanceClose = nDistance
          bClosest = nUnitId == nHandId
        end
      end

      if bClosest then
        self:Alert("Hand", "alert.interrupt_hand_crush")
        self:Sound("Hand")
      end
    end, _arg.self, _arg.nUnitId, _var.tHands}
  }

  CastStart{ strSpellName = "cast.yellow_room_blind",
    Alert("YellowRoomBoss", "alert.blind"),
    Timer("YellowRoomBoss", 29),
    Cast("YellowRoomBoss", _arg.tCast),
    Sound("YellowRoomBoss"),
  }

  _var.bLastMainPhase = false
  _var.bUpcomingIsHandPhase = true
  HealthReached{ strUnitName = "unit.avatus", nHealthReached = 50,
    _var.bLastMainPhase(true),
  }
  Datachron{ strMessage = "datachron.gun_grid",
    If{ [_var.bLastMainPhase] = true,
      Timer("GunGrid", 81)
    }/{
      Timer("GunGrid", 112)
    },
    Alert("GunGrid", "alert.gun_grid"),
    Sound("GunGrid"),

    If{ [_var.bUpcomingIsHandPhase] = true,
      Timer("Hand", 22, "message.next_hands"),
      _var.bUpcomingIsHandPhase(false)
    }/{
      Timer("Cannon", 22, "message.next_cannons"),
      _var.bUpcomingIsHandPhase(true)
    }
  }

  Datachron{ strMessage = "datachron.portals_opened",
    -Timer("GunGrid"),
    -Timer("ObliterationBeam"),
    -Timer("Hand"),
    -Timer("Cannon"),
  }

  Datachron{ strMessage = "datachron.blue_room_purge_increase",
    Timer("PurgeIncrease", 30)
  }

  local tPurgeAbilityIcons = {
    ["Icon_SkillEnergy_UI_srcr_shckcntrp"] = e.PURGE_BLUE,
    ["Icon_SkillFire_UI_ss_srngblst"] = e.PURGE_RED,
    ["Icon_SkillEnergy_UI_srcr_surgeengine"] = e.PURGE_GREEN,
  }
  Callback("OnPurgeAbilityIcon",{ [_arg(tPurgeAbilityIcons)[_arg.strIcon]] = {"~=", _arg.Nil},
    {"RegisterPurgeType", _env.GameLib.GetPlayerCharacterName(), _arg(tPurgeAbilityIcons)[_arg.strIcon]},
    SendCommMessage("PURGE_TYPE", _arg(tPurgeAbilityIcons)[_arg.strIcon]),
  })

  _var.nYellowBossHealth = 101
  CommMessage{ strReason = "MOBIUS_IN_COMBAT",
    [_var.strCurrentPhase] = {"LABYRINTH_PHASE", "GREEN_PHASE"},
    [_var.nYellowBossHealth] = 101,
    --strUnitName, strReason, tData,
    _var.nYellowBossHealth(100),
    Alert("GreenRoomBoss", "alert.yellow_room_start"),
  }

  CommMessage{ strReason = "MOBIUS_HEALTH_UPDATE",
    [_var.strCurrentPhase] = {"LABYRINTH_PHASE", "GREEN_PHASE"},
    If{ tData = {"<", _var.nYellowBossHealth},
      --strUnitName, strReason, tData,
      _var.nYellowBossHealth(_arg.tData),
      Alert("GreenRoomBoss", _arg.self.L["alert.yellow_room_health"]:format(_arg.tData)),
    }
  }

  CommMessage{ strReason = "MOBIUS_DEATH",
    [_var.strCurrentPhase] = {"LABYRINTH_PHASE", "GREEN_PHASE"},
    [_var.nYellowBossHealth] = {">", 0},
    --strUnitName, strReason, tData,
    _var.nYellowBossHealth(0),
    Alert("GreenRoomBoss", _arg.self.L["alert.yellow_room_health"]:format(0)),
  }

  CommMessage{ strReason = "PURGE_TYPE",
    {"RegisterPurgeType", _arg.strUnitName, _arg.tData},
  }
end)

local CARDINAL_MARKERS = {
  ["marker.north"] = { x = 618, y = -198, z = -235 },
  ["marker.south"] = { x = 618, y = -198, z = -114 },
  ["marker.east"] = { x = 678, y = -198, z = -174 },
  ["marker.west"] = { x = 557, y = -198, z = -174 },
}
local GREEN_ROOM_MARKERS = {
  ["1"] = { y = -198, x = 557.58, z = -139.2 },
  ["2"] = { y = -198, x = 583.21, z = -113.5 },
  ["3"] = { y = -198, x = 618.21, z = -104.2 },
  ["4"] = { y = -198, x = 653.21, z = -113.5 },
  ["5"] = { y = -198, x = 678.83, z = -139.2 },
  ["6"] = { y = -198, x = 688.21, z = -174.2 },
  ["7"] = { y = -198, x = 678.83, z = -209.2 },
  ["8"] = { y = -198, x = 653.21, z = -234.8 },
  ["9"] = { y = -198, x = 618.21, z = -244.2 },
  ["10"] = { y = -198, x = 583.21, z = -234.8 },
  ["11"] = { y = -198, x = 557.58, z = -209.2 },
}
function Mod:ApplyWorldMarkersByPhase(strPhase)
  self.tVars.strCurrentPhase = strPhase

  if strPhase == "MAIN_PHASE" or strPhase == "YELLOW_PHASE" or strPhase == "LABYRINTH_PHASE" then
    for strText, tPos in pairs(CARDINAL_MARKERS) do
      self:Text("Cardinal", tPos, strText, strText)
    end
  else
    for strText in pairs(CARDINAL_MARKERS) do
      self:RemoveText("Cardinal", strText)
    end
  end

  if strPhase == "GREEN_PHASE" then
    for strText, tPos in pairs(GREEN_ROOM_MARKERS) do
      self:Text("GreenRoomBoss", tPos, strText, strText)
    end
  else
    for strText in pairs(GREEN_ROOM_MARKERS) do
      self:RemoveText("GreenRoomBoss", strText)
    end
  end
end

function Mod:RefresHandMarkers(bShow)
  if bShow ~= nil then --if its supplied, apply as 'standard'
    self.tVars.bShowHandIcons = bShow
  else  --if its not supplied: retrieve the standard
    bShow = self.tVars.bShowHandIcons
  end

  if bShow then
    local tBoss = self:GetUnit("Avatus")

    local nCos = math.cos(1/6*math.pi*2)
    local nSin = math.sin(1/6*math.pi*2)

    self:Polygon("HandSpawn", tBoss, 2, 6, "A", {
      tVectorOffsets = {
        x = -nSin * 17,
        y = 0,
        z = nCos * 17,
      }
    })
    self:Polygon("HandSpawn", tBoss, 2, 6, "B", {
      tVectorOffsets = {
        x = nSin * 17,
        y = 0,
        z = nCos * 17,
      }
    })
  else
    self:RemovePolygon("HandSpawn", "A")
    self:RemovePolygon("HandSpawn", "B")
  end
end

local e = {
  PURGE_BLUE = 1,
  PURGE_RED = 2,
  PURGE_GREEN = 3,
}
function Mod:DisplayPurgeList()
  self:Print("====== PURGE LIST ======")
  for ePurgeType, arOrdered in pairs(self.tVars.arPurgeOrders) do
    local strPlayers = table.concat(arOrdered, ", ")
    local strNum = "#"..#arOrdered
    if ePurgeType == e.PURGE_BLUE then
      self:Print(self.L["alert.purge_blue"]:format("B", strNum, strPlayers))
    elseif ePurgeType == e.PURGE_RED then
      self:Print(self.L["alert.purge_red"]:format("R", strNum, strPlayers))
    elseif ePurgeType == e.PURGE_GREEN then
      self:Print(self.L["alert.purge_green"]:format("G", strNum, strPlayers))
    end
  end
end

function Mod:EngageActionBarTracker()
  local xmlDoc = XmlDoc.CreateFromTable({ __XmlNode = "Forms",
    { __XmlNode = "Form",
      Name="ShortcutItem", Class="Window", Template="Default", Visible="0",
      LAnchorPoint="0", TAnchorPoint="0", RAnchorPoint="0", BAnchorPoint="0",
      LAnchorOffset="0", TAnchorOffset="1", RAnchorOffset="1", BAnchorOffset="2",
      TooltipColor="", Tooltip="", IgnoreTooltipDelay="0", DoNotBlockTooltip="1", TooltipType="OnCursor",
      Font="Default", Text="", TextColor="UI_WindowTextDefault",
      BGColor="UI_WindowBGDefault", Picture="0", Border="0", Sprite="", RightEdgeControlsAnchor="",
      RelativeToClient="1", SwallowMouseClicks="1", Moveable="0", Escapable="0", Overlapped="1", IgnoreMouse="1",
      {__XmlNode = "Control",
        Name="ShortcutBtn", Class="ActionBarButton", WindowSoundTemplate="ActionBarButton", ContentType="SBar",
        LAnchorPoint="0", TAnchorPoint="0", RAnchorPoint="1", BAnchorPoint="1",
        LAnchorOffset="0", TAnchorOffset="0", RAnchorOffset="0", BAnchorOffset="0",
        RelativeToClient="1", NeverBringToFront="1", IgnoreMouse="0", IfHoldNoSignal="1",
        Picture="0", BGColor="white", ContentId="0", DrawShortcutBottom="1", TopEdgeControlsAnchor="",
        Base="CRB_DEMO_WrapperSprites:btnDemo_CharInvisible",
        TextColor="white", DT_CENTER="1", DT_VCENTER="1",
        TooltipType="OnCursor", TooltipColor="", IgnoreTooltipDelay="1",
      }
    }
  })

  self.wndBtn = Apollo.LoadForm(xmlDoc, "ShortcutItem", "FixedHudStratum", nil):FindChild("ShortcutBtn")
end

function Mod:DisengageActionBarTracker()
  if self.wndBtn then self.wndBtn:Destroy() end
  self.wndBtn = nil
end

function Mod:OnShowActionBarShortcut(eWhichBar, _, _)
  if eWhichBar == ActionSetLib.CodeEnumShortcutSet.FloatingSpellBar then
    -- The GetContent function is not ready... A delay must be added.
    --this delay is automatically stopped, if the mod is disabled, overwritten (by itself) or anything else.
    self:Delay("OnShowActionBarShortcutDelay", 1, {"OnShowActionBarShortcutDelay"})
  end
end

function Mod:OnShowActionBarShortcutDelay()
  local eWhichBar = ActionSetLib.CodeEnumShortcutSet.FloatingSpellBar

  self.wndBtn:SetContentId(eWhichBar * 12 + 0) --this is the first button in the row.
  local tButtonContent = self.wndBtn:GetContent()

  if tButtonContent and tButtonContent.strIcon ~= "" then
    self:FireCallbacks("OnPurgeAbilityIcon", {
      strIcon = tButtonContent.strIcon
    })
  end
end

Mod:RegisterCallback("Enable", Mod.EngageActionBarTracker)
Mod:RegisterCallback("Disable", Mod.DisengageActionBarTracker)
Mod:AutoRegisterEvent("ShowActionBarShortcut", "OnShowActionBarShortcut")

--this is basically copied from RaidCore. (just some variables renamed)
local function BuildBlueRoomPurgeOrderedList(tBlueRoomPurgeList, tBlueRoomPurgeOrderedList)
  for ePurgeType in pairs(tBlueRoomPurgeList) do
    if next(tBlueRoomPurgeList[ePurgeType]) then
      local tCopy = {}
      for k, v in next, tBlueRoomPurgeList[ePurgeType] do
        tCopy[k] = v
      end

      local tOrdered = {}
      while next(tCopy) do
        local f = nil
        for key, val in next, tCopy do
          if f == nil or tCopy[f] > val then
            f = key
          end
        end
        table.insert(tOrdered, f:gmatch("%a+")())
        tCopy[f] = nil
      end
      tBlueRoomPurgeOrderedList[ePurgeType] = tOrdered
    end
  end
end
function Mod:RegisterPurgeType(strName, eType)
  if eType == nil then return end
  self.tVars.tPurgeList[eType][strName] = GameLib.GetGameTime()
  BuildBlueRoomPurgeOrderedList(self.tVars.tPurgeList, self.tVars.arPurgeOrders)
end