local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("Gloomclaw", {
  instance = "Datascape",
  displayName = "Gloomclaw",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 52,
        parentZoneId = 98,
        mapId = 115,
      },
    },
  },
},{
  ["enUS"] = {
    -- Units
    ["unit.boss"] = "Gloomclaw",
    ["unit.essence"] = "Essence of Logic",
    ["unit.add1"] = "Strain Parasite",
    ["unit.add2"] = "Gloomclaw Skurge",
    ["unit.add3"] = "Corrupted Fraz",
    -- Casts
    ["cast.rupture"] = "Rupture",
    ["cast.burrow"] = "Burrow",
    -- Datachron
    ["datachron.step_back"] = "Gloomclaw is pushed back by the purification of the essences!",
    ["datachron.step_forward"] = "Gloomclaw is moving forward to corrupt more essences!",
    -- Alerts
    ["alert.interrupt_rupture"] = "Interrupt Rupture!",
    -- Labels
    ["label.moo"] = "Moment of Opportunity",
    ["label.essence_left"] = "Left Essence",
    ["label.essence_right"] = "Right Essence",
    ["label.next_rupture"] = "Next Rupture",
    ["label.next_adds"] = "Next Wave",
  },
  ["deDE"] = {
    -- Units
    ["unit.boss"] = "Düsterklaue",
    ["unit.essence"] = "Logikessenz",
    ["unit.add1"] = "Transmutierten-Parasit",
    ["unit.add2"] = "Düsterklauen-Geißel",
    ["unit.add3"] = "Korrumpierter Fraz",
    -- Casts
    ["cast.rupture"] = "Aufreißen",
    ["cast.burrow"] = "Bau",
    -- Datachron
    ["datachron.step_back"] = "Düsterklaue wird durch die Reinigung der Essenzen zurückgestoßen!",
    ["datachron.step_forward"] = "???",
    -- Alerts
    ["alert.interrupt_rupture"] = "Boss Unterbrechen!",
    -- Labels
    ["label.moo"] = "Moment of Opportunity",
    ["label.essence_left"] = "Linke Essenz",
    ["label.essence_right"] = "Rechte Essenz",
    ["label.next_rupture"] = "Nächstes Aufreißen",
    ["label.next_adds"] = "Nächste Welle",
  },
  ["frFR"] = {
    -- Units
    ["unit.boss"] = "Serrenox", --V (Verified)
    ["unit.essence"] = "Essence de logique", --V
    ["unit.add1"] = "Parasite de la Souillure", --V
    ["unit.add2"] = "Skurge serrenox", --V
    ["unit.add3"] = "Friz corrompu", --V
    -- Casts
    ["cast.rupture"] = "Rupture", --V
    ["cast.burrow"] = "Terrier", --V
    -- Datachron
    ["datachron.step_back"] = "Serrenox est repoussé par la purification des essences !",
    ["datachron.step_forward"] = "Serrenox s'approche pour corrompre davantage d'essences !",
    -- Alerts
    ["alert.interrupt_rupture"] = "Interrompre Rupture !",
    -- Labels
    ["label.moo"] = "Moment d'opportunité",
    ["label.essence_left"] = "Essence de gauche",
    ["label.essence_right"] = "Essence de droite",
    ["label.next_rupture"] = "Prochaine rupture",
    ["label.next_adds"] = "Prochaine vague",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    cast{label = "label.moo"},
  }

  EssenceLeft{
    label = "label.essence_left",
    unit{},
  }
  EssenceRight{
    label = "label.essence_right",
    unit{},
  }

  Rupture{
    label = "cast.rupture",
    color = "ffffa500",
    timer{},
    cast{},
    alert{},
    sound{file="alert"},
  }
  Adds{
    label = "label.next_adds",
    timer{},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global
  local nCenterX = 4310 --X Coordinate of Room center (to determine L/R Essences)
  local nSouthZ = -16612 --Z Coordinate of southmost point in room (to determine current section)
  local nRoomStep = 84 --Sections are ~84m distant from each other.
  local tFirstRuptures = { -- The timers for first rupture of each section. No value for section 4.
    [0]=30, [1]=32, [2]=28, [3]=26, [5]=30
  }
  local tAddSpawnTimers = { -- Static times for add-timers for each section. 4 not needed, 5 handled seperatly.
     [0]=25, [1]=26, [2]=33, [3]=25
  }
  local tAddSpawnTotal = { -- The amount of add-spawns of each section. 4 not needed, 5 handled seperatly.
    [0]=4,[1]=4, [2]=3, [3]=4, [4]=0
  }

  local function fSection(tUnit)
    local tPos = tUnit:GetPosition()
    return math.floor((nSouthZ-tPos.z)/nRoomStep)
  end
  local function fSide(tUnit)
    local tPos = tUnit:GetPosition()
    return tPos.x<nCenterX and "Left" or "Right"
  end

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
  }

  UnitCreated{ strUnitName = "unit.essence",
    SetCount(_arg.nUnitId, "Essence", _arg(fSection)(_arg.tUnit), _arg(fSide)(_arg.tUnit))
    --basically equivalent to something like Essence[nSection][strSide] = nUnitId
  }

  local function fUpdateEssenceUnits(self)
    local nSection = fSection(self:GetUnit("Boss"))

    local nEssenceL = self:GetCount("Essence", nSection, "Left")
    local nEssenceR = self:GetCount("Essence", nSection, "Right")

    self:RemoveUnit("EssenceLeft")
    self:RemoveUnit("EssenceRight")
    if nEssenceL>0 then --GetCount Defaults to 0.
      self:Unit("EssenceLeft", nEssenceL, nil, {strName = "label.essence_left"})
    end
    if nEssenceR>0 then --GetCount Defaults to 0.
      self:Unit("EssenceRight", nEssenceR, nil, {strName = "label.essence_right"})
    end
  end


  _var.nLastAdds = 0
  Datachron{ strMessage = {"datachron.step_back", "datachron.step_forward"},
    SetCount(0, "Wave")
  }
  UnitCreated{ strUnitName = {"unit.add1", "unit.add2", "unit.add3"}, function(self)
    --we do this in a function, because its just way too much back and forth...
    local nTime = GameLib.GetGameTime()
    if nTime - self.tVars.nLastAdds > 10 then
      self.tVars.nLastAdds = nTime
      local nWave = self:Count("Wave") --WaveCount = WaveCount+1
      local nSection = fSection(self:GetUnit("Boss"))

      if nSection < 5 then
        if nWave < tAddSpawnTotal[nSection] then
          self:Timer("Adds", tAddSpawnTimers[nSection])
        end
      else --nSection == 5
        if nWave == 1 then
          self:Timer("Adds", 20.5)
        elseif nWave == 2 then
          self:Timer("Adds", 30)
        elseif nWave == 3 then
          self:Timer("Adds", 15)
        end
      end
    end
  end}

  CastStart{ strSpellName = "cast.rupture",
    Sound("Rupture"),
    Cast("Rupture", _arg.tCast),
    Alert("Rupture", "alert.interrupt_rupture"),
    Timer("Rupture", 43, "label.next_rupture"),
  }
  CastStart{ strSpellName = "MOO",
    Cast("Boss", _arg.tCast),
    -Timer("Rupture"),
    -Timer("Adds"),
  }
  CastEnd{ strSpellName = "cast.burrow",
    fUpdateEssenceUnits,
    Timer("Rupture", _arg(tFirstRuptures)[_arg(fSection)(_arg.tUnit)], "label.next_rupture"),
  }
end)