local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("LimboInfomatrix", {
  instance = "Datascape",
  displayName = "Limbo Infomatrix",
  tTrigger = {
    tZones = {
      [1] = {
        continentId = 52,
        parentZoneId = 98,
        mapId = 114,
      },
    },
  },
},{
  ["enUS"] = {
    ["unit.antlion"] = "Infomatrix Antlion",
    ["unit.keeper"] = "Keeper of Sands",
    ["unit.circling"] = "16623 Hostile Invisible Unit for Fields (0 hit radius) (Very Fast Move Updates) - BEAX",
    -- Cast.
    ["cast.knockback"] = "Exhaust",
    ["cast.interrupt"] = "Desiccate",
    -- Bar and messages.
    ["alert.knockback"] = "WARNING: KNOCK-BACK",

    ["label.miniboss"] = "Miniboss",
    ["label.antlion"] = "Antlion",
    ["label.interrupt"] = "Interrupt",
    ["label.knockback"] = "Knockback",
    ["label.circling"] = "Circling Telegraph",
  },
  ["deDE"] = {
    ["unit.antlion"] = "Ameisenlöwe der Infomatrix",
    ["unit.keeper"] = "Bewahrer des Sandes",
    ["unit.circling"] = "16623 Hostile Invisible Unit for Fields (0 hit radius) (Very Fast Move Updates) - BEAX",
    -- Cast.
    ["cast.knockback"] = "Abgas",
    ["cast.interrupt"] = "Austrocknen",
    -- Bar and messages.
    ["alert.knockback"] = "ACHTUNG: KNOCK-BACK",

    ["label.miniboss"] = "Miniboss",
    ["label.antlion"] = "Ameisenlöwe",
    ["label.interrupt"] = "Unterbrechen",
    ["label.knockback"] = "Knockback",
    ["label.circling"] = "Kreisender Telegraph",
  },
  ["frFR"] = {
    ["unit.antlion"] = "Fourmilion de l'Infomatrice", --V (Verified)
    ["unit.keeper"] = "Gardien des sables", --Raidcore (as all below)
    ["unit.circling"] = "16623 Hostile Invisible Unit for Fields (0 hit radius) (Very Fast Move Updates) - BEAX",
    -- Cast.
    ["cast.knockback"] = "Épuiser",
    ["cast.interrupt"] = "Dessécher",
    -- Bar and messages.
    ["alert.knockback"] = "ATTENTION: KNOCK-BACK",

    ["label.miniboss"] = "Miniboss",
    ["label.antlion"] = "Antlion",
    ["label.interrupt"] = "Interrupt",
    ["label.knockback"] = "Knockback",
    ["label.circling"] = "Circling Telegraph",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Keeper{
    label = "label.miniboss",
    unit{},
    line{thickness = 6, color = "ffff0000"},
  }
  Antlion{
    label = "label.antlion",
    unit{},
    icon{sprite = "LUIBM_crosshair", size = 60, color = "ff00ffff"},
  }
  Interrupt{
    label = "label.interrupt",
    sound{file = "alert"}
  }
  Knockback{
    label = "label.knockback",
    alert{},
    sound{file = "info"}
  }
  Circling{
    label = "label.circling",
    line{thickness = 3, color = false},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  _var.tKeeperPos = nil
  _var.bKeeperCombat = false
  local function fUpdateKeeperLine(self)
    if not self.tVars.bKeeperCombat and self.tVars.tKeeperPos then
      --min length of 60m
      self:LineBetween("Keeper", self.tVars.tKeeperPos, nil, nil, {min = 60})
    else
      self:RemoveLineBetween("Keeper")
    end
  end

  UnitCreated{ strUnitName = "unit.keeper",
    Unit("Keeper", _arg.tUnit),
    _var.tKeeperPos(_arg.tUnit:GetPosition()),
    _var.bKeeperCombat(_arg.bInCombat),
    fUpdateKeeperLine,
  }

  CastStart{ strSpellName = "cast.knockback", [InRange(_arg.tUnit, 40)] = true,
    Sound("Knockback"),
    Alert("Knockback", "alert.knockback"),
  }

  CastStart{ strSpellName = "cast.interrupt", [InRange(_arg.tUnit, 40)] = true,
    Sound("Interrupt"),
  }

  UnitCreated{ strUnitName = "unit.antlion",
    If{ [_arg.bInCombat] = true,
      Unit("Antlion", _arg.tUnit) % (_arg.nUnitId),
      -Icon("Antlion") % (_arg.nUnitId)
    }/{
      Icon("Antlion", _arg.tUnit) % (_arg.nUnitId)
    }
  }

  UnitCreated{ strUnitName = "unit.circling",
    --at its Position, so its not constantly spinning..
    Polygon("Circling", _arg.tUnit:GetPosition(), 6, 16) % (_arg.nUnitId .. "Inner") + {
      color = "xkcdBrightPurple",
    },
    Polygon("Circling", _arg.tUnit:GetPosition(),15, 16) % (_arg.nUnitId .. "Outer") + {
      color = "xkcdBluishPurple",
    },
  }
  UnitDestroyed{ strUnitName = "unit.circling",
    -Polygon("Circling") % (_arg.nUnitId .. "Inner"),
    -Polygon("Circling") % (_arg.nUnitId .. "Outer"),
  }


  --Communication with RaidCore / other versions of this Addon
  UnitCreated{ strUnitName = "unit.keeper",
    SendCommMessage("KeeperOfSands_Position", _arg.tUnit:GetPosition()),
    SendCommMessage("KeeperOfSands_bInCombat", _arg.bInCombat),
  }
  CommMessage{ strReason = "KeeperOfSands_Position",
    _var.tKeeperPos(_arg.tData),
    fUpdateKeeperLine,
  }
  CommMessage{ strReason = "KeeperOfSands_bInCombat",
    _var.bKeeperCombat(_arg.tData),
    fUpdateKeeperLine,
  }
end)