local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("MaelstromAuthority", {
  instance = "Datascape",
  displayName = "Maelstrom Authority",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 52,
        parentZoneId = 98,
        mapId = 120,
      },
    },
  },
},{
  ["enUS"] = {
    -- Units
    ["unit.boss"] = "Maelstrom Authority",
    ["unit.station"] = "Weather Station",
    ["unit.wind_wall"] = "Wind Wall",
    -- Casts
    ["cast.wather_cycle"] = "Activate Weather Cycle",
    ["cast.ice_breath"] = "Ice Breath",
    ["cast.crystallize"] = "Crystallize",
    ["cast.typhoon"] = "Typhoon",
    ["cast.shatter"] = "Shatter",
    -- Datachron
    ["datachron.tremble"] = "The platform trembles",
    -- Alert
    ["alert.station"] = "Station: %s",
    ["alert.ice_breath"] = "Ice Breath!",
    ["alert.crystallize"] = "Crystallize!",
    ["alert.typhoon"] = "Typhoon!",
    ["alert.tremble"] = "Jump!",
    --Labels
    ["label.NW"] = "NW",
    ["label.NE"] = "NE",
    ["label.SW"] = "SW",
    ["label.SE"] = "SE",
  },
  ["deDE"] = {
    -- Units
    ["unit.boss"] = "Mahlstromgewalt",
    ["unit.station"] = "Wetterstation",
    ["unit.wind_wall"] = "Windwand",
    -- Casts
    ["cast.weather_cycle"] = "Wetterzyklus aktivieren",
    ["cast.ice_breath"] = "Eisatem",
    ["cast.crystallize"] = "Kristallisieren",
    ["cast.typhoon"] = "Taifun",
    ["cast.shatter"] = "Zerschmettern",
    -- Datachron
    ["datachron.tremble"] = "Die Plattform bebt!",
    -- Alert
    ["alert.station"] = "Station: %s",
    ["alert.ice_breath"] = "Eisatem!",
    ["alert.crystallize"] = "Kristallisieren!",
    ["alert.typhoon"] = "Taifun!",
    ["alert.tremble"] = "Springen!",
    --Labels
    ["label.NW"] = "NW",
    ["label.NE"] = "NO",
    ["label.SW"] = "SW",
    ["label.SE"] = "SO",
  },
  ["frFR"] = {
    -- Units
    ["unit.boss"] = "Contrôleur du Maelstrom", --V (Verified)
    ["unit.station"] = "Station météorologique", --V
    ["unit.wind_wall"] = "Mur de vent",
    -- Casts
    ["cast.weather_cycle"] = "Activer cycle climatique", --V
    ["cast.ice_breath"] = "Souffle de glace",
    ["cast.crystallize"] = "Cristalliser",
    ["cast.typhoon"] = "Typhon",
    ["cast.shatter"] = "Fracasser", --V
    -- Datachron
    ["datachron.tremble"] = "La plateforme tremble !",
    -- Alert
    ["alert.station"] = "Station: %s",
    ["alert.ice_breath"] = "Souffle de glace !",
    ["alert.crystallize"] = "Cristalliser !",
    ["alert.typhoon"] = "Typhon !",
    ["alert.tremble"] = "Jump!",
    --Labels
    ["label.NW"] = "NW",
    ["label.NE"] = "NE",
    ["label.SW"] = "SW",
    ["label.SE"] = "SE",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    line{color = "ffff0000", thickness = 10},
  }

  Station{
    label = "unit.station",
    color = "xkcdOrangeYellow",
    unit{},
    line{thickness = 5},
    timer{},
    sound{file= "info"},
    alert{},
  }

  Tremble{
    label = "alert.tremble",
    color = "xkcdSunYellow",
    cast{},
    alert{},
    sound{file = "alert"},
    timer{sound = true},
  }

  IceBreath{
    label = "alert.ice_breath",
    alert{},
    sound{file = "run-away"},
  }
  Crystallize{
    label = "alert.crystallize",
    alert{},
    sound{file = "beware"},
  }
  Typhoon{
    label = "alert.typhoon",
    alert{},
    sound{file = "beware"},
  }
  WindWall{
    label = "unit.wind_wall",
    line{color = "ff00ff00"}
  }

end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
    Line("Boss", _arg.tUnit, 20),
  }

  local function fStationCardinal(self, tUnit)
    local tBoss = self:GetUnit("Boss")
    local tBossPos = tBoss:GetPosition()
    local tUnitPos = tUnit:GetPosition()

    local strLabel = "label."
    strLabel = strLabel.. (tUnitPos.z > tBossPos.z and "S" or "N")
    strLabel = strLabel.. (tUnitPos.x > tBossPos.x and "E" or "W")

    return self.L[strLabel]
  end
  local strStationMark = _arg(fStationCardinal)(_arg.self, _arg.tUnit)

  UnitCreated{ strUnitName = "unit.station",
    Unit("Station", _arg.tUnit) % (_arg.nUnitId) + {strMark = strStationMark},
    LineBetween("Station", _arg.tUnit, nil) % (_arg.nUnitId),

    --Raidcore does these things, only after they're in combat - why?
    Timer("Station", 25),
    Alert("Station", {"alert.station", strStationMark}),
    Sound("Station"),
  }

  UnitCreated{ strUnitName = "unit.wind_wall",
    --Raidcore uses 2 lines: 1) 20m to front, 1) 20m to back.
    --We instead do a line from -20m (behind) of 40m length.
    Line("WindWall", _arg.tUnit, 40) % (_arg.nUnitId) + {nOffset = -20},
  }

  CastStart{ strSpellName = "cast.weather_cycle",
    Timer("Station", 3),
  }

  CastStart{ strSpellName = "cast.ice_breath",
    Alert("IceBreath", "alert.ice_breath"),
    Sound("IceBreath"),
  }
  CastStart{ strSpellName = "cast.crystallize",
    Alert("Crystallize", "alert.crystallize"),
    Sound("Crystallize"),
  }
  CastStart{ strSpellName = "cast.typhoon",
    Alert("Typhoon", "alert.typhoon"),
    Sound("Typhoon"),
  }

  Datachron{ strMessage = "datachron.tremble",
    -Timer("Station"),

    Cast("Tremble", 7), --basically just a bigger timer bar.
    Timer("Tremble", 7),
    Alert("Tremble"),
    Sound("Tremble"),
  }
end)