local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter

Encounter.new("AileronHydroflux", {
  instance = "Datascape",
  groupName = "Elemental Pairs",
  displayName = "Aileron & Hydroflux",
  tTrigger = {
    sType = "ALL",
    tNames = {"unit.boss_air", "unit.boss_water"},
    tZones = {
        [1] = {
            continentId = 52,
            parentZoneId = 98,
            mapId = 118,
        },
    },
  },
},{
  ["enUS"] = {
    -- Unit names
    ["unit.boss_air"] = "Aileron",
    ["unit.boss_water"] = "Hydroflux",
    --Casts
    ["cast.tsunami"] = "Tsunami",
    ["cast.icestorm"] = "Glacial Icestorm",
    --Timers
    ["timer.midphase"] = "Next Midphase",
    ["timer.tombs"] = "Next Frost Tombs",
    ["timer.icestorm"] = "Next Icestorm",
    --Alerts
    ["alert.icestorm"] = "Icestorm!",
    ["alert.twirl"] = "Twirls!",
    ["alert.twirl_you"] = "Twirl on you!",
    ["alert.midphase"] = "Midphase!",
    -- Labels
    ["label.midphase"] = "Midphase",
    ["label.tombs"] = "Tombs",
  },
  ["deDE"] = {
    -- Unit names
    ["unit.boss_air"] = "Aileron",
    ["unit.boss_water"] = "Hydroflux",
    --Casts
    ["cast.tsunami"] = "Tsunami",
    ["cast.icestorm"] = "Frostiger Eissturm",
    --Timers
    ["timer.midphase"] = "Next Midphase",
    ["timer.tombs"] = "Next Frost Tombs",
    ["timer.icestorm"] = "Next Icestorm",
    --Alerts
    ["alert.icestorm"] = "Eissturm!",
    ["alert.twirl"] = "Wirbel!",
    ["alert.twirl_you"] = "Wirbel auf dir!",
    ["alert.midphase"] = "Mittelphase!",
    -- Labels
    ["label.midphase"] = "Mittelphase",
    ["label.tombs"] = "Eisgräber",
  },
  ["frFR"] = {
    -- Unit names
    ["unit.boss_air"] = "Ventemort",
    ["unit.boss_water"] = "Hydroflux",
    --Casts
    ["cast.tsunami"] = "Tsunami",
    ["cast.icestorm"] = "Tempête de neige glaciale",
    --Timers
    ["timer.midphase"] = "Prochaine phase milieu",
    ["timer.tombs"] = "Prochain tombeau glacé",
    ["timer.icestorm"] = "Prochaine tempête glaciale",
    --Alerts
    ["alert.icestorm"] = "TEMPÊTE GLACIALE",
    ["alert.twirl"] = "TOURNOIEMENT",
    ["alert.twirl_you"] = "TOURNOIEMENT SUR VOUS!",
    ["alert.midphase"] = "Midphase!",
    -- Labels
    ["label.midphase"] = "Midphase",
    ["label.tombs"] = "Tombs",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global

  BossAir{
    label = "unit.boss_air",
    unit{color = "ff44ffff"},
    line{color = "ff00ff00", thickness = 8},
  }
  BossWater{
    label = "unit.boss_water",
    unit{color = "ff44bbff"},
  }

  Midphase{
    label = "label.midphase",
    timer{color = "xkcdLavenderBlue"},
    alert{},
    sound{file = "alert"},
  }
  Icestorm{
    label = "cast.icestorm",
    alert{},
    sound{file = "run-away"},
    timer{color = "xkcdTurquoiseBlue"}
  }
  Tombs{
    label = "label.tombs",
    timer{color = "xkcdDarkishBlue"},
  }
  Twirl{
    label = "alert.twirl",
    alert{},
    sound{file = "inferno"},
    icon{sprite = "LUIBM_crosshair", size = 60, color = "ffff0000"},
    aura{label = "alert.twirl_you", sprite = "LUIBM_run", color = "ffff0000"},
    line{color = "ffff0000"},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  _var.bPhase2 = false
  Enable{
    Timer("Midphase", 60, "timer.midphase"),
    Timer("Tombs", 30, "timer.tombs"),
  }

  UnitCreated{ strUnitName = "unit.boss_air",
    Unit("BossAir", _arg.tUnit),
    Line("BossAir", _arg.tUnit, 15),
  }

  UnitCreated{ strUnitName = "unit.boss_water",
    Unit("BossWater", _arg.tUnit),
  }

  CastStart{ strSpellName = "cast.tsunami",
    _var.bPhase2(true),
    Count("Tsunamis"),

    Alert("Midphase", "alert.midphase"),
    Sound("Midphase"),
  }
  CastEnd{ strSpellName = "cast.tsunami",
    Timer("Midphase", 88, "timer.midphase"),
    Timer("Tombs", 30, "timer.tombs"),
  }

  CastStart{ strSpellName = "cast.icestorm",
    Alert("Icestorm", "alert.icestorm"),
    Sound("Icestorm"),
  }

  BuffAdded{ [_var.bPhase2] = true, nSpellid = {69959, 47075}, --these are some Moo debuffs?
    _var.bPhase2(false),
    If{ [GetCount("Tsunamis")] = 2,
      SetCount(0, "Tsunamis"),
      Timer("Icestorm", 15, "timer.icestorm")
    }
  }

  local DEBUFF_TWIRL = 70440
  BuffAdded{ nSpellId = DEBUFF_TWIRL,
    Polygon("Twirl", _arg.tUnit, 7, 20) % (_arg.nUnitId) + {nDuration = _arg.nDuration},
    If{ [_arg.tUnit:IsThePlayer()] = true,
      Aura("Twirl", _arg.nDuration) % (_arg.nUnitId),
      Sound("Twirl"),
      Alert("Twirl", "alert.twirl_you")
    }/{
      Icon("Twirl", _arg.tUnit) % (_arg.nUnitId) + {nDuration = _arg.nDuration},
      Alert("Twirl", "alert.twirl") % (_arg.nUnitId)
    }
  }
  BuffRemoved{ nSpellId = DEBUFF_TWIRL,
    -Polygon("Twirl") % (_arg.nUnitId),
    -Aura("Twirl") % (_arg.nUnitId),
    -Icon("Twirl") % (_arg.nUnitId),
  }
end)