local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("AileronMegalith", {
  instance = "Datascape",
  groupName = "Elemental Pairs",
  displayName = "Aileron & Megalith",
  tTrigger = {
    sType = "ALL",
    tNames = {"unit.boss_air", "unit.boss_earth"},
    tZones = {
      [1] = {
        continentId = 52,
        parentZoneId = 98,
        mapId = 117,
      },
    },
  },
},{
  ["enUS"] = {
    -- Units
    ["unit.boss_air"] = "Aileron",
    ["unit.boss_earth"] = "Megalith",
    ["unit.tornado"] = "Air Column",
    -- Casts
    ["cast.supercell"] = "Supercell",
    ["cast.fierce_swipe"] = "Fierce Swipe",
    -- Datachron
    ["datachron.superquake"] = "The ground shudders beneath Megalith!",
    ["datachron.enrage"] = "Time to die, sapients!",
    -- Alerts
    ["alert.tornado"] = "Tornado incoming!",
    ["alert.supercell"] = "LAND ON MEGALITH!",
    ["alert.superquake"] = "JUMP, JUMP, JUMP!!!",
    -- Timers
    ["timer.supercell"] = "Next supercell",
    ["timer.tornado"] = "Next tornado",
    ["timer.fierce_swipe"] = "Next fierce swipe",
    -- Labels
    ["label.moo"] = "Moment of Opportunity",
    ["label.superquake"] = "Superquake",
    ["label.avatus"] = "Avatus incoming",
    ["label.enrage"] = "Enrage",
  },
  ["deDE"] = {
    -- Units
    ["unit.boss_air"] = "Aileron",
    ["unit.boss_earth"] = "Megalith",
    ["unit.tornado"] = "Luftsäule",
    -- Casts
    ["cast.supercell"] = "Superzelle",
    ["cast.fierce_swipe"] = "Heftiger Hieb",
    -- Datachron
    ["datachron.superquake"] = "Der Boden unter Megalith bebt!",
    ["datachron.enrage"] = "Zeit, zu sterben, Vernunftbegabte!",
    -- Alerts
    ["alert.tornado"] = "Tornado incoming!",
    ["alert.supercell"] = "LANDE AUF MEGALITH!",
    ["alert.superquake"] = "SPRING, SPRING, SPRING!!!",
    -- Timers
    ["timer.supercell"] = "Nächste Superzelle",
    ["timer.tornado"] = "Nächster Tornado",
    ["timer.fierce_swipe"] = "Nächster Heftiger Hieb",
    -- Labels
    ["label.moo"] = "Moment of Opportunity",
    ["label.superquake"] = "Superbeben",
    ["label.avatus"] = "Avatus incoming",
    ["label.enrage"] = "Enrage",
  },
  ["frFR"] = {
    -- Units
    ["unit.boss_air"] = "Ventemort", --V (Verified)
    ["unit.boss_earth"] = "Mégalithe", --V
    ["unit.tornado"] = "Colonne d'air", --V
    -- Casts
    ["cast.supercell"] = "Super-cellule", --V
    ["cast.fierce_swipe"] = "Baffe féroce", --V
    -- Datachron
    ["datachron.superquake"] = "Le sol tremble sous les pieds de Mégalithe !", --V
    ["datachron.enrage"] = "Maintenant c'est l'heure de mourir, misérables !", --V
    -- Alerts
    ["alert.tornado"] = "Arrivée de la tornade !",
    ["alert.supercell"] = "ATTERISSAGE SUR MÉGALITHE !",
    ["alert.superquake"] = "SAUTE, SAUTE, SAUTE !!!",
    -- Timers
    ["timer.supercell"] = "Prochaine super-cellule",
    ["timer.tornado"] = "Prochaine tornade",
    ["timer.fierce_swipe"] = "Prochaine baffe féroce",
    -- Labels
    ["label.moo"] = "Moment d'opportunité",
    ["label.superquake"] = "Super séisme",
    ["label.avatus"] = "Avatus est arrivé",
    ["label.enrage"] = "Enrager",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  BossAir{
    label = "unit.boss_air",
    unit{color = "ff44ffff"},
    line{color = "ff00ff00", thickness = 8},
  }
  BossEarth{
    label = "unit.boss_earth",
    unit{color = "ffcc8844"},
    text{color = "ffcc8844"},
  }
  Supercell{
    label = "cast.supercell",
    timer{color = "xkcdBlueBlue"},
    alert{},
    sound{file = "alarm"},
  }
  Tornado{
    label = "unit.tornado",
    timer{color = "xkcdBrightSkyBlue"},
    line{color = "ff0000ff", thickness = 8},
    alert{},
    sound{file = "info"},
  }
  Swipe{
    label = "cast.fierce_swipe",
    timer{color = "xkcdBurntYellow"},
  }
  Jump{
    label = "label.superquake",
    sound{file = "beware"},
    alert{},
    cast{},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  Enable{
    Timer("Supercell", 65, "timer.supercell"),
    Timer("Tornado", 16, "timer.tornado"),
    Timer("Swipe", 16, "timer.fierce_swipe"),
  }

  UnitCreated{ strUnitName = "unit.boss_air",
    Unit("BossAir", _arg.tUnit),
    Line("BossAir", _arg.tUnit, 15),
  }
  UnitCreated{ strUnitName = "unit.boss_earth",
    Unit("BossEarth", _arg.tUnit),
  }
  local DEBUFF_STACKS_EARTH = 51350
  BuffAdded{ nSpellId = DEBUFF_STACKS_EARTH,
    Text("BossEarth", _arg.tUnit, _arg.nStack),
  }
  BuffUpdated{ nSpellId = DEBUFF_STACKS_EARTH,
    Text("BossEarth", _arg.tUnit, _arg.nStack),
  }
  BuffRemoved{ nSpellId = DEBUFF_STACKS_EARTH,
    Text("BossEarth", _arg.tUnit, 0),
  }

  UnitCreated{ strUnitName = "unit.tornado",
    Line("Tornado", _arg.tUnit, 30),
    Timer("Tornado", 17, "timer.tornado"),
    Alert("Tornado", "alert.tornado"),
    Sound("Tornado"),
  }

  CastStart{ strSpellName = "cast.fierce_swipe",
    Timer("Swipe", 16.5, "timer.fierce_swipe"),
  }

  _var.nLastSupercell = 0
  local nGameTime = _env.GameLib.GetgameTime()
  CastStart{ strSpellName = "cast.supercell", [_var.nLastSupercell] = {"<", nGameTime-30},
    _var.nLastSupercell(nGameTime),
    Timer("Supercell", 80, "timer.supercell"),
    Alert("Supercell", "alert.supercell"),
    Sound("Supercell"),
  }

  Datachron{ strMessage = "datachron.superquake",
    Sound("Jump"),
    Alert("Jump", "alert.superquake"),
    Cast("Jump", 2),
  }
end)