local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("PyrobaneHydroflux", {
  instance = "Datascape",
  groupName = "Elemental Pairs",
  displayName = "Pyrobane & Hydroflux",
  tTrigger = {
    sType = "ALL",
    tNames = {"unit.boss_fire", "unit.boss_water"},
    tZones = {
      [1] = {
        continentId = 52,
        parentZoneId = 98,
        mapId = 118,
      },
    },
  },
},{
  ["enUS"] = {
    -- Units
    ["unit.boss_fire"] = "Pyrobane",
    ["unit.boss_water"] = "Hydroflux",
    ["unit.flame_wave"] = "Flame Wave",
    ["unit.ice_tomb"] = "Ice Tomb",
    -- Alerts
    ["alert.bombs"] = "Bombs!",
    ["alert.ice_tomb"] = "Ice Tombs!",
    ["alert.stacks"] = "WATCH YOUR DEBUFF STACKS!",
    -- Labels
    ["label.bombs"] = "Bombs",
    ["label.bombs_player"] = "Bomb on player",
    ["label.fire_bomb"] = "Fire Bomb",
    ["label.water_bomb"] = "Water Bomb",
    ["label.stacks"] = "Debuff Stacks Warning",
    ["label.avatus"] = "Avatus incoming",
    ["label.enrage"] = "Enrage",
    -- Datachron
    ["datachron.enrage"] = "Time to die, sapients!",
    -- Texts
    ["text.next_bombs"] = "Next bombs",
    ["text.next_ice_tomb"] = "Next ice tomb",
    ["text.stacks"] = "/p I hit %d stacks and just failed the challenge!",
  },
  ["deDE"] = {
    -- Units
    ["unit.boss_fire"] = "Pyroman",
    ["unit.boss_water"] = "Hydroflux",
    ["unit.flame_wave"] = "Flammenwelle",
    ["unit.ice_tomb"] = "Eisgrab",
    -- Alerts
    ["alert.bombs"] = "Bomben!",
    ["alert.ice_tomb"] = "Eisgräber!",
    -- Labels
    ["label.bombs"] = "Bomben",
    ["label.bombs_player"] = "Bombe auf Spieler",
    ["label.fire_bomb"] = "Feuer Bombe",
    ["label.water_bomb"] = "Wasser Bombe",
    ["label.avatus"] = "Avatus incoming",
    ["label.enrage"] = "Enrage",
    -- Datachron
    ["datachron.enrage"] = "Zeit, zu sterben, Vernunftbegabte!",
    -- Texts
    ["text.next_bombs"] = "Nächste Bomben",
    ["text.next_ice_tomb"] = "Nächstes Eisgrab",
    ["text.stacks"] = "/gr Ich hab %d stacks und somit die Challenge vermasselt!",
  },
  ["frFR"] = {
    -- Units
    ["unit.boss_fire"] = "Pyromagnus", --V (Verified)
    ["unit.boss_water"] = "Hydroflux", --V
    ["unit.flame_wave"] = "Vague de feu", --V
    ["unit.ice_tomb"] = "Tombeau de glace", --V
    -- Alerts
    ["alert.bombs"] = "Bombes !",
    ["alert.ice_tomb"] = "Tombeau de glace !",
    -- Labels
    ["label.bombs"] = "Bombes",
    ["label.bombs_player"] = "Bombe sur le joueur",
    ["label.fire_bomb"] = "Bombe de feu",
    ["label.water_bomb"] = "Bombe d'eau",
    ["label.avatus"] = "Avatus est arrivé",
    ["label.enrage"] = "Enrager",
    -- Datachron
    ["datachron.enrage"] = "Maintenant c'est l'heure de mourir, misérables !", --V
    -- Texts
    ["text.next_bombs"] = "Bombes suivantes",
    ["text.next_ice_tomb"] = "Tombeau de glace suivant",
    ["text.stacks"] = "/éq J'ai atteint %d stacks et j'ai perdu le défi !",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  BossFire{
    label = "unit.boss_fire",
    unit{color = "ffff0000"},
  }
  BossWater{
    label = "unit.boss_water",
    unit{color = "ff44bbff"},
  }
  FlameWave{
    label = "unit.flame_wave",
    line{thickness = 8, color = "ffff0000"},
  }
  IceTomb{
    label = "unit.ice_tomb",
    color = "ff44ffff",
    line{thickness = 6},
    sound{file = "alert"},
    alert{},
    timer{},
  }
  Bombs{
    label = "label.bombs",
    color = "ffdd00ff",
    alert{},
    sound{file = "info"},
    timer{label = "text.next_bombs"},
    text{label = "text.next_bombs"},
  }
  WaterBomb{
    label = "label.water_bomb",
    color = "ff44bbff",
    icon{sprite = "LUIBM_bomb", size= 60},
    aura{sprite = "LUIBM_bomb"},
    line{},
  }
  FireBomb{
    label = "label.fire_bomb",
    color = "ffff0000",
    icon{sprite = "LUIBM_bomb", size= 60},
    aura{sprite = "LUIBM_bomb"},
    line{},
  }
  Stacks{
    label = "label.stacks",
    alert{},
    sound{file = "alert"},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  Enable{
    Timer("Bombs", 30, "text.next_bombs"),
    Timer("IceTomb", 26, "text.next_ice_tomb"),
  }

  UnitCreated{ strUnitName = "unit.boss_fire",
    Unit("BossFire", _arg.tUnit),
  }
  UnitCreated{ strUnitName = "unit.boss_water",
    Unit("BossWater", _arg.tUnit),
  }

  UnitCreated{ strUnitName = "unit.flame_wave",
    Line("FlameWave", _arg.tUnit, 25) % (_arg.nUnitId),
  }

  UnitCreated{ strUnitName = "unit.ice_tomb", [InRange(_arg.tUnit, 45)] = true,
    LineBetween("IceTomb", _arg.tUnit, nil) % (_arg.nUnitId),
  }

  local DEBUFF_ICE_TOMB = 74326
  _var.nLastTombWarn = 0
  BuffAdded{ nSpellId = DEBUFF_ICE_TOMB, [_var.nLastTombWarn] = {"<", GameTime()-5},
    _var.nLastTombWarn(GameTime()),
    Sound("IceTomb"),
    Alert("IceTomb", "alert.ice_tomb"),
    Timer("IceTomb", 15, "text.next_ice_tomb"),
  }

  _var.tFireBombs = _arg(function() return {} end)()
  _var.bPlayerFireBomb = false
  _var.tWaterBombs = _arg(function() return {} end)()
  _var.bPlayerWaterBomb = false
  local function fBombHandler(self)
    self:Alert("Bombs", "alert.bombs")
    self:Sound("Bombs")
    self:Timer("Bombs", 29.8, "text.next_bombs")
    local tFireBoss = self:GetUnit("BossFire")
    local tWaterBoss = self:GetUnit("BossWater")
    if tFireBoss then
      self:TimerText("Bombs", tFireBoss, 29.8, "FireBoss")
    end
    if tWaterBoss then
      self:TimerText("Bombs", tWaterBoss, 29.8, "WaterBoss")
    end

    local bWaterLines = self.tVars.bPlayerFireBomb
    local bFireLines = self.tVars.bPlayerWaterBomb

    for nUnitId, tUnit in pairs(self.tVars.tFireBombs) do
      if tUnit:IsThePlayer() then
        self:Aura("FireBomb", 9.8)
      else
        self:Icon("FireBomb", tUnit, nUnitId, {nDuration = 9.8, overlay = true})
      end
      if bFireLines then
        self:LineBetween("FireBomb", tUnit, nil, nUnitId)
      end
    end

    for nUnitId, tUnit in pairs(self.tVars.tWaterBombs) do
      if tUnit:IsThePlayer() then
        self:Aura("WaterBomb", 9.8)
      else
        self:Icon("WaterBomb", tUnit, nUnitId, {nDuration = 9.8, overlay = true})
      end
      if bWaterLines then
        self:LineBetween("WaterBomb", tUnit, nil, nUnitId)
      end
    end

    --reset all variables for next runthrough
    self.tVars.tFireBombs = {}
    self.tVars.bPlayerFireBomb = false
    self.tVars.tWaterBombs = {}
    self.tVars.bPlayerWaterBomb = false
  end
  local DEBUFF_WATER_BOMB = 75058
  local DEBUFF_FIRE_BOMB = 75059
  BuffAdded{ nSpellId = DEBUFF_WATER_BOMB,
    _env.table.insert(_var.tWaterBombs, _arg.nUnitId, _arg.tUnit),
    If{ [_arg.tUnit:IsThePlayer()] = true,
      _var.bPlayerWaterBomb(true),
    },
    Delay("BombHandler", 0.1, fBombHandler),
  }
  BuffAdded{ nSpellId = DEBUFF_FIRE_BOMB,
    _env.table.insert(_var.tFireBombs, _arg.nUnitId, _arg.tUnit),
    If{ [_arg.tUnit:IsThePlayer()] = true,
      _var.bPlayerFireBomb(true),
    },
    Delay("BombHandler", 0.1, fBombHandler),
  }
  BuffRemoved {nSpellId = {DEBUFF_WATER_BOMB, DEBUFF_FIRE_BOMB},
    -LineBetween("WaterBomb") % (nUnitId),
    -LineBetween("FireBomb") % (nUnitId),
    -Icon("WaterBomb") % (nUnitId),
    -Icon("FireBomb") % (nUnitId),

    If{ [_arg.tUnit:IsThePlayer()] = true,
      -Aura("FireBomb"),
      -Aura("WaterBomb"),
    }
  }

  local DEBUFF_DRENCHED = 52874
  local DEBUFF_ENGULFED = 52876
  _var.bStackWarned = false
  BuffUpdated{ nSpellId = {DEBUFF_DRENCHED, DEBUFF_ENGULFED}, [_arg.tUnit:IsThePlayer()] = true,
    If{ [_arg.nStack] = {">=" , 10}, [_var.bStackWarned] = false,
      _var.bStackWarned(true),
      Sound("Stacks"),
      Alert("Stacks", "alert.stacks"),
    },
    If{ [_arg.nStack] = {">" , 13},
      {_env.ChatSystemLib.Command, _arg.self.L["text.stacks"]:format(_arg.nStack)}
    },
  }
  BuffRemoved{ nSpellId = {DEBUFF_DRENCHED, DEBUFF_ENGULFED}, [_arg.tUnit:IsThePlayer()] = true,
    _var.bStackWarned(false),
  }
end)