local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("PyrobaneMegalith", {
  instance = "Datascape",
  groupName = "Elemental Pairs",
  displayName = "Pyrobane & Megalith",
  tTrigger = {
    sType = "ALL",
    tNames = {"unit.boss_fire", "unit.boss_earth"},
    tZones = {
      [1] = {
        continentId = 52,
        parentZoneId = 98,
        mapId = 117,
      },
    },
  },
},{
  ["enUS"] = {
    -- Units
    ["unit.boss_fire"] = "Pyrobane",
    ["unit.boss_earth"] = "Megalith",
    ["unit.flame_wave"] = "Flame Wave",
    ["unit.stone"] = "Obsidian Outcropping",
    ["unit.lava_floor"] = "e395- [Datascape] Fire Elemental - Lava Floor (invis unit)",
    -- Alerts
    ["alert.superquake"] = "JUMP, JUMP, JUMP!!!",
    -- Datachron
    ["datachron.superquake"] = "The ground shudders beneath Megalith!",
    ["datachron.lava_floor"] = "The lava begins to rise through the floor!",
    ["datachron.enrage"] = "Time to die, sapients!",
    -- Labels
    ["label.superquake"] = "Superquake",
    ["label.lava_floor"] = "Lava Floor Phase",
    ["label.next_lava_floor"] = "Next Lava Floor",
    ["label.next_stone"] = "Next Stone",
    ["label.avatus"] = "Avatus incoming",
    ["label.enrage"] = "Enrage",

    ["timer.next_stone"] = "Next Stone: %s/%s",
  },
  ["deDE"] = {
    -- Units
    ["unit.boss_fire"] = "Pyroman",
    ["unit.boss_earth"] = "Megalith",
    ["unit.flame_wave"] = "Flammenwelle",
    ["unit.stone"] = "Obsidian Outcropping",
    ["unit.lava_floor"] = "e395- [Datascape] Fire Elemental - Lava Floor (invis unit)",
    -- Alerts
    ["alert.superquake"] = "SPRING, SPRING, SPRING!!!",
    -- Datachron
    ["datachron.superquake"] = "Der Boden unter Megalith bebt!",
    ["datachron.lava_floor"] = "Die Lava beginnt durch den Boden nach oben aufzusteigen!",
    ["datachron.enrage"] = "Zeit, zu sterben, Vernunftbegabte!",
    -- Labels
    ["label.superquake"] = "Superbeben",
    ["label.lava_floor"] = "Lava Boden Phase",
    ["label.next_lava_floor"] = "Nächster Lava Boden",
    ["label.next_stone"] = "Nächster Stein",
    ["label.avatus"] = "Avatus incoming",
    ["label.enrage"] = "Enrage",

    ["timer.next_stone"] = "Nächster Stein: %s/%s",
  },
  ["frFR"] = {
    -- Units
    ["unit.boss_fire"] = "Pyromagnus", --V (Verified)
    ["unit.boss_earth"] = "Mégalithe", --V
    ["unit.flame_wave"] = "Vague de feu", --V
    ["unit.stone"] = "Affleurement d'obsidienne", --V
    ["unit.lava_floor"] = "e395- [Datascape] Fire Elemental - Lava Floor (invis unit)", --V
    -- Alerts
    ["alert.superquake"] = "SAUTE, SAUTE, SAUTE !!!",
    -- Datachron
    ["datachron.superquake"] = "Le sol tremble sous les pieds de Mégalithe !", --V
    ["datachron.lava_floor"] = "La lave apparaît par les fissures du sol !", --V
    ["datachron.enrage"] = "Maintenant c'est l'heure de mourir, misérables !", --V
    -- Labels
    ["label.superquake"] = "Super séisme",
    ["label.lava_floor"] = "Phase de lave",
    ["label.next_lava_floor"] = "Phase de lave suivante",
    ["label.next_stone"] = "Rocher suivante",
    ["label.avatus"] = "Avatus est arrivé",
    ["label.enrage"] = "Enrager",

    ["timer.next_stone"] = "Rocher suivante: %s/%s",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global

  BossFire{
    label = "unit.boss_fire",
    unit{color = "ffff0000"},
  }
  BossEarth{
    label = "unit.boss_earth",
    unit{color = "ffcc8844"},
    text{color = "ffcc8844"},
  }

  Stone{
    label = "label.next_stone",
    timer{},
  }
  LavaFloor{
    label = "label.lava_floor",
    timer{},
  }
  FlameWave{
    label = "unit.flame_wave",
    line{thickness = 8, color = "ffff0000"},
  }
  Jump{
    label = "label.superquake",
    sound{file = "beware"},
    alert{},
    cast{},
  }

end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss_fire",
    Unit("BossFire", _arg.tUnit),
  }
  UnitCreated{ strUnitName = "unit.boss_earth",
    Unit("BossEarth", _arg.tUnit),
  }
  local DEBUFF_STACKS_EARTH = 51350
  BuffAdded{ nSpellId = DEBUFF_STACKS_EARTH,
    Text("BossEarth", _arg.tUnit, _arg.nStack),
  }
  BuffUpdated{ nSpellId = DEBUFF_STACKS_EARTH,
    Text("BossEarth", _arg.tUnit, _arg.nStack),
  }
  BuffRemoved{ nSpellId = DEBUFF_STACKS_EARTH,
    Text("BossEarth", _arg.tUnit, 0),
  }

  _var.nMaxStones = 6
  _var.nUpcomingStone = 1
  local nStoneTime = 14
  Enable{
    Timer("Stone", nStoneTime, {"timer.next_stone", _var.nUpcomingStone, _var.nMaxStones}),
    Timer("LavaFloor", 94, "label.next_lava_floor"),
  }

  UnitCreated{ strUnitName = "unit.flame_wave",
    Line("FlameWave", _arg.tUnit, 25) % (_arg.nUnitId),
  }

  UnitCreated{ strUnitName = "unit.stone", [_var.nUpcomingStone] = {"<", _var.nMaxStones},
    _var.nUpcomingStone+1,
    Timer("Stone", nStoneTime, {"timer.next_stone", _var.nUpcomingStone, _var.nMaxStones}),
  }

  Datachron{ strMessage = "datachron.lava_floor",
    Timer("LavaFloor", 28, "label.lava_floor"), --duration of phase
  }
  UnitDestroyed{ strUnitName = "unit.lava_floor",
    Timer("LavaFloor", 89, "label.next_lava_floor"),

    _var.nUpcomingStone(1),
    _var.nMaxStones(_env.max(_var.nMaxStones, 4)),
    _var.nMaxStones-2,-- at this time its either 4 or 2.
    function(self)
      --copied from RaidCore:
      local nTime = (6 - self.tVars.nObsidianPopMax) * nStoneTime + 8
      self:Timer("Stone", nTime, {"timer.next_stone", self.tVars.nUpcomingStone, self.tVars.nMaxStones})
    end,
  }
  UnitDestroyed{ strUnitName = "unit.boss_fire",
    -Timer("LavaFloor"),
  }

  Datachron{ strMessage = "datachron.superquake",
    Sound("Jump"),
    Alert("Jump", "alert.superquake"),
    Cast("Jump", 2),
  }
end)