local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("PyrobaneVisceralus", {
  instance = "Datascape",
  groupName = "Elemental Pairs",
  displayName = "Pyrobane & Visceralus",
  tTrigger = {
    sType = "ALL",
    tNames = {"unit.boss_fire", "unit.boss_life"},
    tZones = {
      [1] = {
          continentId = 52,
          parentZoneId = 98,
          mapId = 119,
      },
    },
  },
},{
  ["enUS"] = {
    -- Units
    ["unit.boss_fire"] = "Pyrobane",
    ["unit.boss_life"] = "Visceralus",
    ["unit.flame_wave"] = "Flame Wave",
    ["unit.life_orb"] = "Life Force",
    ["unit.life_essence"] = "Essence of Life",

    ["cast.blinding"] = "Blinding Light",
    -- Messages
    ["message.next_midphase"] = "Next Midphase",
    ["alert.no_heals"] = "No-Healing Debuff!",
    -- Labels
    ["label.life_force_shackle"] = "No-Healing Debuff",
    ["label.midphase"] = "Midphase",
    ["label.root"] = "Roots",
    ["label.no_heals"] = "No-Healing Debuff",
  },
  ["deDE"] = {
    -- Units
    ["unit.boss_fire"] = "Pyroman",
    ["unit.boss_life"] = "Viszeralus",
    ["unit.flame_wave"] = "Flammenwelle",
    ["unit.life_orb"] = "Lebenskraft",
    ["unit.life_essence"] = "Lebensessenz",

    ["cast.blinding"] = "Blendendes Licht",
    -- Messages
    ["message.next_midphase"] = "Nächste Mitte Phase",
    ["alert.no_heals"] = "Debuff: Keine Heilung!",
    -- Labels
    ["label.life_force_shackle"] = "Keine-Heilung Debuff",
    ["label.midphase"] = "Mittelphase",
    ["label.root"] = "Wurzeln",
    ["label.no_heals"] = "Keine-Heilung Debuff",
  },
  ["frFR"] = {
    -- Units
    ["unit.boss_fire"] = "Pyromagnus", --V (Verified)
    ["unit.boss_life"] = "Visceralus", --V
    ["unit.flame_wave"] = "Vague de feu", --V
    ["unit.life_orb"] = "Force vitale", --V
    ["unit.life_essence"] = "Essence de vie", --not sure, maybe "Essence de la vie"

    ["cast.blinding"] = "Lumière aveuglante",
    -- Messages
    ["message.next_midphase"] = "Prochaine Phase du Milieu",
    ["alert.no_heals"] = "Aucun-Soin Debuff!",
    -- Labels
    ["label.life_force_shackle"] = "Debuff: Aucun-Soin",
    ["label.midphase"] = "Phase du Milieu",
    ["label.root"] = "Roots",
    ["label.no_heals"] = "Debuff: Aucun-Soin",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  BossFire{
    label = "unit.boss_fire",
    unit{color = "ffff0000"},
  }
  BossLife{
    label = "unit.boss_life",
    unit{color = "ff22aa00"},
  }

  FlameWave{
    label = "unit.flame_wave",
    line{thickness = 8, color = "ffff0000"},
  }
  Orb{
    label = "label.orb",
    line{color = "ff0000ff"},
  }
  Midphase{
    label = "label.midphase",
    timer{color = "xkcdLightOrange"},
  }
  Root{
    label = "label.root",
    icon{color = "ffdd00ff", sprite = "LUIBM_tentacle", size= 60}
  }
  NoHeal{
    label = "label.no_heals",
    color = "ffff00ff",
    icon{sprite = "LUIBM_biohazard", size = 60},
    aura{sprite = "LUIBM_biohazard"},
    alert{},
    sound{sound = "alarm"},
  }
  Blinding{
    label = "cast.blinding",
    alert{},
    sound{file = "beware"},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  Enable{
    Timer("Midphase", 90, "message.next_midphase"),
  }

  UnitCreated{ strUnitName = "unit.boss_fire", [_arg.tUnit:GetHealth()] = {"~=", _arg(nil)},
    --The Fire-Wall is called 'Pyrobane' aswell...
    Unit("BossFire", _arg.tUnit),
  }
  UnitCreated{ strUnitName = "unit.boss_life",
    Unit("BossLife", _arg.tUnit),
  }

  UnitCreated{ strUnitName = "unit.flame_wave",
    Line("FlameWave", _arg.tUnit, 25) % (_arg.nUnitId),
  }
  UnitCreated{ strUnitName = "unit.life_orb",
    LineBetween("Orb", _arg.tUnit, _arg.self:GetUnit("BossLife")) % (_arg.nUnitId),
  }

  UnitCreated{ strUnitName = "unit.life_essence", [GetCount("Orb", _arg.nUnitId)] = 0,
    SetCount(1, "Orb", _arg.nUnitId),
    Count("Orb"),
  }
  UnitDestroyed{ strUnitName = "unit.life_essence", [GetCount("Orb", _arg.nUnitId)] = 1,
    SetCount(0, "Orb", _arg.nUnitId),
    SetCount(GetCount("Orb")-1, "Orb"),
    If{ [GetCount("Orb")] = 0,
      Timer("Midphase", 90, "message.next_midphase"),
    }
  }

  local DEBUFF_ROOT1 = 73179 -- After the root ability.
  local DEBUFF_ROOT2 = 73177 -- Before the root ability.
  BuffAdded{ nSpellId = {DEBUFF_ROOT1, DEBUFF_ROOT2},
    Icon("Root", _arg.tUnit) % (_arg.nUnitId),
  }
  BuffRemoved{ nSpellId = {DEBUFF_ROOT1, DEBUFF_ROOT2},
    -Icon("Root") % (_arg.nUnitId),
  }

  local DEBUFF_NOHEAL = 74366
  BuffAdded{ nSpellId = DEBUFF_NOHEAL,
    If{ [_arg.tUnit:IsThePlayer()] = true,
      Aura("NoHeal", nil) % (_arg.nUnitId),
      Sound("NoHeal"),
      Alert("NoHeal", "alert.no_heals")
    }/{
      Icon("NoHeal", _arg.tUnit) % (_arg.nUnitId)
    },
  }
  BuffRemoved{ nSpellId = DEBUFF_NOHEAL,
    -Aura("NoHeal") % (_arg.nUnitId),
    -Icon("NoHeal") % (_arg.nUnitId),
  }

  CastStart{ strSpellName = "cast.blinding", [InRange(_arg.tUnit, 33)] = true,
    Alert("Blinding", "cast.blinding"),
    Sound("Blinding"),
  }
end)