local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("VolatilityLattice", {
  instance = "Datascape",
  displayName = "Volatility Lattice",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"}, --thats the friendly unit 'Avatus' to the side.
    tZones = {
      [1] = {
        continentId = 52,
        parentZoneId = 98,
        mapId = 116,
      },
    },
  },
},{
  ["enUS"] = {
    -- Units
    ["unit.boss"] = "Avatus",
    ["unit.add"] = "Obstinate Logic Wall",
    ["unit.devourer"] = "Data Devourer",
    -- Casts
    ["cast.null_void"] = "Null and Void",
    -- Datachron
    ["datachron.laser"] = "Avatus sets his focus on (.*)!",
    ["datachron.delete_all"] = "Avatus prepares to delete all data!",
    ["datachron.secure_sector"] = "The Secure Sector Enhancement Ports have been activated!",
    ["datachron.jump"] = "The Vertical Locomotion Enhancement Ports have been activated!",
    -- Alerts
    ["alert.laser"] = "LASER ON %s!",
    ["alert.laser_player"] = "LASER ON YOU!",
    ["alert.pillar"] = "Click Pillars!",
    ["alert.shield_phase"] = "Shield Phase!",
    ["alert.jump_phase"] = "Jump Phase!",
    -- Messages
    ["timer.laser"] = "Next Laser",
    ["timer.pillar"] = "Next Pillars",
    ["timer.devourer"] = "Next Devourers",
    ["timer.pillar_timeout"] = "Pillar Timeout",
    ["timer.enrage"] = "Enrage",
    ["timer.explosion"] = "Explosion",
    -- Labels
    ["label.laser"] = "Laser",
    ["label.pillar"] = "Pillar",
    ["label.add"] = "Big Add",
    ["label.midphase"] = "Midphase",
  },
  ["deDE"] = {
    -- Units
    ["unit.boss"] = "Avatus",
    ["cast.add"] = "Hartnäckige Logikmauer",
    ["unit.devourer"] = "Datenverschlinger",
    -- Casts
    ["cast.null_void"] = "Unordnung und Chaos",
    -- Datachron
    ["datachron.laser"] = "Avatus nimmt (.*) ins Visier!",
    ["datachron.delete_all"] = "???",
    ["datachron.secure_sector"] = "???",
    ["datachron.jump"] = "???",
    -- Alerts
    ["alert.laser"] = "Laser auf %s!",
    ["alert.laser_player"] = "Laser auf dir!",
    ["alert.pillar"] = "Säulen klicken!",
    ["alert.shield_phase"] = "Schild-Phase!",
    ["alert.jump_phase"] = "Sprung-Phase!",
    -- Messages
    ["timer.laser"] = "Nächster Laser",
    ["timer.pillar"] = "Nächste Säulen",
    ["timer.devourer"] = "Nächste Verschlinger",
    ["timer.pillar_timeout"] = "Säulen Explosion",
    ["timer.enrage"] = "Enrage",
    ["timer.explosion"] = "Explosion",
    -- Labels
    ["label.laser"] = "Laser",
    ["label.pillar"] = "Säule",
    ["label.add"] = "Großes Add",
    ["label.midphase"] = "Mittelphase",
  },
  ["frFR"] = {
    -- Units
    ["unit.boss"] = "Avatus", --V (Verified)
    ["cast.add"] = "Mur de logique obstiné", --Raidcore
    ["unit.devourer"] = "Dévoreur de données", --V
    -- Casts
    ["cast.null_void"] = "Caduque",
    -- Datachron
    ["datachron.laser"] = "Avatus se concentre sur (.*) !", --V
    ["datachron.delete_all"] = "Avatus se prépare à effacer toutes les données !", --V
    ["datachron.secure_sector"] = "Les ports d'amélioration de secteur sécurisé ont été activés !", --V
    ["datachron.jump"] = "Les ports d'amélioration de locomotion verticale ont été activés !", --V
    -- Alerts
    ["alert.laser"] = "LASER SUR %s !",
    ["alert.laser_player"] = "LASER SUR TOI !",
    ["alert.pillar"] = "Click Pillars!",
    ["alert.shield_phase"] = "PHASE BOUCLIER",
    ["alert.jump_phase"] = "PHASE SAUTER",
    -- Messages
    ["timer.laser"] = "Prochain Laser",
    ["timer.pillar"] = "Prochain Pilier",
    ["timer.devourer"] = "Prochain Dévoreur",
    ["timer.pillar_timeout"] = "Pillier Expiration",
    ["timer.enrage"] = "Enrage",
    ["timer.explosion"] = "Explosion",
    -- Labels
    ["label.laser"] = "Laser",
    ["label.pillar"] = "Pilier",
    ["label.add"] = "Big Add",
    ["label.midphase"] = "Midphase",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global

  Add{
    label = "label.add",
    unit{},
  }

  Enrage{
    label = "timer.enrage",
    timer{color = "xkcdAmethyst"},
  }

  Devourer{
    label = "unit.devourer",
    timer{color = "xkcdBrightLimeGreen"},
    line{thickness = 5, color = "ff0000ff"},
  }

  Pillar{
    label = "label.pillar",
    timer{color = "xkcdOrangeYellow"},
    alert{},
    sound{file = "info"},
    -- ["PILLAR_TIMEOUT"] = { sColor = "" },
  }

  Laser{
    label = "label.laser",
    timer{color = "xkcdLipstickRed"},
    icon{sprite = "LUIBM_crosshair", size = 60, color = "ffff0000"},
    aura{sprite = "LUIBM_run", color = "ffff0000"},
    alert{},
    sound{file = "run-away"},
  }

  Midphase{
    label = "label.midphase",
    alert{},
    sound{file = "alert"},
    timer{color = "xkcdBabyPurple"},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  Enable{
    Timer("Enrage", 576, "timer.enrage"),
    Timer("Devourer", 10, "timer.devourer"),
    Timer("Pillar", 45, "timer.pillar"),
  }

  _var.nLastDevourer = 0
  local nGameTime = _env.GameLib.GetGameTime()
  UnitCreated{ strUnitName = {"===", "unit.devourer"},
    LineBetween("Devourer", _arg.tUnit, nil) % (_arg.nUnitId) + {max = 45},
    If{ [_var.nLastDevourer] = {"<", nGameTime-13},
      _var.nLastDevourer(nGameTime),
      Timer("Devourer", 15, "timer.devourer")
    },
  }

  UnitCreated{ strUnitname = "unit.add",
    -Timer("Pillar") % ("Timeout"),
    Unit("Add", _arg.tUnit) % (_arg.nUnitId),
  }

  local DEBUFF_LASER_TARGET = 47160
  BuffAdded{ nSpellId = DEBUFF_LASER_TARGET,
    Timer("Laser", 15, "timer.laser"),
    If{ [_arg.tUnit:IsThePlayer()] = true,
      Alert("Laser", "alert.laser_player"),
      Sound("Laser"),
      Aura("Laser", 15)
    }/{
      Alert("Laser", {"alert.laser", _arg.strUnitName}),
      Icon("Laser", _arg.tUnit) % (_arg.nUnitId) + {nDuration = _arg.nDuration}
    },
  }
  BuffRemoved{ nSpellId = DEBUFF_LASER_TARGET,
    -Icon("Laser") % (_arg.nUnitId),
  }

  Datachron{ strMessage = "datachron.delete_all",
    Alert("Pillar", "alert.pillar"),
    Sound("Pillar"),
    Timer("Pillar", 50, "timer.pillar"),
    Timer("Pillar", 10, "timer.pillar_timeout") % ("Timeout") + {color = "xkcdAppleGreen"},
  }

  Datachron{ strMessage = "datachron.secure_sector",
    Alert("Midphase", "alert.shield_phase"),
    Sound("Midphase"),
    Timer("Midphase", 15, "timer.explosion"),
    Timer("Laser", 44, "timer.laser"),
    Timer("Devourer", 53, "timer.devourer"),
    Timer("Pillar", 58, "timer.pillar"),
  }
  Datachron{ strMessage = "datachron.jump",
    Alert("Midphase", "alert.jump_phase"),
    Sound("Midphase"),
    Timer("Laser", 58, "timer.laser"),
    Timer("Devourer", 68, "timer.devourer"),
    Timer("Pillar", 75, "timer.pillar"),
  }
end)