local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter

local tZones = {
  [1] = {
    continentId = 15,    -- KV
    parentZoneId = 0,
    mapId = 21,
  },
}

-- ##########################################################################################################
-- # Grond (1)
-- ##########################################################################################################


Encounter.new("KVGrond", {
  instance = "Dungeons",
  groupName = "KV",
  displayName = "(1) Grond",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Grond the Corpsemaker",
    ["unit.flee"] = "Bloodsucker Flea",

    ["cast.thrash"] = "Thrash",
    ["cast.disarm"] = "Bellow",

    ["label.thrash"] = "Thrash",
    ["label.disarm"] = "Disarm",
    ["label.flee"] = "Flee",
  },
  ["deDE"] = {
    ["unit.boss"] = "Grond der Leichenmacher",
    ["unit.flee"] = "Blutsaugerfloh",

    ["cast.thrash"] = "Dreschen",
    ["cast.disarm"] = "Gebrüll",

    ["label.thrash"] = "Dreschen",
    ["label.disarm"] = "Entwaffnung",
    ["label.flee"] = "Floh",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Thrash{
    label = "label.thrash",
    cast{},
    timer{},
    line{color = "ffffaa00"},
  }
  Disarm{
    label = "label.disarm",
    cast{},
    timer{},
  }
  Flee{
    label = "label.flee",
    line{thickness = 6},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    Unit("Boss", _arg.tUnit),
    Timer("Thrash", 16),
  }

  CastStart{ strSpellName = "cast.thrash",
    Cast("Thrash", _arg.tCast),
    Timer("Thrash", 28.5),
  }

  CastStart{ strSpellName = "cast.thrash", nDuration = 1,
    {function(self, tUnit)
      local tPosition = tUnit:GetPosition()
      local tFacing = tUnit:GetFacing()

      local nFullCircle = 11

      for i = 0, 10, 1 do
        local nRotationFactor = i/nFullCircle

        local nCos = math.cos(nRotationFactor*math.pi*2)
        local nSin = math.sin(nRotationFactor*math.pi*2)

        self:Line("Thrash", tPosition, 8, i.."L", {
          tVectorFacing = tFacing,
          nDuration = 6.5,
          nOffset = 18,
          tVectorOffsets = {x = 5*nCos, y = 0, z = 5*nSin},
          nRotation = 180+nRotationFactor*360,
        })
        self:Line("Thrash", tPosition, 8, i.."R", {
          tVectorFacing = tFacing,
          nDuration = 6.5,
          nOffset = 18,
          tVectorOffsets = {x = -5.5*nCos, y = 0, z = -5.5*nSin},
          nRotation = 180+nRotationFactor*360,
        })
      end
    end, _arg.self, _arg.tUnit}
  }

  CastStart{ strSpellName = "cast.disarm",
    Cast("Disarm", _arg.tCast),
    Timer("Disarm", 25),
  }

  UnitCreated{ strUnitName = "unit.flee",
    {function(self, tUnit, nId)
      local tTarget = tUnit:GetTarget()
      if tTarget and tTarget == GameLib.GetPlayerUnit() then
        self:LineBetween("Flee", tUnit, nil, nId)
      end
    end, _arg.self, _arg.tUnit, _arg.nUnitId},
  }
end)

-- ##########################################################################################################
-- # Drokk (2)
-- ##########################################################################################################


Encounter.new("KVDrokk", {
  instance = "Dungeons",
  groupName = "KV",
  displayName = "(2) Drokk",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Slavemaster Drokk",
    ["unit.massTether"] = "Enslavement Unit",
    ["unit.tether"] = "Tether Anchor",

    ["cast.wave1"] = "Suppression Wave",
    ["cast.wave2"] = "Deplete Silo",

    ["datachron.tracked"] = "A tracking",
    ["datachron.tether"] = "lockdown!",

    ["label.tracked"] = "Tracked by Adds",
    ["label.wave"] = "Wave",
    ["label.tether"] = "Tether",
  },
  ["deDE"] = {
    ["unit.boss"] = "Sklavenmeister Drokk",
    ["unit.massTether"] = "Versklavungseinheit",
    ["unit.tether"] = "Haltestrickverankerung",

    ["cast.wave1"] = "Unterdrückungswelle",
    ["cast.wave2"] = "Silo leeren",

    ["datachron.tracked"] = "Peilsender angebracht!",
    ["datachron.tether"] = "ist abgeriegelt!",

    ["label.tracked"] = "Verfolgt von Adds",
    ["label.wave"] = "Welle",
    ["label.tether"] = "Tether",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    text{},
  }
  Tracked{
    label = "label.tracked",
    text{color = "ffff0000"},
    timer{},
  }
  Wave{
    label = "label.wave",
    text{color = "ffff0000"},
    timer{}
  }
  Tether{
    label = "label.tether",
    line{thickness = 6, color = "ffff0000"},
    timer{},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  local tPhaseSwitch = {85, 65, 45, 25}
  local tMidphaseLength = {26, 57, 31.5, 57}


  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    {"Unit", "Boss", _arg.tUnit},
    {"HealthText", "Boss", _arg.tUnit, tPhaseSwitch[1]},
  }

  --##### Midphase1: Moving Probes #####--
  HealthReached{ nHealthReached = tPhaseSwitch[1], strUnitName = "unit.boss",
    {"TimerText", "Boss", _arg.tUnit, tMidphaseLength[1]},
    {"Delay", tMidphaseLength[1],
      {"HealthText", "Boss", _arg.tUnit, tPhaseSwitch[2]},
    },
  }

  --##### Midphase2: Tracking Probes #####--
  HealthReached{ nHealthReached = tPhaseSwitch[2], strUnitName = "unit.boss",
    {"TimerText", "Boss", _arg.tUnit, tMidphaseLength[2]},
    {"Delay", tMidphaseLength[2],
      {"HealthText", "Boss", _arg.tUnit, tPhaseSwitch[3]},
    },
  }

  --##### Midphase3: MassTethers #####--
  local bMassTetherPhase = false
  local tMassTethers = {}
  Enable{function()
    bMassTetherPhase = false
    tMassTethers = {}
  end}

  UnitCreated{ strUnitName = "unit.massTether", --Start of Phase 3
    function() bMassTetherPhase = true end,
    {"TimerText", "Boss", _arg.self:GetUnit("Boss"), tMidphaseLength[3]},
    {function(tUnit, nId)
      tMassTethers[nId] = tUnit
    end, _arg.tUnit, _arg.nUnitId}
  }
  UnitDestroyed{ strUnitName = "unit.massTether", --End of Phase 3 (possibly)
    {function(self, nId)
      tMassTethers[nId] = nil
      if not next(tMassTethers) then
        bMassTetherPhase = false
        local tBoss = self:GetUnit("Boss")
        if tBoss:GetHealth() / tBoss:GetMaxHealth() > tPhaseSwitch[4]/100 then
          self:HealthText("Boss", tBoss, tPhaseSwitch[4])
        else
          self:TimerText("Boss", _arg.tUnit, tMidphaseLength[4])
        end
      end
    end, _arg.self, _arg.nUnitId}
  }


  --##### Midphase4: Tracking Probes #2 #####--
  HealthReached{ nHealthReached = tPhaseSwitch[4], strUnitName = "unit.boss",
    {function(self, tUnit)
      if not bMassTetherPhase then
        self:TimerText("Boss", tUnit, tMidphaseLength[4])
      end
    end, _arg.self, _arg.tUnit}
  }

  Datachron{ strMessage = "datachron.tracked",
    {"Timer", "Tracked", 17},
    {function(self, strMsg)
      for _, tUnit in ipairs(self:GetGroupUnits()) do
        if strMsg:find(tUnit:GetName(), nil, true) then
          self:TimerText("Tracked", tUnit, 17)
          return
        end
      end
    end,_arg.self, _arg.strMessage}
  }
  BuffAdded{ nSpellId = 39293,
    {"TimerText", "Tracked", _arg.tUnit, _arg.nDuration},
  }

  Datachron{ strMessage = "datachron.tether",
    {"Timer", "Tether", 20},
  }
  UnitCreated{ strUnitName = "unit.tether",
    {"LineBetween", "Tether", _arg.tUnit, nil}
  }

  CastStart{ strSpellName = "cast.wave1",
    {"TimerText", "Wave", _arg.tUnit, 15, nil, {nOffset = -50}},
    {"Timer", "Wave", 15},
  }

  CastStart{ strSpellName = "cast.wave2", --waves in massTether
    {"TimerText", "Wave", _arg.tUnit, 8.2, nil, {nOffset = -50}},
    {"Timer", "Wave", 8.2},
  }
end)

-- ##########################################################################################################
-- # Trogun (3)
-- ##########################################################################################################


Encounter.new("KVTrogun", {
  instance = "Dungeons",
  groupName = "KV",
  displayName = "(3) Trogun",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Forgemaster Trogun",
    ["unit.orb"] = "Sphere of Primal Fire",

    ["cast.hammer"] = "Volcanic Strike",
    ["cast.midphase"] = "Exanite Weapon",

    ["datachron.orb"] = "primal fires of",

    ["label.orb"] = "Orbs",
    ["label.hammer"] = "Hammer Strike",
    ["label.midphase"] = "Midphase",
  },
  ["deDE"] = {
    ["unit.boss"] = "Schmiedemeister Trogun",
    ["unit.orb"] = "Sphäre des Urfeuers",

    ["cast.hammer"] = "Vulkanschlag",
    ["cast.midphase"] = "Exanit-Waffe",

    ["datachron.orb"] = "ruft die Urfeuer",

    ["label.orb"] = "Sphären",
    ["label.hammer"] = "Hammerschlag",
    ["label.midphase"] = "Mittelphase",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Orb{
    label = "label.orb",
    line{color = "ff00ffff", thickness=4},
    timer{},
    text{color = "ff00ffff"},
  }
  Hammer{
    label = "label.hammer",
    timer{},
    text{color = "ffff0000"},
  }
  Midphase{
    label = "label.midphase",
    timer{},
    text{},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    {"Unit", "Boss", _arg.tUnit},
  }

  function RemoveAllTimers(self)
    self:RemoveTimer("Midphase")
    self:RemoveTimerText("Midphase")
    self:RemoveTimer("Orb")
    self:RemoveTimerText("Orb")
    self:RemoveTimer("Hammer")
    self:RemoveTimerText("Hammer")
  end


  local nNextOrbs = 0
  local function nextTimers(self, nTimeHammer)
    local nTimeOrbs = nNextOrbs - GameLib.GetGameTime()

    RemoveAllTimers(self)

    if nTimeOrbs > 0 and nTimeOrbs < nTimeHammer+1 then --we're giving ourselves an error of 1sec.
      self:Timer("Orb", nTimeOrbs)
      self:TimerText("Orb", self:GetUnit("Boss"), nTimeOrbs)
    else
      self:Timer("Hammer", nTimeHammer)
      self:TimerText("Hammer", self:GetUnit("Boss"), nTimeHammer)
    end
  end

  CastStart{ strSpellName = "cast.midphase",
    {RemoveAllTimers, _arg.self},
    {"Timer", "Midphase", _arg.nDuration},
    {"TimerText", "Midphase", _arg.tUnit, _arg.nDuration},
  }

  CastEnd{ strSpellName = "cast.midphase",
    function() nNextOrbs = GameLib.GetGameTime() + 22 end,
    {nextTimers, _arg.self, 8.5},
  }

  Datachron{ strMessage = "datachron.orb",
    {"Timer", "Orb", 8},
    {"TimerText", "Orb", _arg.self:GetUnit("Boss"), 8},
    function() nNextOrbs = GameLib.GetGameTime() + 33 end,
  }

  CastStart{ strSpellName = "cast.hammer",
    {"Timer", "Hammer", 13.5},
    {"TimerText", "Hammer", _arg.tUnit, 13.5},
    {nextTimers, _arg.self, 13.5},
  }

  UnitCreated{ strUnitName = "unit.orb",
    {"LineBetween", "Orb", _arg.tUnit, _arg.self:GetUnit("Boss"), _arg.nUnitId}
  }
end)