local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("PAInvulnotron", {
  instance = "Dungeons",
  groupName = "PA",
  displayName = "(1) Invulnotron",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 90,
        parentZoneId = 469,
        mapId = 471,
      },
    },
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Invulnotron",
    ["unit.orb"] = "Unstable Energy", --guessed!

    ["cast.shield"] = "Shield and Destroy",

    ["chat.stacks"] = "%s removed %s Stacks!",

    ["label.midphase"] = "Midphase",
    ["label.player_stacks"] = "Player Stacks",
  },
  ["deDE"] = {
    ["unit.boss"] = "Invulnotron",
    ["unit.orb"] = "Instabile Energie",

    ["cast.shield"] = "Abschirmen und zerstören",

    ["chat.stacks"] = "%s entfernte %s Stacks!",

    ["label.midphase"] = "Mittelphase",
    ["label.player_stacks"] = "Player Stacks",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Midphase{
    label = "label.midphase",
    cast{},
    timer{sound = true},
    text{},
    line{thickness = 3, color = "blue"}
  }
  PlayerStacks{
    label = "label.player_stacks",
    text{color = "ffffa500"}
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
    HealthText("Midphase", _arg.tUnit, 70),
  }

  _var.bMidphase = false
  _var.nStacksLeft = 0
  CastStart{ strSpellName = "cast.shield",
    _var.nStacksLeft(8),
    _var.bMidphase(true),
    Cast("Midphase", _arg.nDuration),
    Timer("Midphase", _arg.nDuration),
    Text("Midphase", _arg.tUnit, 8),
  }
  CastEnd{ strSpellName = "cast.shield",
    HealthText("Midphase", _arg.tUnit, 40),
    _var.nStacksLeft(0),
    _var.bMidphase(false),
    -Cast("Midphase"),
    -Timer("Midphase"),
    -Text("Midphase"),
  }

  BuffAdded{ nSpellId = 81670,
    Text("PlayerStacks", _arg.tUnit, _arg.nStack) % (_arg.nUnitId),
    SetCount(_arg.nStack, "PlayerStacks", _arg.nUnitId),
  }
  BuffUpdated{ nSpellId = 81670, nStack = {">", GetCount("PlayerStacks", _arg.nUnitId)},
    Text("PlayerStacks", _arg.tUnit, _arg.nStack) % (_arg.nUnitId),
    SetCount(_arg.nStack, "PlayerStacks", _arg.nUnitId),
  }
  BuffRemoved{ nSpellId = 81670,
    -Text("PlayerStacks") % (_arg.nUnitId),
    If{ [_var.bMidphase] = true, [_arg.tUnit:IsDead()] = false, [GetCount("PlayerStacks", _arg.nUnitId)] = {"~=", 0},
      _var.nStacksLeft-GetCount("PlayerStacks", _arg.nUnitId),
      Print({"chat.stacks", _arg.strUnitName, GetCount("PlayerStacks", _arg.nUnitId)}),
      Text("Midphase", _arg.self:GetUnit("Boss"), _var.nStacksLeft)
    },
    SetCount(0, "PlayerStacks", _arg.nUnitId),
  }

  UnitCreated{ strUnitName = "unit.orb",
    LineBetween("Midphase", nil, _arg.tUnit) % (_arg.nUnitId) + {min = 5, max = 40, nOffset = 2},
  }
end)


Encounter.new("PAGromka", {
  instance = "Dungeons",
  groupName = "PA",
  displayName = "(2) Gromka",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 90,
        parentZoneId = 469,
        mapId = 471,
      },
    },
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Gromka the Flamewitch",

    ["cast.midphase"] = "Inferno",
    ["cast.jump"] = "Demon's Step",

    ["alert.midphase"] = "Midphase - Run!",
    ["alert.interrupt"] = "INTERRUPT!",

    ["label.midphase"] = "Midphase",
    ["label.debuff"] = "Dispelable Debuff",
    ["label.jump"] = "Bosses Jump Ability",
  },
  ["deDE"] = {
    ["unit.boss"] = "Flammenhexe Gromka",

    ["cast.midphase"] = "Inferno",
    ["cast.jump"] = "Schritt des Dämons",

    ["alert.midphase"] = "Mittelphase - Lauf!",
    ["alert.interrupt"] = "UNTERBRECHEN!",

    ["label.midphase"] = "Mittelphase",
    ["label.debuff"] = "Dispelbarer Debuff",
    ["label.jump"] = "Boss Sprung-Fähigkeit",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }

  Midphase{
    label = "label.midphase",
    text{},
    alert{},
    sound{file = "alert"},
  }

  Debuff{
    label = "label.debuff",
    icon{size = 60, sprite = "LUIBM_twirl", color = "ffff00ff"}
  }

  Jump{
    label = "label.jump",
    cast{},
    timer{},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
    HealthText("Midphase", _arg.tUnit, 75),
  }

  CastStart{ strSpellName = "cast.midphase",
    If{ nDuration = 2,
      Alert("Midphase", "alert.midphase"),
      Sound("Midphase"),
    }/{ nDuration = 25,
      Delay( "Interrupt", 10,
        Alert("Midphase", "alert.interrupt"),
        Sound("Midphase")
      )
    }
  }

  CastEnd{ strSpellName = "cast.midphase", nDuration = 25,
    HealthText("Midphase", _arg.tUnit, 25),
  }

  local DEBUFF_DISPEL = 82202
  BuffAdded{ nSpellId = DEBUFF_DISPEL,
    Icon("Debuff", _arg.tUnit) % (_arg.nUnitId),
  }
  BuffRemoved{ nSpellId = DEBUFF_DISPEL,
    -Icon("Debuff") % (_arg.nUnitId),
  }

  CastStart{ strSpellName = "cast.jump",
    Cast("Jump", _arg.tCast),
    Timer("Jump", 14),
  }
end)


Encounter.new("PAIruki", {
  instance = "Dungeons",
  groupName = "PA",
  displayName = "(3) Iruki",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 90,
        parentZoneId = 469,
        mapId = 471,
      },
    },
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Iruki Boldbeard",
    ["unit.shield"] = "Shield Generator",

    ["cast.midphase"] = "Annihilate",

    ["alert.midphase"] = "Midphase!",

    ["label.midphase"] = "Midphase",
  },
  ["deDE"] = {
    ["unit.boss"] = "Iruki Stolzbart",
    ["unit.shield"] = "Schildgenerator",

    ["cast.midphase"] = "Auslöschen",

    ["alert.midphase"] = "Mittelphase!",

    ["label.midphase"] = "Mittelphase",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }

  Midphase{
    label = "label.midphase",
    text{},
    alert{},
    sound{file = "alert"},
    cast{},
    timer{},
    line{thickness = 5, color = "blue"},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
    HealthText("Midphase", 75),
  }

  CastStart{ strSpellName = "cast.midphase",
    Alert("Midphase", "alert.midphase"),
    Sound("Midphase"),
    Cast("Midphase", _arg.tCast),
    Timer("Midphase", _arg.nDuration),
  }
  CastEnd{ strSpellName = "cast.midphase",
    HealthText("Midphase", 25),
  }

  _var.tShields = _var({})
  UnitCreated{ strUnitName = "unit.shield",
    _env.rawset(_var.tShields, _arg.nUnitId, _arg.tUnit),
    LineBetween("Midphase", nil, _arg.tUnit) % (_arg.nUnitId),
  }
  UnitDestroyed{ strUnitName = "unit.shield",
    _env.rawset(_var.tShields, _arg.nUnitId, _arg(nil), 1),
    -LineBetween("Midphase") % (_arg.nUnitId),
  }
  local DEBUFF_SHIELDED = 82254
  BuffAdded{ nSpellId = DEBUFF_SHIELDED,
    {function(self, tUnit)
      local tShields = self.tVars.tShields
      --search the closest shield -> remove the line towards it.
      local v3UnitPos = Vector3.New(tUnit:GetPosition())
      local nClosest = nil --nShieldId
      local nDistance = nil
      for nShieldId, tShield in pairs(tShields) do
        local v3Shield = Vector3.New(tShield:GetPosition())
        local nNewDist = (v3Shield-v3UnitPos):Length()
        if not nDistance or nNewDist < nDistance then
          nClosest = nShieldId
          nDistance = nNewDist
        end
      end

      if nClosest then
        self:RemoveLineBetween("Midphase", nClosest)
      end
    end, _arg.self, _arg.tUnit}
  }
end)


Encounter.new("PASeekNSlaughter", {
  instance = "Dungeons",
  groupName = "PA",
  displayName = "(4) Seek-N-Slaughter",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 90,
        parentZoneId = 469,
        mapId = 472,
      },
    },
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Seek-N-Slaughter",
    ["unit.bomb"] = "Bomb-O-Tron",

    ["alert.interrupt"] = "INTERRUPT!",

    ["label.midphase"] = "Midphase",
    ["label.tank_cast"] = "Tank Cast",
  },
  ["deDE"] = {
    ["unit.boss"] = "Such-und-Tot-Bot",
    ["unit.bomb"] = "Bomb-O-Tron",

    ["alert.interrupt"] = "UNTERBRECHEN!",

    ["label.midphase"] = "Mittelphase",
    ["label.tank_cast"] = "Tank Cast",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Bomb{
    label = "unit.bomb",
    unit{color = "ffff0000"},
    text{},
  }
  Midphase{
    label = "label.midphase",
    text{},
  }
  TankCast{
    label = "label.tank_cast",
    cast{},
    alert{},
    sound{file = "alert"},
    timer{},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
    HealthText("Midphase", _arg.tUnit, 66),
  }

  _var.nValidBombs = 0
  _var.arBombCache = _var({})
  UnitCreated{ strUnitName = "unit.bomb", [_arg.self:HasUnit("Bomb", _arg.nUnitId)] = false,
    _var.nValidBombs+1,
    _env.table.insert(_var.arBombCache, _arg.tUnit), --just a simple array insert.
    Delay("MarkBombs", 0.1, function(self)
      local arBombCache = self.tVars.arBombCache
      local tBoss = self:GetUnit("Boss")
      local v3Boss = Vector3.New(tBoss:GetPosition())
      table.sort(arBombCache, function(tBomb1, tBomb2)
        v3Bomb1 = Vector3.New(tBomb1:GetPosition())
        v3Bomb2 = Vector3.New(tBomb2:GetPosition())
        return (v3Bomb1-v3Boss):Length() < (v3Bomb2-v3Boss):Length()
      end)

      for strMark, tBomb in ipairs(arBombCache) do
        self:Unit("Bomb", tBomb, tBomb:GetId(), {
          strMark = strMark,
        })
        self:Text("Bomb", tBomb, strMark, tBomb:GetId())
      end

      self.tVars.arBombCache = {}
    end),
  }
  UnitDestroyed{ strUnitName = "unit.bomb",
    _var.nValidBombs-1,
    If{ [_var.nValidBombs] = 0,
      HealthText("Midphase", _arg.self:GetUnit("Boss"), 33),
    },
  }

  local DEBUFF_TANKCAST = 81647
  BuffAdded{ nSpellId = DEBUFF_TANKCAST,
    Alert("TankCast", "alert.interrupt"),
    Sound("TankCast"),
    Cast("TankCast", 4.5) + {strText = "alert.interrupt"},
    Timer("TankCast", 20, "label.tank_cast"),
  }
  CastStart{ strUnitName = "unit.boss", strSpellName = "MOO",
    -Cast("TankCast"),
  }

end)



Encounter.new("PAIcebox", {
  instance = "Dungeons",
  groupName = "PA",
  displayName = "(5) Icebox",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 90,
        parentZoneId = 469,
        mapId = 472,
      },
    },
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Icebox Mk. 2",
    ["unit.add"] = "Melee Mech",
  },
  ["deDE"] = {
    ["unit.boss"] = "Eisschrank V2",
    ["unit.add"] = "Nahkampf-Mech",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global
  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
  }
end)

Encounter.new("PASuperInvulnotron", {
  instance = "Dungeons",
  groupName = "PA",
  displayName = "(6) Super-Invulnotron",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 90,
        parentZoneId = 469,
        mapId = 472,
      },
    },
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Super-Invulnotron",
    ["unit.bot"] = "Botherbot",
    ["unit.generator"] = "Energized Generator",

    ["alert.generator"] = "Generator Spawned - Move the Boss!",
    ["alert.knockback"] = "Knockback on %s!",

    ["label.midphase"] = "Midphase",
    ["label.knockback"] = "Knockback",
  },
  ["deDE"] = {
    ["unit.boss"] = "Super-Invulnotron",
    ["unit.bot"] = "Störbot",
    ["unit.generator"] = "Energiegeladener Generator",

    ["alert.midphase"] = "Generator erschienen! - Beweg den Boss!",
    ["alert.knockback"] = "Rückstoß auf %s!",

    ["label.midphase"] = "Mittelphase",
    ["label.knockback"] = "Rückstoß",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Generator{
    label = "label.midphase",
    line{thickness = 10, color = "ffff0000"},
    text{},
    alert{},
    sound{file = "alert"},
  }
  Knockback{
    label = "label.knockback",
    line{color = "blue", thickness = 6},
    alert{},
    sound{file = "long"},
    icon{color = "green", sprite = "LUIBM_crosshair", size = 60},
    cast{},
  }

  Midphase{
    label = "label.midphase",
    text{},
    alert{},
    sound{file = "alert"},
    line{thickness = 9, color = "red"},
  }

end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
    HealthText("Midphase", _arg.tUnit, 75),
  }

  UnitCreated{ strUnitName = "unit.generator",
    Alert("Midphase", "alert.midphase"),
    Sound("Midphase"),
    LineBetween("Midphase", _arg.tUnit, _arg.self:GetUnit("Boss")),
  }
  UnitDestroyed{ strUnitName = "unit.generator",
    If{ [_arg.self:GetHealth("Boss")] = {">", 50},
      HealthText("Midphase", _arg.self:GetUnit("Boss"), 50),
    }/{ [_arg.self:GetHealth("Boss")] = {">", 25},
      HealthText("Midphase", _arg.self:GetUnit("Boss"), 25),
    }
  }

  _var.tBots = _var({})
  UnitCreated{ strUnitName = "unit.bot",
    _env.rawset(_var.tBots, _arg.nUnitId, _arg.tUnit), -- tBots[nUnitId] = tUnit
  }
  UnitDestroyed{ strUnitName = "unit.bot",
    _env.rawset(_var.tBots, _arg.nUnitId, _arg(nil), 1), -- tBots[nUnitId] = nil
  }

  BuffAdded{ nSpellId = 81943, --Knockback Target
    Cast("Knockback", _arg.nDuration),
    Alert("Knockback", {"alert.knockback", _arg.strUnitName}),
    If{ [_arg.tUnit:IsThePlayer()] = true,
      Sound("Knockback"),
      LineBetween("Knockback", _arg.self:GetUnit("Boss"), nil) % ("Player") + {
        nOffset = 2,
        nLength = 20,
        color = "green",
      },
      function(self)
        local tBoss = self:GetUnit("Boss")
        for nBotId, tBot in pairs(self.tVars.tBots) do
          if tBot:IsValid() then
            self:LineBetween("Knockback", tBoss, tBot, nBotId, {
              nOffset = 4,
              nLength = 20,
            })
          end
        end
      end,
    }/{
      Icon("Knockback", _arg.tUnit),
    },
  }
  BuffRemoved{ nSpellId = 81943, --Knockback Target
    -Icon("Knockback"),
    -LineBetween("Knockback") % ("Player"),
    function(self)
      for nBotId, _ in pairs(self.tVars.tBots) do
        self:RemoveLineBetween("Knockback", nBotId)
      end
    end,
  }
end)


Encounter.new("PAWrathbone", {
  instance = "Dungeons",
  groupName = "PA",
  displayName = "(7) Wrathbone",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 90,
        parentZoneId = 469,
        mapId = 473,
      },
    },
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Wrathbone",
    ["unit.add"] = "Flameling",
    ["unit.meteor"] = "Rock",

    ["cast.pound"] = "Ground and Pound",
    ["cast.jump"] = "[UPJ] Room 3 - Boss - Leap [DNT]", --jump to, and from midphase location.

    ["alert.interrupt"] = "Interrupt!",

    ["label.mdiphase"] = "Midphase",
    ["label.meteor"] = "Meteor",
  },
  ["deDE"] = {
    ["unit.boss"] = "Bosbein",
    ["unit.add"] = "Flammling",
    ["unit.meteor"] = "Rock",

    ["cast.pound"] = "Niederhämmern",
    ["cast.jump"] = "[UPJ] Room 3 - Boss - Leap [DNT]", --jump to, and from midphase location.

    ["alert.interrupt"] = "Unterbrechen!",

    ["label.mdiphase"] = "Mittelphase",
    ["label.meteor"] = "Meteor",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }

  Midphase{
    label = "label.mdiphase",
    text{},
  }

  Pound{
    label = "cast.pound",
    cast{},
    alert{},
    sound{file = "alert"},
  }

  Meteor{
    label = "label.meteor",
    line{thickness = 6, color = "green"},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
      [_arg.tUnit:GetHealth()] = {"~=", _arg(nil)},
    Unit("Boss", _arg.tUnit),
    HealthText("Midphase", _arg.tUnit, 66),
    --I dont have a trigger for when midphase ends, but there should be a healthtext for 33% be added.
  }

  CastStart{ strSpellName = "cast.jump",
    If{ [Count("Jumps")] = {">", 1}, --if we've done more than 1 jump yet (aka we atleast left the first midphase)
      HealthText("Midphase", _arg.tUnit, 33),
    }
  }

  CastStart{ strSpellName = "cast.pound",
    Cast("Pound", _arg.tCast),
    Alert("Pound", "alert.interrupt"),
    Sound("Pound"),
  }

  UnitCreated{ strUnitName = {"===", _arg.self.L["unit.meteor"]},
    Polygon("Meteor", _arg.tUnit, 4, 10) + {nDuration = 2.5},
  }
end)