local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter

local tZones = {
  [1] = {
    continentId = 14,    -- SC
    parentZoneId = 0,
    mapId = 20,
  },
}

-- ##########################################################################################################
-- # Tugga (1)
-- ##########################################################################################################


Encounter.new("SCTugga", {
  instance = "Dungeons",
  groupName = "SC",
  displayName = "(1) Tugga",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Stew-Shaman Tugga",

    ["cast.stew"] = "Into the Stew",
    ["cast.totem"] = "Magma Totem",
    ["cast.rain"] = "Molten Rain",

    ["label.cast"] = "Next Cast",
  },
  ["deDE"] = {
    ["unit.boss"] = "Suppenschamane Tugga",

    ["cast.stew"] = "In den Eintopf",
    ["cast.totem"] = "Magmatotem",
    ["cast.rain"] = "Geschmolzener Regen",

    ["label.cast"] = "Nächster Zauber",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Cast{
    label = "label.cast",
    timer{},
    text{ color="ffff0000" },
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    {"Unit", "Boss", _arg.tUnit},
  }

  CastStart{ strSpellName = {"cast.stew", "cast.totem", "cast.rain"},
    {"Timer", "Cast", 7},
    {"TimerText", "Cast", _arg.tCast.tUnit, 7},
  }
end)


-- ##########################################################################################################
-- # "Thunderfoot" (2)
-- ##########################################################################################################


Encounter.new("SCThunderfoot", {
  instance = "Dungeons",
  groupName = "SC",
  displayName = "(2) Thunderfoot",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Thunderfoot",

    ["cast.jump"] = "Pulverize",
    ["cast.beat"] = "Thunder Pound" ,

    ["alert.jump"] = "Jump! Jump! Jump!",

    ["label.jump"] = "Jump!",
  },
  ["deDE"] = {
    ["unit.boss"] = "Donnerhuf",

    ["cast.jump"] = "Pulverisieren",
    ["cast.beat"] = "Donnerhammer" ,

    ["alert.jump"] = "Spring! Spring! Spring!",

    ["label.jump"] = "Spring!",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Jump{
    label = "label.jump",
    cast{},
    sound{ file="alert" },
    alert{},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    {"Unit", "Boss", _arg.tUnit},
  }

  CastStart{ strSpellName = {"cast.jump", "cast.beat"},
    {"Cast", "Jump", _arg.tCast},
    {"Alert", "Jump", "alert.jump"},
    {"Sound", "Jump"},
  }
end)


-- ##########################################################################################################
-- # "Octog" (3)
-- ##########################################################################################################


Encounter.new("SCOctog", {
  instance = "Dungeons",
  groupName = "SC",
  displayName = "(3) Octog",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Bosun Octog",
    ["unit.bomb"] = "Monkey Mine",

    ["cast.shred"] = "Shred",
    ["cast.midphase"] = "Ink Shield",

    ["label.bomb"] = "Bomb",
    ["label.blind"] = "Blind",
    ["label.shred"] = "Shred",
    ["label.midphase"] = "Midphase",

--
  },
  ["deDE"] = {
    ["unit.boss"] = "Deckoffizier Octog",
    ["unit.bomb"] = "Affenmine",

    ["cast.shred"] = "Zerfetzen",
    ["cast.midphase"] = "Tintenschild",

    ["label.bomb"] = "Bombe",
    ["label.blind"] = "Blendung",
    ["label.shred"] = "Zerfetzen",
    ["label.midphase"] = "Mittelphase",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Bomb{
    label = "label.bomb",
    icon{sprite="LUIBM_bomb", size=80},
  }
  Blind{
    label = "label.blind",
    aura{sprite="LUIBM_waterdrop2"},
    icon{sprite="LUIBM_waterdrop2", size=80},
  }
  Shred{
    label = "label.shred",
    cast{},
  }
  Midphase{
    label = "label.midphase",
    timer{},
    text{},
    line{ thickness=4 },
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    Unit("Boss", _arg.tUnit),
    HealthText("Midphase", _arg.tUnit, 66.6),
  }
  UnitCreated{ strUnitName = "unit.bomb",
    Icon("Bomb", _arg.tUnit, _arg.nUnitId),
  }
  UnitDestroyed{ strUnitName = "unit.bomb",
    -Icon("Bomb", _arg.nUnitId),
  }

  CastStart{ strSpellName = "cast.shred",
    Cast("Shred", _arg.tCast),
  }

  local SE = {x=-628.5, y=-870.8, z=-963.4}
  local SW = {x=-690.3, y=-872.9, z=-980.0}
  local NW = {x=-673.8, y=-876.7, z=-1052}
  local NE = {x=-611.6, y=-873.0, z=-1034}

  CastStart{ strSpellName = "cast.midphase",
    Timer("Midphase", 38.5),
    TimerText("Midphase", _arg.tUnit, 38.5),

    LineBetween("Midphase", SE, SW, "S"),
    LineBetween("Midphase", SW, NW, "W"),
    LineBetween("Midphase", NW, NE, "N"),
    LineBetween("Midphase", NE, SE, "E"),
  }
  CastEnd{ strSpellName = "cast.midphase",
    -Timer("Midphase"),
    HealthText("Midphase", _arg.tUnit, 33.3),

    -LineBetween("Midphase", "S"),
    -LineBetween("Midphase", "W"),
    -LineBetween("Midphase", "N"),
    -LineBetween("Midphase", "E"),
  }

  BuffAdded{ nSpellId = 36613, [_arg.tUnit:IsThePlayer()] = true,
    Aura("Blind", _arg.nDuration)
  }
  BuffAdded{ nSpellId = 36613, [_arg.tUnit:IsThePlayer()] = false,
    Icon("Blind", _arg.tUnit, _arg.nUnitId)+{ nDuration=_arg.nDuration, overlay=true }
  }
  BuffRemoved{ nSpellId = 36613, [_arg.tUnit:IsThePlayer()] = true,
    -Aura("Blind")
  }
  BuffRemoved{ nSpellId = 36613, [_arg.tUnit:IsThePlayer()] = false,
    -Icon("Blind", _arg.nUnitId)
  }
end)

-- ##########################################################################################################
-- # "Mordechai" (4)
-- ##########################################################################################################


Encounter.new("SCMordechai", {
  instance = "Dungeons",
  groupName = "SC",
  displayName = "(4) Mordechai",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Mordechai Redmoon",
    ["unit.puddle"] = "Terraformer",
    ["unit.bullet"] = "Plasma Ball",
    ["unit.cannon"] = "Redmoon Chaingun",

    ["cast.bullet"] = "Plasma Ball",

    ["datachron.midphase"] = "Terraformer begins",
    ["datachron.blind"] = "is overcharging!",

    ["label.midphase"] = "Midphase",
    ["label.cannon"] = "Cannon",
    ["label.blind"] = "Blind",
  },
  ["deDE"] = {
    ["unit.boss"] = "Mordechai Rotmond",
    ["unit.puddle"] = "Terraformer",
    ["unit.bullet"] = "Plasmaball",
    ["unit.cannon"] = "Rotmonds Kettengeschütz",

    ["cast.bullet"] = "Plasmaball",

    ["datachron.midphase"] = "Terraformer beginnt",
    ["datachron.blind"] = "ist überlastet!",

    ["label.midphase"] = "Mittelphase",
    ["label.cannon"] = "Kanone",
    ["label.blind"] = "Blenden!",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    text{},
  }
  Midphase{
    label = "label.midphase",
    line{thickness = 4, enable = false},
    timer{},
  }
  Cannon{
    label = "label.cannon",
    line{},
  }
  Blind{
    label = "label.blind",
    timer{},
    text{color = "fffff0000"},
    alert{},
    sound{file = "alert"},
    cast{},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    Unit("Boss", _arg.tUnit),
  }


  local nHealthTarget = 70
  Enable{function() nHealthTarget=70 end}
  HealthReached{ nHealthReached = 70,
    function() nHealthTarget = 40 end,
  }

  v3Center = Vector3.New(-752.8, -692.7, -445.0)
  arPuddles = {}
  Enable{function() arPuddles={} end}
  UnitCreated{ strUnitName = "unit.puddle",
    {function(tUnit)
      if (Vector3.New(tUnit:GetPosition())-v3Center):Length() > 20 then
        table.insert(arPuddles, tUnit)
      end
    end, _arg.tUnit}
  }
  Datachron{ strMessage = "datachron.midphase",
    function(self)
      for i, tUnit in ipairs(arPuddles) do
        self:Polygon("Midphase", tUnit, 5, 10, i, {nDuration = 27})
      end
    end,
    Delay( 27,
      function(self)
        self:HealthText("Boss", self:GetUnit("Boss"), nHealthTarget, nil, {nOffset=-50})
      end
    ),
    -Timer("Blind"),
    -TimerText("Blind"),
    Timer("Midphase", 27),
  }


  UnitCreated{ strUnitName = "unit.cannon",
    TrackUnit(_arg.tUnit) --so that CastStart events are fired for this unit.
  }
  CastStart{ strSpellName = "cast.bullet",
    Line("Cannon", _arg.tUnit, 10, _arg.nUnitId)+{nOffset=4.5},
  }
  CastEnd{ strSpellName = "cast.bullet",
    -Line("Cannon", _arg.nUnitId)
  }
  UnitCreated{ strUnitName = "unit.bullet",
    Line("Cannon", _arg.tUnit, 10, _arg.nUnitId),
  }

  Datachron{ strMessage = "datachron.blind",
    Timer("Blind", 22.5),
    TimerText("Blind", _arg.self:GetUnit("Boss"), 22.5),
    Sound("Blind"),
    Alert("Blind"),
    Cast("Blind", 2.5),
  }
end)