local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter

local tZones = {
  [1] = {
    continentId = 48,    -- SSM
    parentZoneId = 0,
    mapId = 85,
  },
}

-- ##########################################################################################################
-- # Shallaos (1)
-- ##########################################################################################################


Encounter.new("SSMShallaos", {
  instance = "Dungeons",
  groupName = "SSM",
  displayName = "(1) Shallaos",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Deadringer Shallaos",

    ["cast.call"] = "Harmonic Call",
    ["cast.echo"] = "Echo",

    ["datachron.midphase"] = "Torine chimes",

    ["label.cast"] = "Next Echo or Harmonic Call",
    ["label.midphase"] = "Midphase",
    ["label.stacks"] = "Stacks",
  },
  ["deDE"] = {
    ["unit.boss"] = "Doppelgängerin Shallaos",

    ["cast.call"] = "Harmonischer Ruf",
    ["cast.echo"] = "Echo",

    ["datachron.midphase"] = "erzeugen ein sonores Brummen",

    ["label.cast"] = "Next Echo or Harmonic Call",
    ["label.midphase"] = "Midphase",
    ["label.stacks"] = "Stacks",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    line{thickness = 4, color = "ffff0000"},
  }
  Cast{
    label = "label.cast",
    timer{},
  }
  Midphase{
    label = "label.midphase",
    timer{},
    text{},
  }
  Stacks{
    label = "label.stacks",
    text{color = "ffffaa00"},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true,
    {"Unit", "Boss", _arg.tUnit},
    {"Line", "Boss", _arg.tUnit, 9, "LeftCleave", {nRotation = -80, nOffset = 0.5}},
    {"Line", "Boss", _arg.tUnit, 9, "RightCleave", {nRotation = 80, nOffset = 0.5}},
  }
  UnitDestroyed{ strUnitName = "unit.boss",
    {"RemoveUnit", "Boss"},
    {"RemoveLine", "Boss", "LeftCleave"},
    {"RemoveLine", "Boss", "RightCleave"},
  }

  CastStart{ strSpellName = {"cast.call", "cast.echo"},
    {"Timer", "Cast", 9},
  }

  Datachron{ strMessage = "datachron.midphase",
    {"Timer", "Midphase", 36},
    {"Text", "Midphase", _arg.self:GetUnit("Boss"), "", nil, {nDuration=36, timer=true}},
  }

  BuffAdded{ nSpellId = 63438, --Resonance Debuff (+Damage Stacks)
    {"Text", "Stacks", _arg.tUnit, _arg.nStack, _arg.nUnitId},
  }
  BuffUpdated{ nSpellId = 63438,
    {"Text", "Stacks", _arg.tUnit, _arg.nStack, _arg.nUnitId},
  }
  BuffRemoved{ nSpellId = 63438,
    {"RemoveText", "Stacks", _arg.nUnitId},
  }
end)

-- ##########################################################################################################
-- # Rayna (2A)
-- ##########################################################################################################

Encounter.new("SSMRayna", {
  instance = "Dungeons",
  groupName = "SSM",
  displayName = "(2A) Rayna",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Rayna Darkspeaker",

    ["cast.wave"] = "Molten Wave",

    ["datachron.leash"] = "Eine Flammenleine ist erschienen",

    ["label.midphase"] = "Midphase",
    ["label.wave"] = "Waves",
    ["label.smoldering"] = "Smoldering (DoT)",
  },
  ["deDE"] = {
    ["unit.boss"] = "Rayna Düsterbotin",

    ["cast.wave"] = "Welle",

    ["datachron.leash"] = "Eine Flammenleine ist erschienen",

    ["label.midphase"] = "Mittelphase",
    ["label.wave"] = "Wellen",
    ["label.smoldering"] = "Glühen (DoT)",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Smoldering{
    label = "label.smoldering",
    color = "ffff0000",
    timer{},
    text{},
  }
  Wave{
    label = "label.wave",
    line{},
  }
  Midphase{
    label = "label.midphase",
    timer{},
    text{},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  --the waves are also called 'unit.boss', but these are not in combat.
  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    {"Unit", "Boss", _arg.tUnit},
    HealthText("Midphase", _arg.tUnit, 70)+{nOffset=-50},
  }

  --the waves are also called 'unit.boss', so we need to check whether its the correct one.
  UnitDestroyed{ strUnitName = "unit.boss", tUnit = _arg.self:GetUnit("Boss"),
    {"RemoveUnit", "Boss"},
  }

  BuffAdded{ nSpellId = 63319, --Smoldering (Circle Debuffs)
    {"Timer", "Smoldering", 12},
    {"Text", "Smoldering", _arg.self:GetUnit("Boss"), "", nil, { timer = true, nDuration = 12}},
  }

  CastStart{  [_arg.tCast.tUnit:GetCastDuration()] = 32000, --MIDPHASE!
    {"Timer", "Midphase", 32},
    {"Text", "Midphase", _arg.self:GetUnit("Boss"), "", nil, { timer = true, nDuration = 32}},
    {"RemoveText", "Smoldering"},
  }
  CastEnd{  nDuration = 32, --MIDPHASE END!
    HealthText("Midphase", _arg.tUnit, 40)+{nOffset=-50},
  }

  UnitCreated{ strUnitName = "unit.boss", bInCombat = false,--these are the waves!
    {function(self, tWaveUnit)
      local tFacing = tWaveUnit:GetFacing()

      tFacing.x = -tFacing.x
      tFacing.y = 0
      tFacing.z = -tFacing.z

      self:Line("Wave", GameLib.GetPlayerUnit(), 10, nil, {
        tVectorFacing = tFacing,
        nDuration = 8,
        nOffset = 1,
      })

    end, _arg.self, _arg.tUnit}
  }
end)

-- ##########################################################################################################
-- # Skash (2B)
-- ##########################################################################################################

Encounter.new("SSMSkash", {
  instance = "Dungeons",
  groupName = "SSM",
  displayName = "(2B) Skash",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Moldwood Overlord Skash",

    ["cast.shockwave"] = "Tentacle Wrath",
    ["cast.runPhase"] = "Corruption Heartseeker",
    ["cast.eyes"] = "Corrultion Siphon ",

    ["label.runPhase"] = "Run Phase",
    ["label.shockwave"] = "Big Shockwave",
    ["label.center"] = "Center",
    ["label.eyes"] = "Eyes",
  },
  ["deDE"] = {
    ["unit.boss"] = "Moderwald-Oberherr Skash",

    ["cast.shockwave"] = "Tentakelzorn",
    ["cast.runPhase"] = "Korrumpierungsherzsucher",
    ["cast.eyes"] = "Korrumpierungsabsauger",

    ["label.runPhase"] = "Renn-Phase",
    ["label.shockwave"] = "Große Schockwelle",
    ["label.center"] = "Mitte",
    ["label.eyes"] = "Augen",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Center{
    label = "label.center",
    line{color = "ff888888"},
  }
  RunPhase{
    label = "label.runPhase",
    line{},
    timer{},
    text{color = "ffff0000"},
  }
  Shockwave{
    label = "label.shockwave",
    timer{},
  }
  Eyes{
    label = "label.eyes",
    timer{},
    alert{},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global
  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    {"Unit", "Boss", _arg.tUnit},
    {"Timer", "Shockwave", 12},
    {"Timer", "RunPhase", 25},
    {"Text", "RunPhase", _arg.tUnit, "", "Timer", {timer = true, nDuration = 25}},
  }

  local tMidphaseCenter = {x=0,y=0,z=0} --X Y Z WILL BE CHANGED! but the reference stays.
  UnitCreated{ strUnitName = "unit.boss", bInCombat = false,
    {function(tPos)
      local v3Center = Vector3.New(tPos) + Vector3.New(2.5, 0, -2.5)
      tMidphaseCenter.x = v3Center.x
      tMidphaseCenter.y = v3Center.y
      tMidphaseCenter.z = v3Center.z
    end,_arg.tUnit:GetPosition()},
    --the function above will always be executed first, so tMidphaseCenter will be changed by then.
    {"Polygon", "Center", tMidphaseCenter, 2, 8},
  }


  CastStart{ strSpellName = "cast.runPhase",
    {"Polygon", "RunPhase", tMidphaseCenter, 23, 20, nil, {nDuration = 16}},
    {"Timer", "RunPhase", 16},
    {"Text", "RunPhase", tMidphaseCenter, "", "Timer", {timer = true, nDuration = 16}},
  }

  CastEnd{ strSpellName = "cast.runPhase",
    {"Timer", "Shockwave", 7.5},
    {"Timer", "RunPhase", 42},
    {"Text", "RunPhase", _arg.tUnit, "", "Timer", {timer = true, nDuration = 42}},
    {"Timer", "Eyes", 28},
  }

  CastStart{ strSpellName = "cast.shockwave",
    {"Timer", "Shockwave", 20},
  }
  CastStart{ strSpellName = "cast.eyes",
    {"Alert", "Eyes"},
  }
end)

-- ##########################################################################################################
-- # Ondu (3)
-- ##########################################################################################################

Encounter.new("SSMOndu", {
  instance = "Dungeons",
  groupName = "SSM",
  displayName = "(3) Ondu",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Ondu Lifeweaver",

    ["cast.clover"] = "Plague Splatter",
    ["cast.midphase"] = "Creeping Blight",
    ["cast.heal"] = "Life Weave",

    ["datachron.enrage"] = "enraged!",
    ["datachron.armor"] = "skin becomes",

    ["label.clover"] = "Clover",
    ["label.nextPhase"] = "Next Phase",
    ["label.heal"] = "Heal",
  },
  ["deDE"] = {
    ["unit.boss"] = "Ondu Lebensflechter",

    ["cast.clover"] = "Seuchenspritzer",
    ["cast.midphase"] = "Kriechende Fäulnis",
    ["cast.heal"] = "Lebensgeflecht",

    ["datachron.enrage"] = "wird wütend!",
    ["datachron.armor"] = "Haut wird",

    ["label.clover"] = "Kleeblatt",
    ["label.nextPhase"] = "Nächste Phase",
    ["label.heal"] = "Heilung",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  Clover{
    label = "label.clover",
    timer{},
    text{color = "ffff0000"},
  }
  NextPhase{
    label = "label.nextPhase",
    timer{},
    text{},
  }
  Heal{
    label = "label.heal",
    cast{}
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    {"Unit", "Boss", _arg.tUnit},
    {"Text", "Clover", _arg.tUnit, "", nil, { timer = true, nDuration = 4}},
    {"Timer", "Clover", 4},
    {"Timer", "NextPhase", 30},
  }

  Datachron{ strMessage = {"datachron.enrage", "datachron.armor"},
    {"Text", "NextPhase", _arg.self:GetUnit("Boss"), "", nil, { timer = true, nDuration = 30}},
    {"Timer", "NextPhase", 30},
    {"RemoveText", "Clover"},
    {"RemoveTimer", "Clover"},
  }

  BuffAdded{ strUnitName = "unit.boss", nSpellId = 42914, --(heal)-Phase-Buff
    {"RemoveText", "NextPhase"},
    {"Timer", "NextPhase", 30},
  }

  CastStart{ strSpellName = "cast.midphase",
    {"Text", "NextPhase", _arg.tUnit, "", nil, { timer = true, nDuration = 25}},
    {"Timer", "NextPhase", 25},
    {"RemoveText", "Clover"},
    {"RemoveTimer", "Clover"},
  }

  CastStart{ strSpellName = "cast.clover",
    {"Text", "Clover", _arg.tUnit, "", nil, { timer = true, nDuration = 10}},
    {"Timer", "Clover", 10},
    {"RemoveText", "NextPhase"},
  }

  CastStart{ strSpellName = "cast.heal",
    {"Cast", "Heal", _arg.tCast},
  }
end)

-- ##########################################################################################################
-- # Selene (4)
-- ##########################################################################################################

Encounter.new("SSMSelene", {
  instance = "Dungeons",
  groupName = "SSM",
  displayName = "(4) Selene",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Spiritmother Selene the Corrupted",
    ["unit.light"] = "Luminous Sanctuary Target",
    ["unit.statue"] = "Darkness Effigy",

    ["cast.darkPhase"] = "Shadow Meld",

    ["datachron.darkPhase"] = "one with the",

    ["label.activeLight"] = "Active Light",
    ["label.inactiveLight"] = "Inactive Light",
    ["label.darkPhase"] = "Dark Phase",
    ["label.statues"] = "Statues (Midphase-Adds)",
    ["label.statueCounter"] = "Remaining Statues",
  },
  ["deDE"] = {
    ["unit.boss"] = "Seelenmutter Selene die Korrumpierte",
    ["unit.light"] = "Ziel: Leuchtende Zuflucht",
    ["unit.statue"] = "Dunkelheitsstatue",

    ["cast.darkPhase"] = "Schattenverschmelzung",

    ["datachron.darkPhase"] = "eins mit",

    ["label.activeLight"] = "Aktives Licht",
    ["label.inactiveLight"] = "Inaktives Licht",
    ["label.darkPhase"] = "Dunkelheits Phase",
    ["label.statue"] = "Statuen (Mittelphasen-Adds)",
    ["label.statueCounter"] = "Übrige Statuen",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
  }
  ActiveLight{
    label = "label.activeLight",
    line{color = "ff00ff00"},
  }
  InactiveLight{
    label = "label.inactiveLight",
    line{color = "ffaaaaaa"}
  }
  DarkPhase{
    label = "label.darkPhase",
    text{color = "ffff0000"},
    timer{},
  }
  Statue{
    label = "label.statue",
    text{color = "ffaaaaaa"},
  }
  StatueCounter{
    label = "label.statueCounter",
    text{color = "ffff0000"},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true,
    {"Unit", "Boss", _arg.tUnit},
  }

  -- ##### BlindPhase Adds: #####
  local tStatues = {}
  Enable{function() --reset with start of fight.
    tStatues = {}
  end}

  UnitCreated{ strUnitName = "unit.statue",
    {function(self, nId)
      for _, nStatueId in ipairs(tStatues) do
        if nStatueId == nId then
          return
        end
      end

      local nIdx = #tStatues+1
      tStatues[nIdx] = nId --basically a table.insert() - but we know where we put it.

      self:Text("Statue", nId, nIdx, nId)
      self:Text("StatueCounter", self:GetUnit("Boss"), #tStatues)
    end, _arg.self, _arg.nUnitId},
  }
  UnitDestroyed{ strUnitName = "unit.statue",
    {function(self, nId)
      local nStatue;
      for i, nStatueId in ipairs(tStatues) do
        if nStatueId == nId then
          nStatue = i
          break;
        end
      end

      if nStatue then
        table.remove(tStatues, nStatue)
      end

      self:RemoveText("Statue", nId)

      local nStatues = #tStatues
      if nStatues == 0 then
        self:RemoveText("StatueCounter")
      else
        self:Text("StatueCounter", self:GetUnit("Boss"), nStatues)
      end
    end, _arg.self, _arg.nUnitId},
  }


  -- ##### DarkPhase Variables: #####

  local bDarkPhase = false
  local tLights = {}
  Enable{function() --reset with start of fight.
    tLights = {}
    bDarkPhase = false
  end}

  UnitCreated{ strUnitName = "unit.light",
    {function(tUnit)
      table.insert(tLights, {
        tUnit = tUnit,
        tPos = {x=0,y=0,z=0},
        bActive = nil, --true/false/nil for Active/Inactive/Hidden aka. Green/Grey/Invis.
      })
    end, _arg.tUnit}
  }

  -- ##### DarkPhase Start&End Triggers: #####

  CastStart{ strSpellName = "cast.darkPhase", --this is 2sec previous to the below, but needs Localization
    function() bDarkPhase = true end,
  }
  Datachron{ strMessage = "datachron.darkPhase",
    {"Timer", "DarkPhase", 29},
    {"Text", "DarkPhase", _arg.self:GetUnit("Boss"), "", nil, { timer = true, nDuration = 29}},
    function() bDarkPhase = true end, --if the above was not triggered.
  }
  UnitCreated{ strUnitName = "unit.statue", --once the Statues are spawned, we dont need the circles anymore.
    function(self)
      bDarkPhase = false
      for nLight, tLight in ipairs(tLights) do
        tLight.bActive = nil
        self:RemovePolygon("InactiveLight", nLight)
        self:RemovePolygon("ActiveLight", nLight)
      end
    end,
  }

  -- ##### DarkPhase Circle Drawing: #####

  OnFrame{function(self)
    if bDarkPhase then
      for nLight, tLight in ipairs(tLights) do
        local tPos = tLight.tUnit:GetPosition()
        local bActive = true

        if tLight.bActive == nil then --assume Inactive! (start of Phase)
          bActive = false
        elseif tPos.x == tLight.tPos.x and tPos.z == tLight.tPos.z then --Is Stationary
          bActive = false
        end

        tLight.tPos = tPos
        if tLight.bActive ~= bActive then --redraw!
          tLight.bActive = bActive
          if bActive then
            self:RemovePolygon("InactiveLight", nLight)
            self:Polygon("ActiveLight", tLight.tUnit, 7, 10, nLight)
          else
            self:RemovePolygon("ActiveLight", nLight)
            self:Polygon("InactiveLight", tLight.tUnit, 7, 10, nLight)
          end
        end
      end
    end
  end}
end)