local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter

local tZones = {
  [1] = {
    continentId = 13,    -- STL
    parentZoneId = 0,
    mapId = 19,
  },
}

-- ##########################################################################################################
-- # Invoker (1)
-- ##########################################################################################################


Encounter.new("STLInvoker", {
  instance = "Dungeons",
  groupName = "STL",
  displayName = "(1) Invoker",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Blade-Wind the Invoker",
    ["unit.add"] = "Thundercall Channeler",

    ["cast.cross"] = "Thunder Cross",
    ["cast.pulse"] = "Electrostatic Pulse",

    ["datachron.midphaseStart"] = "The channelers have made The Invoker invulnerable!",
    ["datachron.midphaseEnd"] = "The Invoker has lost his protection!",

    ["label.cross"] = "Thunder Cross",
    ["label.pulse"] = "Electrostatic Pulse",
  },
  ["deDE"] = {
    ["unit.boss"] = "Klingenwind der Beschwörer",
    ["unit.add"] = "Donnerhall-Kanalisierer",

    ["cast.cross"] = "Donnerkreuz",
    ["cast.pulse"] = "Elektrostatischer Impuls",

    ["datachron.midphaseStart"] = "Die Kanalisierer haben den Vorsänger unverwundbar gemacht!",
    ["datachron.midphaseEnd"] = "Der Vorsänger hat keinen Schutz mehr!",

    ["label.cross"] = "Donnerkreuz",
    ["label.pulse"] = "Elektrostatischer Impuls",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    text{},
  }
  Cross{
    label = "label.cross",
    timer{},
    text{ color="ffff0000" },
  }
  Pulse{
    label = "label.pulse",
    timer{},
    text{ color="ffff0000"},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    Unit("Boss", _arg.tUnit),
    HealthText("Boss", _arg.tUnit, 60, nil,{nOffset = -50}),
    Timer("Cross", 5),
    TimerText("Cross", _arg.tUnit, 5),
  }

  local tAdds = {}
  Enable{function() tAdds = {} end}
  UnitCreated{ strUnitName = "unit.add",
    {function(nUnitId, tUnit) tAdds[nUnitId] = tUnit end, _arg.nUnitId, _arg.tUnit},
  }

  local bMidPhase = false
  Enable{function() bMidPhase=false end}
  local function fCrossTimer(self, nTimer, nTimerMid)
    nTimer = bMidPhase and nTimerMid or nTimer

    self:RemoveTimer("Cross")
    self:RemoveTimerText("Cross")
    for nUnitId, _ in pairs(tAdds) do
      self:RemoveTimerText("Cross", nUnitId)
    end

    if nTimer then
      self:Timer("Cross", nTimer)
      self:TimerText("Cross", self:GetUnit("Boss"), nTimer)

      if bMidPhase then
        for nUnitId, tUnit in pairs(tAdds) do
          self:TimerText("Cross", tUnit, nTimer, nUnitId)
        end
      end
    end
  end

  CastStart{ strSpellName = "cast.cross",
    {fCrossTimer, _arg.self, 12, 11},
  }

  Datachron{ strMessage = "datachron.midphaseStart",
    function() bMidPhase = true end,
    {fCrossTimer, _arg.self, 8, 8},
  }

  Datachron{ strMessage = "datachron.midphaseEnd",
    function() bMidPhase = false end,
    {fCrossTimer, _arg.self, nil, nil},
    Timer("Pulse", 6),
    TimerText("Pulse", _arg.self:GetUnit("Boss"), 6),
  }

  local nNextPulse = 0
  CastStart{ strSpellName = "cast.pulse",
    {function(self, tUnit)
      local t = GameLib.GetGameTime()
      if nNextPulse-t < 3 then
        nNextPulse = t+25
        self:Timer("Pulse", 25)
        self:TimerText("Pulse", tUnit, 25)
      end
    end, _arg.self, _arg.tUnit},
  }
end)

-- ##########################################################################################################
-- # Aethros (2)
-- ##########################################################################################################


Encounter.new("STLAethros", {
  instance = "Dungeons",
  groupName = "STL",
  displayName = "(2) Aethros",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Aethros",

    ["cast.circle"] = "Torrent",

    ["datachron.runPhaseStart"] = "A zephyr sweeps you off your feet!",

    ["label.star"] = "Stars",
    ["label.circle"] = "Circles",
  },
  ["deDE"] = {
    ["unit.boss"] = "Ätheros",

    ["cast.circle"] = "Strömung",

    ["datachron.runPhaseStart"] = "Ein Zephir holt dich von deinen Beinen!",

    ["label.star"] = "Sterne",
    ["label.circle"] = "Kreise",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    text{},
  }
  Star{
    label = "label.star",
    line{},
  }
  Circle{
    label = "label.circle",
    timer{},
    text{ color="ffff0000" },
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  --there are MANY other units called things like "Aethros Storm" and such. so we need an exact match.
  local bIsBossName = _arg.strUnitName:match("^".._arg.self.L["unit.boss"].."$")

  UnitCreated{ [bIsBossName] = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    {"Unit", "Boss", _arg.tUnit},
    {"HealthText", "Boss", _arg.tUnit, 80},
    {"Timer", "Circle", 10.5},
    {"TimerText", "Circle", _arg.tUnit, 10.5, nil, {nOffset=-50}},
  }

  CastStart{ strSpellName = "cast.circle",
    {"Timer", "Circle", 8},
    {"TimerText", "Circle", _arg.tCast.tUnit, 8, nil, {nOffset=-50}},
  }

  UnitCreated{ [bIsBossName] = "unit.boss", bInCombat = false,
    {"Line", "Star", _arg.tUnit, 10, _arg.nUnitId, {nRotation = -90}},
  }
end)


-- ##########################################################################################################
-- # Stormtalon (3)
-- ##########################################################################################################


Encounter.new("STLStormtalon", {
  instance = "Dungeons",
  groupName = "STL",
  displayName = "(3) Stormtalon",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = tZones,
  },
},{
  ["enUS"] = {
    ["unit.boss"] = "Stormtalon",

    ["cast.knockback"] = "Static Wave",

    ["datachron.storm"] = "ELECTRICITY FLOWS",

    ["label.knockback"] = "Knockback",
    ["label.storm"] = "Eye of the Storm",
  },
  ["deDE"] = {
    ["unit.boss"] = "Sturmklaue",

    ["cast.knockback"] = "Statische Welle",

    ["datachron.storm"] = "ELEKTRIZITÄT FLIESST",

    ["label.knockback"] = "Rückstoß",
    ["label.storm"] = "Auge des Sturms",
  },
  ["frFR"] = {},
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    text{},
  }
  Knockback{
    label = "label.knockback",
    timer{},
    text{ color="ffffbb00" }
  }
  Storm{
    label = "label.storm",
    timer{},
    text{ color="ffff0000" }
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    Unit("Boss", _arg.tUnit),
    HealthText("Boss", _arg.tUnit, 90),
  }

  CastStart{ strSpellName = "cast.knockback",
    Timer("Knockback", 36),
    TimerText("Knockback", _arg.tUnit, 36)+{nOffset = -50},
  }

  CastEnd{ strSpellName = "cast.knockback",
    HealthText("Boss", _arg.tUnit, 55),
  }

  HealthReached{ nHealthReached = 55, strUnitName = "unit.boss",
    -Timer("Knockback"),
    -TimerText("Knockback"),
  }

  Datachron{ strMessage = "datachron.storm",
    Timer("Storm", 35),
    TimerText("Storm", _arg.self:GetUnit("Boss"), 35)+{nOffset = -50},

    -Timer("Knockback"),
    -TimerText("Knockback"),
  }
end)