local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("ExperimentX89", {
  instance = "Genetic Archives",
  groupName = nil,
  displayName = "Experiment X-89",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 67,
        parentZoneId = 147,
        mapId = 148,
      },
    },
  },
},{
  ["enUS"] = {
    -- Units
    ["unit.boss"] = "Experiment X-89",
    -- Casts
    ["cast.shockwave"] = "Shattering Shockwave",
    ["cast.spew"] = "Repugnant Spew",
    ["cast.knockback"] = "Resounding Shout",
    -- Alerts
    ["alert.small_bomb"] = "Small Bomb on %s!",
    ["alert.small_bomb_player"] = "SMALL BOMB ON YOU! - RUN AWAY!",
    ["alert.big_bomb"] = "Big Bomb on %s!",
    ["alert.big_bomb_player"] = "BIG BOMB ON YOU! - JUMP OFF!",
    ["alert.knockback"] = "KNOCKBACK!",
    ["alert.spew"] = "SPEW!",
    -- Labels
    ["label.small_bomb"] = "Small Bomb",
    ["label.big_bomb"] = "Big Bomb",
    ["label.shockwave"] = "Shockwave",
    ["label.knockback"] = "Knockback",
    ["label.spew"] = "Spew",
  },
  ["deDE"] = {
    -- Units
    ["unit.boss"] = "Experiment X-89",
    -- Casts
    ["cast.shockwave"] = "Zerschmetternde Schockwelle",
    ["cast.spew"] = "Widerliches Erbrochenes",
    ["cast.knockback"] = "Widerhallender Schrei",
    -- Alerts
    ["alert.small_bomb"] = "Kleine Bombe auf %s!",
    ["alert.small_bomb_player"] = "KLEINE BOMBE AUF DIR! - LAUF WEG!",
    ["alert.big_bomb"] = "Große Bombe auf %s!",
    ["alert.big_bomb_player"] = "GROßE BOMBE AUF DIR! - SPRING HINUNTER!",
    ["alert.knockback"] = "KNOCKBACK!",
    ["alert.spew"] = "SPEW!",
    -- Labels
    ["label.small_bomb"] = "Kleine Bombe",
    ["label.big_bomb"] = "Große Bombe",
    ["label.shockwave"] = "Schockwelle",
    ["label.knockback"] = "Knockback",
    ["label.spew"] = "Spew",
  },
  ["frFR"] = {
    -- Units
    ["unit.boss"] = "Expérience X-89",
    -- Casts
    ["cast.shockwave"] = "Onde de choc dévastatrice",
    ["cast.spew"] = "Crachat répugnant",
    ["cast.knockback"] = "Hurlement retentissant",
    -- Alerts
    ["alert.small_bomb"] = "Petite bombe sur %s !",
    ["alert.small_bomb_player"] = "Petite bombe sur toi ! - Eloigne toi !",
    ["alert.big_bomb"] = "Grosse bombe sur %s !",
    ["alert.big_bomb_player"] = "Grosse Bombe sur toi ! - Saute dans le vide !",
    ["alert.knockback"] = "Repoussement !",
    ["alert.spew"] = "CRACHAT !",
    -- Labels
    ["label.small_bomb"] = "Petite bombe",
    ["label.big_bomb"] = "Grosse bombe",
    ["label.shockwave"] = "Onde de choc",
    ["label.knockback"] = "Repoussement",
    ["label.spew"] = "Crachat",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    line{thickness = 4, color = "ffff0000",}
  }
  SmallBomb{
    label = "label.small_bomb",
    alert{},
    sound{file = "run-away"},
    aura{sprite = "LUIBM_bomb", color = "ff00ff00"},
    icon{sprite = "LUIBM_bomb", size = 80, color = "ffffd700"},
  }
  BigBomb{
    label = "label.big_bomb",
    alert{},
    sound{file = "alert"},
    aura{sprite = "LUIBM_mushroom", color = "ffff0000"},
    icon{sprite = "LUIBM_mushroom", size = 80, color = "ffff0000"},
  }
  Shockwave{
    label = "label.shockwave",
    cast{},
  }
  Spew{
    label = "label.spew",
    cast{},
    alert{},
    sound{file = "info"},
  }
  Knockback{
    label = "label.knockback",
    cast{},
    alert{},
    sound{file = "long"},
  }
end, function(_arg, _env) --luacheck: no unused args
  -- luacheck: no global

  local ENUM_CLEAVE_ACTIVE = 1
  local ENUM_CLEAVE_INACTIVE = 2
  local ENUM_CLEAVE_SPEW = 3
  local strCleaveColorInactive = "ffababab"
  local function fDrawCleave(self, eCleave)
    local strColor = eCleave == ENUM_CLEAVE_INACTIVE and strCleaveColorInactive or nil
    local tUnit = self:GetUnit("Boss")

    if eCleave == ENUM_CLEAVE_SPEW then
      local tOffsetsL = Vector3.New(-4, 0, 0)
      local tOffsetsR = Vector3.New( 4, 0, 0)
      self:RemovePolygon("Boss", "Circle")
      self:Line("Boss", tUnit, 30, "CleaveL", {nRotation =  18, tVectorOffsets = tOffsetsL, color = strColor})
      self:Line("Boss", tUnit, 30, "CleaveR", {nRotation = -18, tVectorOffsets = tOffsetsR, color = strColor})
    else
      local tOffsetsL = Vector3.New(-7.5, 0, -3.669) -- -(9.5 - sqrt(34))
      local tOffsetsR = Vector3.New( 7.5, 0, -3.669)
      self:Polygon("Boss", tUnit, 9.5, 20, "Circle", {color = strColor})
      self:Line("Boss", tUnit, 30, "CleaveL", {nOffset = 9.5, tVectorOffsets = tOffsetsL, color = strColor})
      self:Line("Boss", tUnit, 30, "CleaveR", {nOffset = 9.5, tVectorOffsets = tOffsetsR, color = strColor})
    end
  end

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:GetUnit("Boss")] = _arg.Nil,
    Unit("Boss", _arg.tUnit),
    {fDrawCleave, _arg.self, ENUM_CLEAVE_INACTIVE},
  }

  local DEBUFF_SMALL_BOMB = 47316
  BuffAdded{ nSpellId = DEBUFF_SMALL_BOMB, [_arg.tUnit:IsThePlayer()] = true,
    Aura("SmallBomb", _arg.nDuration)+{strText="alert.small_bomb_player"},
    Alert("SmallBomb", "alert.small_bomb_player"),
    Sound("SmallBomb"),
  }
  BuffAdded{ nSpellId = DEBUFF_SMALL_BOMB, [_arg.tUnit:IsThePlayer()] = false,
    Icon("SmallBomb", _arg.tUnit)+{nDuration = _arg.nDuration},
    Alert("SmallBomb", _arg.self.L["alert.small_bomb"]:format(_arg.strUnitName)),
  }
  BuffRemoved{ nSpellId = DEBUFF_SMALL_BOMB,
    -Aura("SmallBomb"),
    -Icon("SmallBomb"),
  }

  local DEBUFF_BIG_BOMB = 47285
  BuffAdded{ nSpellId = DEBUFF_BIG_BOMB, [_arg.tUnit:IsThePlayer()] = true,
    Aura("BigBomb", _arg.nDuration)+{strText="alert.big_bomb_player"},
    Alert("BigBomb", "alert.big_bomb_player"),
    Sound("BigBomb"),
  }
  BuffAdded{ nSpellId = DEBUFF_BIG_BOMB, [_arg.tUnit:IsThePlayer()] = false,
    Icon("BigBomb", _arg.tUnit)+{nDuration = _arg.nDuration},
    Alert("BigBomb", _arg.self.L["alert.big_bomb"]:format(_arg.strUnitName)),
  }
  BuffRemoved{ nSpellId = DEBUFF_BIG_BOMB,
    -Aura("BigBomb"),
    -Icon("BigBomb"),
  }

  CastStart{ strSpellName = "cast.shockwave",
    Cast("Shockwave", _arg.tCast),
    {fDrawCleave, _arg.self, ENUM_CLEAVE_ACTIVE},
  }
  CastEnd{ strSpellName = "cast.shockwave",
    {fDrawCleave, _arg.self, ENUM_CLEAVE_INACTIVE},
  }

  CastStart{ strSpellName = "cast.spew",
    Cast("Spew", _arg.tCast),
    Alert("Spew", "alert.spew"),
    Sound("Spew"),
    {fDrawCleave, _arg.self, ENUM_CLEAVE_SPEW},
  }
  CastEnd{ strSpellName = "cast.spew",
    {fDrawCleave, _arg.self, ENUM_CLEAVE_INACTIVE},
  }

  CastStart{ strSpellName = "cast.knockback",
    Cast("Knockback", _arg.tCast),
    Alert("Knockback", "alert.knockback"),
    Sound("Knockback"),
  }
end)
