local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


Encounter.new("PhageMaw", {
  instance = "Genetic Archives",
  displayName = "Phage Maw",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 67,
        parentZoneId = 147,
        mapId = 149,
      },
    },
  },
},{
  ["enUS"] = {
    -- Unit names.
    ["unit.boss"] = "Phage Maw",
    ["unit.bomb"] = "Detonation Bomb",
    -- Datachron messages.
    ["datachron.shield_destroyed"] = "The augmented shield has been destroyed",
    ["datachron.orbital_strike"] = "Phage Maw begins charging an orbital strike",
    -- Timer bars.
    ["message.bomb_wave1"] = "Bombs wave #1",
    ["message.bomb_wave2"] = "Bombs wave #2",
    ["message.bomb_wave3"] = "Bombs wave #3",
    ["message.bomb_timeout"] = "Timeout all bombs!",
    ["message.bomb_name"] = "Bomb #%d",
    --Labels
    ["label.bombId"] = "Bomb Numbers",
    ["label.bomb_line"] = "Bomb Lines",
  },
  ["deDE"] = {
    -- Unit names.
    ["unit.boss"] = "Phagenschlund",
    ["unit.bomb"] = "Sprengbombe",
    -- Datachron messages.
    ["datachron.shield_destroyed"] = "Der augmentierte Schild wurde zerstört",
    ["datachron.orbital_strike"] = "Phagenschlund beginnt einen Orbitalschlag aufzuladen",
    -- Timer bars.
    ["message.bomb_wave1"] = "Bombenwelle Nr.1",
    ["message.bomb_wave2"] = "Bombenwelle Nr.2",
    ["message.bomb_wave3"] = "Bombenwelle Nr.3",
    ["message.bomb_timeout"] = "Timeout aller Bomben!",
    ["message.bomb_name"] = "Bombe Nr.%d",
    --Labels
    ["label.bombId"] = "Bomben Nummerierung",
    ["label.bomb_line"] = "Bomben Linien",
  },
  ["frFR"] = {
    -- Unit names.
    ["unit.boss"] = "Phagegueule",
    ["unit.bomb"] = "Bombe à détonateur",
    -- Datachron messages.
    ["datachron.shield_destroyed"] = "Le bouclier augmenté a été détruit",
    ["datachron.orbital_strike"] = "La Méga Gueule d'acier commence à charger une frappe orbitale",
    -- Timer bars.
    ["message.bomb_wave1"] = "Vague de bombes n°1",
    ["message.bomb_wave2"] = "Vague de bombes n°2",
    ["message.bomb_wave3"] = "Vague de bombes n°3",
    ["message.bomb_timeout"] = "Timeout toutes les bombes!",
    ["message.bomb_name"] = "Bomb #%d",
    --Labels
    ["label.bombId"] = "Bomb Numbers",
    ["label.bomb_line"] = "Bomb Lines",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{position = 0}, --bombs are positions 1, 2, 3, ...
  }
  Bomb{
    label = "unit.bomb",
    unit{color = "ffa52a2a"},
    text{color = "ffffffff"},
    line{enable = false, thickness = 10, color = "ff2dfe54"},
  }

  BombWave1{
    label = "message.bomb_wave1",
    timer{position = 1, color = "ffa00498"},
  }
  BombWave2{
    label = "message.bomb_wave2",
    timer{position = 2, color = "ff5a06ef"},
  }
  BombWave3{
    label = "message.bomb_wave3",
    timer{position = 3, color = "ff36013f"},
  }
  BombTimeout{
    label = "message.bomb_timeout",
    timer{position = 4, color = "ff980002"},
  }
end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
  }

  Datachron{ strMessage = "datachron.shield_destroyed",
    Timer("BombWave1", 20),
    Timer("BombWave2", 49),
    Timer("BombWave3", 78),
    Timer("BombTimeout", 104),
  }

  _var.nBombCounter = 0
  UnitCreated{ strUnitName = "unit.bomb", [_arg.self:HasUnit("Bomb", _arg.nUnitId)] = false,
    _var.nBombCounter+1,
    Unit("Bomb", _arg.tUnit) % (_arg.nUnitId) + {position = _var.nBombCounter, strMark = _var.nBombCounter},
    Text("Bomb", _arg.tUnit, _var.nBombCounter) % (_arg.nUnitId),
    LineBetween("Bomb", _arg.tUnit, nil) % (_arg.nUnitId)
  }

  Datachron{ strMessage = "datachron.orbital_strike",
    _var.nBombCounter(0),
  }

end)