local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter

local Mod = Encounter.new("Octog", {
  instance = "Redmoon Terror",
  displayName = "Octog",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 104,
        parentZoneId = 0,
        mapId = 548,
      },
    },
  },
},{
  ["enUS"] = {
      -- Units
      ["unit.boss"] = "Star-Eater the Voracious",
      ["unit.shard"] = "Astral Shard",
      ["unit.squirgling"] = "Squirgling",
      ["unit.chaos_orb"] = "Chaos Orb",
      ["unit.noxious_ink_pool"] = "Noxious Ink Pool",
      -- Casts
      ["cast.hookshot"] = "Hookshot",
      ["cast.summon_squirglings"] = "Summon Squirglings",
      ["cast.flamethrower"] = "Flamethrower",
      ["cast.supernova"] = "Supernova",
      -- Alerts
      ["alert.hookshot"] = "HOOKSHOT!",
      ["alert.flamethrower"] = "FLAMETHROWER!",
      ["alert.chaos_tether"] = "STAY INSIDE CHAOS ORB!!!",
      -- Labels
      ["label.next_hookshot"] = "Next Hookshot",
      ["label.next_flamethrower"] = "Next Flamethrower",
      ["label.next_orbs"] = "Next Orbs",
      ["label.chaos_tether"] = "Chaos Tether",
      ["label.midphase"] = "Midphase",
  },
  ["deDE"] = {
      ["unit.boss"] = "Sternenschlinger der Gefräßige", --V (Verified)
      ["unit.shard"] = "Astral Shard",
      ["unit.squirgling"] = "Squirgling",
      ["unit.chaos_orb"] = "Chaos Orb",
      ["unit.noxious_ink_pool"] = "Noxious Ink Pool",
      -- Casts
      ["cast.hookshot"] = "Hookshot",
      ["cast.summon_squirglings"] = "Summon Squirglings",
      ["cast.flamethrower"] = "Flamethrower",
      ["cast.supernova"] = "Supernova",
      -- Alerts
      ["alert.hookshot"] = "HOOKSHOT!",
      ["alert.flamethrower"] = "FLAMMENWERFER!",
      ["alert.chaos_tether"] = "BLEIB IM CHAOS ORB!!!",
      -- Labels
      ["label.next_hookshot"] = "Nächster Hookshot",
      ["label.next_flamethrower"] = "Nächster Flammenwerfer",
      ["label.next_orbs"] = "Nächste Orbs",
      ["label.chaos_tether"] = "Chaos Tether",
      ["label.midphase"] = "Mittelphase",
  },
  ["frFR"] = {
      -- Units
      ["unit.boss"] = "Dévore-Astre le Vorace", --V (Verified)
      ["unit.shard"] = "Astral Shard",
      ["unit.squirgling"] = "Squirgling", --V
      ["unit.chaos_orb"] = "Chaos Orb", --V
      ["unit.noxious_ink_pool"] = "Noxious Ink Pool",
      -- Casts
      ["cast.hookshot"] = "Hookshot", --V
      ["cast.summon_squirglings"] = "Summon Squirglings",
      ["cast.flamethrower"] = "Flamethrower",
      ["cast.supernova"] = "Supernova",
      -- Alerts
      ["alert.hookshot"] = "HOOKSHOT!",
      ["alert.flamethrower"] = "LANCE-FLAMMES !",
      ["alert.chaos_tether"] = "RESTE DANS L'ORBE DU CHAOS !!!",
      -- Labels
      ["label.next_hookshot"] = "Prochain Hookshot",
      ["label.next_flamethrower"] = "Prochain Lance-Flammes",
      ["label.next_orbs"] = "Prochaines Orbs",
      ["label.chaos_tether"] = "Ancre du Chaos",
      ["label.midphase"] = "Midphase",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    unit{},
    line{thickness = 6, color = "ffff0000"},
  }

  Squirgling{
    label = "unit.squirgling",
    line{thickness = 6, color = "ff00ffff"},
  }

  Shard{
    label = "unit.shard",
    line{thickness = 6, color = "ff00ffff"},
  }

  Orb{
    label = "label.chaos_tether",
    color = "ff9932cc",
    timer{},
    aura{sprite = "LUIBM_position"},
    alert{},
    sound{file = "beware"},
  }

  Hookshot{
    label = "cast.hookshot",
    color = "ffffd700",
    timer{},
    alert{},
    sound{file = "burn"},
  }

  Midphase{
    label = "label.midphase",
    timer{color = "ff00ffff"},
    text{},
    aler{},
    sound{file="beware"},
  }

  Flamethrower{
    label = "cast.flamethrower",
    color = "ffff4500",
    timer{},
    alert{},
    sound{file = "inferno"},
  }

end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global
  local GameTime = _env.GameLib.GetGameTime

  UnitCreated{ strUnitName = "unit.boss", bInCombat = true, [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
    Polygon("Boss", _arg.tUnit, 10.5, 20) % ("Hookshot1") + {tVectorOffsets = Vector3.New( 3,0,9.25)},
    Polygon("Boss", _arg.tUnit, 10.5, 20) % ("Hookshot2") + {tVectorOffsets = Vector3.New(-3,0,9.25)},
  }

  UnitCreated{ strUnitName = "unit.squirgling",
    LineBetween("Squirgling", _arg.tUnit, nil) % (_arg.nUnitId),
  }
  UnitDestroyed{ strUnitName = "unit.squirgling",
    -LineBetween("Squirgling") % (_arg.nUnitId),
  }

  _var.tShardTimer = nil
  _var.tShardIds = {}
  UnitCreated{ strUnitName = "unit.shard", [_arg.self.config.lines.Shard.enable] = true,
    {function(self, nUnitId)

      self.tVars.tShardIds[nUnitId] = false

    end, _arg.self, _arg.nUnitId},
    If{ [_var.tSharTimer] = _arg(nil),
      _var.tShardTimer(_env.ApolloTimer.Create(.1, true, "CheckShardsTimer", _arg.self))
    }
  }
  UnitDestroyed{ strUnitName = "unit.shard",
    -LineBetween("Shard") % (_arg.nUnitId),
    {function(self, nUnitId)

      self.tVars.tShardIds[nUnitId] = nil

    end, _arg.self, _arg.nUnitId},
  }


  local BUFF_PLACING_ORB = 86876 --BUFF_CHAOS_AMPLIFIER
  _var.nNextOrbs = GameTime()+48
  BuffAdded{ nSpellId = BUFF_PLACING_ORB,
    Timer("Orb", 80),
    _var.nNextOrbs(GameTime()+80)
  }
  BuffUpdated{ nSpellId = BUFF_PLACING_ORB,
    Timer("Orb", 85),
    _var.nNextOrbs(GameTime()+85)
  }

  local DEBUFF_ORB_CIRCLE = 85583 --DEBUFF_CHAOS_TETHER
  BuffAdded{ nSpellId = DEBUFF_ORB_CIRCLE, [_arg.tUnit:IsThePlayer()] = true,
    Aura("Orb", nil),
    Alert("Orb", "alert.chaos_tether"),
    Sound("Orb"),
  }
  BuffRemoved{ nSpellId = DEBUFF_ORB_CIRCLE, [_arg.tUnit:IsThePlayer()] = true,
    -Aura("Orb"),
  }

  CastStart{ strSpellName = "cast.hookshot",
    Timer("Hookshot", 30),
    Alert("Hookshot", "alert.hookshot"),
    Sound("Hookshot"),
  }

  CastStart{ strSpellName = "cast.supernova",
    -Timer("Hookshot"),
    -Timer("Flamethrower"),
    -Timer("Orb"),
    Timer("Midphase", _arg.nDuration),
  }

  CastEnd{ strSpellName = "cast.supernova",
    -Timer("Midphase"),
    If{ [_var.tShardTimer] = {"~=", _arg(nil)},
      _var.tShardTimer:Stop(),
      _var.tShardTimer(nil)
    },
  }
  CastEnd{ strSpellName = "cast.supernova", [_var.nNextOrbs] = {"<", GameTime()+10},
    Timer("Orb", 10),
  }
  CastEnd{ strSpellName = "cast.supernova", [_var.nNextOrbs] = {">=", GameTime()+10},
    Timer("Orb", _var.nNextOrbs),
  }

  Disable{
    If{ [_var.tShardTimer] = {"~=", _arg(nil)},
      function(self) self.tVars.tShardTimer:Stop() end,
      _var.tShardTimer(nil)
    }
  }

  Enable{
    Timer("Hookshot", 10),
    Timer("Flamethrower", 34),
    Timer("Orb", 48),
    _var.nNextOrbs(GameTime()+48)
  }



  CastStart{ strSpellName = "cast.flamethrower",
    Timer("Flamethrower", 45),
    Alert("Flamethrower", "alert.flamethrower"),
    Sound("Flamethrower"),
  }

  HealthReached{ strUnitName = "unit.boss", nHealthReached = {67, 37, 7},
    Alert("Midphase", "alert.midphase"),
    Sound("Midphase"),
  }
  HealthReached{ strUnitName = "unit.boss", nHealthReached = {70, 40, 10},
    HealthText("Midphase", _arg.tUnit, _arg.nHealthReached-5),
  }


end)

local ROCKET_HEIGHT = 20
local ROOM_FLOOR_Y = 378

local Vector2 = require "Vector2"
local Vector3 = require "Vector3"
local GameLib = require "GameLib"

function Mod:CheckShardsTimer()
  local v3PlayerPos = Vector3.New(GameLib.GetPlayerUnit():GetPosition())
  if not self.tVars.tShardIds then return end
  for nShardId, bHasLine in pairs(self.tVars.tShardIds) do
      local unit = GameLib.GetUnitById(nShardId)
      if unit then
          local v3ShardPos = Vector3.New(unit:GetPosition())
          local fHorDist = self:HorizontalDistance(v3PlayerPos, v3ShardPos)
          local bIsClose = fHorDist < 20
          if v3ShardPos.y + 5 < v3PlayerPos.y then
              -- Don't draw lines to shards far below than player
              self:RemoveLineBetween("Shard", nShardId)
              self.tVars.tShardIds[nShardId] = false
          elseif bIsClose and v3ShardPos.y < (v3PlayerPos.y + ROCKET_HEIGHT + 2) then
              -- Draw lines to shards player can reach with rocket boost
              if not bHasLine then
                  if v3ShardPos.y <= ROOM_FLOOR_Y + 40 then
                      local v3ShardPos_Lower = Vector3.New(v3ShardPos)
                      v3ShardPos_Lower.y = ROOM_FLOOR_Y --v3ShardPos_Lower.y - 42
                      self:LineBetween("Shard", v3ShardPos, v3ShardPos_Lower, nShardId)
                  else
                      self:LineBetween("Shard", v3ShardPos, nil, nShardId)
                  end
                  self.tVars.tShardIds[nShardId] = true
              end
          else
              -- Don't draw lines to shards the player can't reach
              self:RemoveLineBetween("Shard", nShardId)
              self.tVars.tShardIds[nShardId] = false
          end
      else
          self.tVars.tShardIds[nShardId] = nil
      end
  end
end
function Mod:HorizontalDistance(pos1, pos2) --luacheck: ignore self
  return (Vector2.New(pos1.x, pos1.z) - Vector2.New(pos2.x, pos2.z)):Length()
end
