local Apollo = require "Apollo"
local Vector3 = require "Vector3"
local GameLib = require "GameLib"

local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


local Mod = Encounter.new("Starface", {
  instance = "Redmoon Terror",
  displayName = "Starmap",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.boss"},
    tZones = {
      [1] = {
        continentId = 104,
        parentZoneId = 548,
        mapId = 556,
      },
    },
  },
},{
  ["enUS"] = {
      -- Units
    ["unit.boss"] = "Alpha Cassus",
    ["unit.aldinari"] = "Aldinari",
    ["unit.cassus"] = "Cassus",
    ["unit.vulpes_nix"] = "Vulpes Nix",
    ["unit.rogue_asteroid"] = "Rogue Asteroid",
    ["unit.debris_field"] = "Debris Field",
    ["unit.world_ender"] = "World Ender",
    ["unit.pulsar"] = "Pulsar",
    ["unit.cosmic_debris"] = "Cosmic Debris",
    ["unit.black_hole"] = "Black Hole",
    ["unit.wormhole"] = "Wormhole",
    -- Casts
    ["cast.solar_flare"] = "Solar Flare",
    ["cast.midphase"] = "Catastrophic Solar Event",
    -- Alert
    ["alert.world_ender"] = "World Ender spawned!",
    ["alert.pulsar"] = "Pulsar spawned!",
    ["alert.solar_winds"] = "Reset your stacks",
    ["alert.midphase"] = "Midphase soon, reset stacks",
    ["alert.critical_mass"] = "Critical Mass on %s (%d)",
    -- Labels
    ["label.world_ender"] = "World ender spawn points",
    ["label.aldinari_sun"] = "Aldinari to Sun",
    ["label.cassus_sun"] = "Cassus to Sun",
    ["label.vulpes_nix_sun"] = "Vulpes nix to Sun",
    ["label.planet_orbit_aldinari"] = "Planet orbit Aldinari",
    ["label.planet_orbit_cassus"] = "Planet orbit Cassus",
    ["label.planet_orbit_vulpes_nix"] = "Planet orbit Vulpes Nix",
    ["label.planet_orbit"] = "Planet Orbit",
    ["label.world_ender_player"] = "World Ender to player",
    ["label.world_ender_direction"] = "World Ender direction",
    ["label.world_ender_position"] = "World Ender position",
    ["label.wormhole"] = "Wormhole position",
    ["label.rogue_asteroid_player"] = "Rogue Asteroid to player",
    ["label.rogue_asteroid_direction"] = "Rogue Asteroid direction",
    ["label.cardinal"] = "Cardinal directions",
    ["label.solar_winds"] = "Solar Winds warning at 7 stacks",
    ["label.sun_stack_cast"] = "Every other Solar Flare (Tanks/Collectors)",
    ["label.irradiated_armor"] = "Irradiated Armor stack (Tanks/Collectors)",
    ["label.midphase"] = "Midphase warning",
    ["label.solar_winds_timer"] = "Solar Wind debuff timer",
    ["label.world_ender#"] = "World Ender #%d",
    ["label.asteroids"] = "Asteroids (%s)",
    ["label.cosmic_debris_line"] = "Cosmic Debris Line",
    ["label.cosmic_debris_polygon"] = "Cosmic Debris Outline",
    ["label.critical_mass"] = "Critical Mass",

    ["label.N"] = "N",
    ["label.E"] = "E",
    ["label.S"] = "S",
    ["label.W"] = "W",
  },
  ["deDE"] = {
    -- Units
    ["unit.boss"] = "Alpha-Cassus",
    ["unit.aldinari"] = "Aldinari",
    ["unit.cassus"] = "Cassus",
    ["unit.vulpes_nix"] = "Vulpes Nix",
    ["unit.rogue_asteroid"] = "Rogue Asteroid",
    ["unit.debris_field"] = "Trümmerfeld",
    ["unit.world_ender"] = "World Ender",
    ["unit.pulsar"] = "Pulsar",
    ["unit.cosmic_debris"] = "Cosmic Debris",
    ["unit.black_hole"] = "Black Hole",
    ["unit.wormhole"] = "Wormhole",
    -- Casts
    ["cast.solar_flare"] = "Solar Flare",
    ["cast.midphase"] = "Katastrophales Sonnenereignis",
    -- Alert
    ["alert.world_ender"] = "World Ender erschienen!",
    ["alert.pulsar"] = "Pulsar erschienen!",
    ["alert.solar_winds"] = "Stacks zurücksetzen!",
    ["alert.midphase"] = "Mittelphase bald! Stacks zurücksetzen!",
    ["alert.critical_mass"] = "Kritische Masse auf %s (%d)",
    -- Labels
    ["label.world_ender"] = "World Ender Spawnpunkte",
    ["label.aldinari_sun"] = "Aldinari zur Sonne",
    ["label.cassus_sun"] = "Cassus zur Sonne",
    ["label.vulpes_nix_sun"] = "Vulpes nix zur Sonne",
    ["label.planet_orbit_aldinari"] = "Umlaufbahn Aldinari",
    ["label.planet_orbit_cassus"] = "Umlaufbahn Cassus",
    ["label.planet_orbit_vulpes_nix"] = "Umlaufbahn Vulpes Nix",
    ["label.planet_orbit"] = "Umlaufbahn der Planeten",
    ["label.world_ender_player"] = "World Ender zum Spieler",
    ["label.world_ender_direction"] = "World Ender Flugrichtung",
    ["label.world_ender_position"] = "World Ender Position",
    ["label.wormhole"] = "Wurmloch Position",
    ["label.rogue_asteroid_player"] = "Rogue Asteroid zum Spieler",
    ["label.rogue_asteroid_direction"] = "Rogue Asteroid Flugrichtung",
    ["label.cardinal"] = "Himmelsrichtungen",
    ["label.solar_winds"] = "Stackwarnung bei 7 Stacks 'Solar Flare'",
    ["label.sun_stack_cast"] = "Jede zweite Solar Flare (Tanks/Collectors)",
    ["label.irradiated_armor"] = "Rüstungs stacks (Tanks/Collectors)",
    ["label.midphase"] = "Mittelphasen Warnung",
    ["label.solar_winds_timer"] = "Solar Wind debuff timer",
    ["label.world_ender#"] = "World Ender #%d",
    ["label.asteroids"] = "Astereoiden (%s)",
    ["label.cosmic_debris_line"] = "Cosmic Debris zum Spieler",
    ["label.cosmic_debris_polygon"] = "Cosmic Debris Umrandung",
    ["label.critical_mass"] = "Kritische Masse",

    ["label.N"] = "N",
    ["label.E"] = "O",
    ["label.S"] = "S",
    ["label.W"] = "W",
  },
  ["frFR"] = {


    ["label.N"] = "N",
    ["label.E"] = "E",
    ["label.S"] = "S",
    ["label.W"] = "W",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global
  Boss{
    label = "unit.boss",
    color = "ffff8900",
    unit{},
    line{label = "label.sun_stack_cast"},
  }
  Aldinari{
    label = "unit.aldinari",
    color = "ff800080",
    unit{},
    line{thickness = 6, label = "label.aldinari_sun", enable = false},
  }
  Cassus{
    label = "unit.cassus",
    color = "ff00bfff",
    unit{},
    line{thickness = 6, label = "label.cassus_sun", enable = false},
  }
  VulpesNix{
    label = "unit.vulpes_nix",
    color = "ffff8c00",
    unit{},
    line{thickness = 6, label = "label.vulpes_nix_sun", enable = false},
  }
  Pulsar{
    label = "unit.pulsar",
    unit{},
    alert{color = "ffff0000"},
    sound{file = "alert"},
  }
  WorldEnder{
    label = "unit.world_ender",
    color = "ffff0000",
    timer{},
    text{label = "label.world_ender_position"},
    unit{},
    alert{},
    sound{file = "beware"},
    line{thickness = 6, label = "label.world_ender_player"},
  }
  Asteroid{
    label = "unit.rogue_asteroid",
    color = "ffff1493",
    timer{},
    alert{},
    unit{enable = false},
    line{label = "label.rogue_asteroid_player", thickness = 6, enable = false},
    text{},
  }
  PlanetOrbit{
    label = "label.planet_orbit",
    line{color = false, thickness = 4, enable = false},
  }
  Cardinal{
    label = "label.cardinal",
    text{color = "ffff4500"},
  }
  CriticalMass{
    label = "label.critical_mass",
    aura{sprite = "LUIBM_atom"},
    alert{},
    sound{file = "info"},
  }
  DebrisField{
    label = "unit.debris_field",
    text{color = "ffff0000"}
  }
  DebrisPolygon{
    label = "label.cosmic_debris_polygon",
    line{thickness = 5, color = "ffff8c00"}
  }
  DebrisLine{
    label = "label.cosmic_debris_line",
    line{thickness = 5, color = "ffff8c00", enable = false}
  }
  BlackHole{
    label = "unit.black_hole",
    unit{},
  }
  SolarWinds{
    label = "alert.solar_winds",
    alert{color = "ffff0000"},
    sound{file = "run-away"},
    text{color = "ffffffff"},
  }

  Midphase{
    label = "label.midphase",
    text{},
    alert{color = "ffff0000"},
    sound{file = "long"},
  }

end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global
  local ASTEROID_NAME = _arg(setmetatable({
    [1] = "B", --the name for the upcoming, if current is 1.
    [2] = "C",
    [3] = "A",
  }, {__index=function(t,k)
    local v = rawget(t,((k-1)%3)+1)
    rawset(t,k,v)
    return v
  end}))
  _var.nAsteroidWaveCounter = 0
  Callback("AsteroidWave1Spawned", {
    Alert("Asteroid", _arg.self.L["label.asteroids"]:format(ASTEROID_NAME[_var.nAsteroidWaveCounter])),
    _var.nAsteroidWaveCounter+1,
    Timer("Asteroid", 26, _arg.self.L["label.asteroids"]:format(ASTEROID_NAME[_var.nAsteroidWaveCounter])) + {
      fHandler = "AsteroidWave2Spawned",
    },
  })
  Callback("AsteroidWave2Spawned", {
    Alert("Asteroid", _arg.self.L["label.asteroids"]:format(ASTEROID_NAME[_var.nAsteroidWaveCounter])),
    _var.nAsteroidWaveCounter+1,
    Timer("Asteroid", 52, _arg.self.L["label.asteroids"]:format(ASTEROID_NAME[_var.nAsteroidWaveCounter])) + {
      fHandler = "AsteroidWave1Spawned",
    },
  })
  Callback("WorldEnderSpawned", {
    Timer("Asteroid", 26, _arg.self.L["label.asteroids"]:format(ASTEROID_NAME[_var.nAsteroidWaveCounter])) + {
      fHandler = "AsteroidWave1Spawned",
    },
    {function(self, nWorldEnder)
      self:Timer("WorldEnder", 78, self.L["label.world_ender#"]:format(nWorldEnder+1))
    end, _arg.self, _var.nWorldEnderCount}
  })

  UnitCreated{ strUnitName = "unit.boss", [_arg.self:HasUnit("Boss")] = false,
    Unit("Boss", _arg.tUnit),
  }

  local v3Center = Vector3.New(-76.779, -96.22, 356.814)
  Enable{
    --Ender Timer: 52s
    Timer("WorldEnder", 52, _arg.self.L["label.world_ender#"]:format(1)),
    --AsteroidTimer: 26s
    FireCallbacks("AsteroidWave1Spawned"),

    Polygon("PlanetOrbit", v3Center, 16, 40) % ("Aldinari_Inner")  + {color = "ff800080"},
    Polygon("PlanetOrbit", v3Center, 23, 40) % ("Aldinari_Outer")  + {color = "ff800080"},
    Polygon("PlanetOrbit", v3Center, 36, 50) % ("Cassus_Inner")    + {color = "ff00bfff"},
    Polygon("PlanetOrbit", v3Center, 44, 50) % ("Cassus_Outer")    + {color = "ff00bfff"},
    Polygon("PlanetOrbit", v3Center, 50, 60) % ("VulpesNix_Inner") + {color = "ffff8c00"},
    Polygon("PlanetOrbit", v3Center, 63, 60) % ("VulpesNix_Outer") + {color = "ffff8c00"},

    Text("Cardinal", Vector3.New(-76.75, -96.21, 343.5), "label.N") % ("N"),
    Text("Cardinal", Vector3.New(-64   , -96.22, 357  ), "label.E") % ("E"),
    Text("Cardinal", Vector3.New(-76.75, -96.21, 369.5), "label.S") % ("S"),
    Text("Cardinal", Vector3.New(-90   , -96.21, 357  ), "label.W") % ("W"),

    Text("WorldEnder", Vector3.New(  -155, -97,    347), "W1") % (1),
    Text("WorldEnder", Vector3.New(   -43, -97,    284), "W2") % (2),
    Text("WorldEnder", Vector3.New(-19.37, -96, 414.22), "W3") % (3),
    Text("WorldEnder", Vector3.New(     3, -97,    340), "W4") % (4),
  }

  UnitCreated{ strUnitName = "unit.aldinari",
    Unit("Aldinari", _arg.tUnit),
    LineBetween("Aldinari", v3Center, _arg.tUnit) + {nOffset = 9},
  }
  UnitCreated{ strUnitName = {"===", _arg.self.L["unit.cassus"]},
    Unit("Cassus", _arg.tUnit),
    LineBetween("Cassus", v3Center, _arg.tUnit) + {nOffset = 9},
  }
  UnitCreated{ strUnitName = "unit.vulpes_nix",
    Unit("VulpesNix", _arg.tUnit),
    LineBetween("VulpesNix", v3Center, _arg.tUnit) + {nOffset = 9},
  }


  _var.nAsteroidCounter = 0 --for :GetAsteroidMark()
  _var.nLastAsteroidWave = 0
  UnitCreated{ strUnitName = "unit.rogue_asteroid", [_arg.self:HasUnit("Asteroid", _arg.nUnitId)] = false,
    {function(self, tUnit, nUnitId)
      local strMark = self:GetAsteroidMark(tUnit)
      self:Unit("Asteroid", tUnit, nUnitId, {strMark = strMark})
      self:LineBetween("Asteroid", tUnit, nil, nUnitId)
      self:Text("Asteroid", tUnit, strMark, nUnitId)
    end, _arg.self, _arg.tUnit, _arg.nUnitId}
  }

  _var.nWorldEnderCount = 0
  UnitCreated{ strUnitName = "unit.world_ender", [_arg.self:HasUnit("WorldEnder", _arg.nUnitId)] = false,
    _var.nWorldEnderCount+1,
    FireCallbacks("WorldEnderSpawned"),
    Unit("WorldEnder", _arg.tUnit),
    Alert("WorldEnder", "alert.world_ender"),
    Sound("WorldEnder"),
    LineBetween("WorldEnder", _arg.tUnit, nil),
    Delay(10,
      -Text("WorldEnder", _var.nWorldEnderCount),
      -LineBetween("WorldEnder") --if this worldender is ignored by the raid, we dont want to show the line permanently
    ),
  }

  _var.nUpcomingWormhole = 1
  UnitCreated{ strUnitName = "unit.wormhole",
    _var.nUpcomingWormhole+1,
  }

  local CRITICAL_MASS = 84345
  BuffAdded{ nSpellId = CRITICAL_MASS,
    Alert("CriticalMass", _arg.self.L["alert.critical_mass"]:format(_arg.strUnitName, _var.nUpcomingWormhole)),
    If{ [_arg.tUnit:IsThePlayer()] = true,
      Aura("CriticalMass", _arg.nDuration),
      Sound("CriticalMass")
    },
  }
  BuffRemoved{ nSpellId = CRITICAL_MASS,
    If{ [_arg.tUnit:IsThePlayer()] = true,
      -Aura("CriticalMass")
    },
  }

  _var.nLastPulsarSound = 0
  local nGameTime = _env.GameLib.GetGameTime()
  UnitCreated{ strUnitName = "unit.pulsar",
    Unit("Pulsar", _arg.tUnit) % (_arg.nUnitId),
    Alert("Pulsar", "alert.pulsar"),
    If{ [_var.nLastPulsarSound] = {"<", nGameTime-5}, --make this sound max every 5sec.
      _var.nLastPulsar(nGameTime),
      Sound("Pulsar"),
    }
  }

  UnitCreated{ strUnitName = "unit.debris_field",
    {function(self, tUnit)
      local v3Pos = Vector3.New(tUnit:GetPosition())
      local nDistance = (v3Center - v3Pos):Length()
      if nDistance > 45 then -- further away than Cassus
        self:TrackUnit("Boss", tUnit, "DebrisFieldVulpesNix")
        self:TimerText("DebrisField", _arg.tUnit, 26, "DebrisFieldVulpesNix")
      elseif nDistance > 25 then --further away than Aldinari
        self:TrackUnit("Boss", tUnit, "DebrisFieldCassus")
        self:TimerText("DebrisField", _arg.tUnit, 26, "DebrisFieldCassus")
      else
        self:TrackUnit("Boss", tUnit, "DebrisFieldAldinari")
        self:TimerText("DebrisField", _arg.tUnit, 26, "DebrisFieldAldinari")
      end
    end, _arg.self, _arg.tUnit}
  }
  Callback("DebrisFieldSpawnedAsteroid", {
    TimerText("DebrisField", _arg.self:GetUnit("Boss", _arg.strUnitSuffix), 26) % (_arg.strUnitSuffix),
  })

  UnitCreated{ strUnitName = "unit.cosmic_debris",
    LineBetween("DebrisLine", _arg.tUnit, nil) % (_arg.nUnitId),
    Polygon("DebrisPolygon", _arg.tUnit, 3, 6) % (_arg.nUnitId),

  }

  UnitCreated{ strUnitName = "unit.black_hole",
    Unit("BlackHole", _arg.tUnit),
  }

  _var.bIsStackingFlare = true
  CastStart{ strSpellName = "cast.solar_flare",
    If{ [_var.bIsStackingFlare] = true,
      Line("Boss", _arg.tUnit, 25),
      _var.bIsStackingFlare(false)
    }/{
      _var.bIsStackingFlare(true)
    },
  }
  CastEnd{ strSpellName = "cast.solar_flare",
    -Line("Boss"),
  }
  CastStart{ strSpellName = "cast.midphase",
    _var.bIsStackingFlare(true)
  }

  local SOLAR_WINDS = 87536
  BuffAdded{ nSpellId = SOLAR_WINDS, [_arg.tUnit:IsThePlayer()] = true,
    TimerText("SolarWinds", _arg.self:GetUnit("Aldinari"), 4),
  }
  _var.nLastWinds = 0
  BuffUpdated{ nSpellId = SOLAR_WINDS, [_arg.tUnit:IsThePlayer()] = true,
    TimerText("SolarWinds", _arg.self:GetUnit("Aldinari"), 4),
    If{ [_arg.nStack] = {">=", 7}, [_arg.nStack] = {">", _var.nLastWinds},
      Sound("SolarWinds"),
      Alert("SolarWinds", "alert.solar_winds")
    },
    _var.nLastWinds(_arg.nStack)
  }

  HealthReached{ strUnitName = "unit.boss", nHealthReached = {77, 47, 15},
    Alert("Midphase", "alert.midphase"),
    Sound("Midphase"),
  }
  HealthReached{ strUnitName = "unit.boss", nHealthReached = {80, 50, 17.5},
    HealthText("Midphase", _arg.tUnit, _arg.nHealthReached-5),
  }

end)

local v3Center = Vector3.New(-76.779, -96.22, 356.814)
local tDistances = {
  ["A"] = 23, --outer orbit distance. if the Asteroid spawns farther away, then its not from Aldinari.
  ["C"] = 44,
  ["V"] = 63,
}
function Mod:GetAsteroidMark(tUnit)
  local nDistance = (v3Center - Vector3.New(tUnit:GetPosition())):Length()
  if nDistance > tDistances["V"] then --we're further away, than any of the planet spawns could be - numeric assignment
    local nTime = GameLib.GetGameTime()
    if nTime - self.tVars.nLastAsteroidWave > 20 then
      self.tVars.nLastAsteroidWave = nTime
      self.tVars.nAsteroidCounter = 0
    end
    self.tVars.nAsteroidCounter = self.tVars.nAsteroidCounter+1
    return self.tVars.nAsteroidCounter
  elseif nDistance > tDistances["C"] then --we're further away, than cassus, but inside Vulpes Nix's range.
    self:FireCallbacks("DebrisFieldSpawnedAsteroid", {
      strUnitSuffix = "DebrisFieldVulpesNix"
    })
    return "V"
  elseif nDistance > tDistances["A"] then -- further than Aldinari, but in Cassus.
    self:FireCallbacks("DebrisFieldSpawnedAsteroid", {
      strUnitSuffix = "DebrisFieldCassus"
    })
    return "C"
  else
    self:FireCallbacks("DebrisFieldSpawnedAsteroid", {
      strUnitSuffix = "DebrisFieldAldinari"
    })
    return "A"
  end
end