local Vector3 = require "Vector3"

local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Encounter = tAddon.Encounter


local Mod = Encounter.new("Augmentors", {
  instance = "Initialization Core Y-83",
  displayName = "Augmentors",
  tTrigger = {
    sType = "ANY",
    tNames = {"unit.augmentor.inactive", "unit.augmentor.active", "unit.incinerator"},
    tZones = {
        [1] = {
            continentId = 8,
            parentZoneId = 0,
            mapId = 78,
        },
    },
  },
},{
  ["enUS"] = {
    -- Unit names.
    ["unit.augmentor.inactive"] = "Prime Evolutionary Operant",
    ["unit.augmentor.active"] = "Prime Phage Distributor",
    ["unit.add"] = "Sternum Buster",
    ["unit.incinerator"] = "Organic Incinerator",
    -- Datachron messages.
    ["datachron.irradiated"] = "([^%s]+%s[^%s]+) is being irradiated!",
    ["datachron.transmission"] = "ENGAGING TECHNO-PHAGE TRASMISSION",
    ["datachron.corrupted"] = "A Prime Purifier has been corrupted!",
    ["chron.decontamination"] = "INITIATING DECONTAMINATION SEQUENCE",
    -- Casts.
    ["cast.incinerator.disintegrate"] = "Disintegrate",
    ["cast.augmentor.digitize"] = "Digitize",
    ["cast.augmentor.injection"] = "Strain Injection",
    ["cast.augmentor.spike"] = "Corruption Spike",
    -- Bars messages.
    ["timer.next_bath"] = "Next bath in ...",
    ["alert.switch_soon"] = "Switch soon!",
    ["alert.corruption"] = "%u STACKS BEFORE CORRUPTION",

    ["label.safespot"] = "Safespot",
    ["label.boss_markers"] = "Boss Markers",
    ["label.boss"] = "Bosses",
    ["label.incubation"] = "Incubation (DoT)",
    ["label.corruption"] = "Stacks before corruption",

    ["label.N"] = "N",
    ["label.E"] = "E",
    ["label.S"] = "S",
    ["label.W"] = "W",
  },
  ["deDE"] = {
    ["unit.augmentor.inactive"] = "Hauptreiniger", --not sure. (from pre-release video)
    ["unit.augmentor.active"] = "Primärer Phage-Verteiler", --not sure. (from pre-release video)
    ["unit.add"] = "Brustbeinbrecher", --from adds infront
    ["unit.incinerator"] = "Organischer Flammenwerfer", --Valid

    ["datachron.irradiated"] = "([^%s]+%s[^%s]+) wird verstrahlt!", --guessed!
    ["datachron.transmission"] = "ÜBERTRAGUNG", --guessed!
    ["datachron.corrupted"] = "Ein Hauptreiniger wurde korrumpiert!", --guessed!
    ["chron.decontamination"] = "INITIIERE", --guessed!

    ["cast.incinerator.disintegrate"] = "Zerfall", --wtf? is this correct??
    ["cast.augmentor.digitize"] = "Digitalisieren",
    ["cast.augmentor.injection"] = "Transmutationsinjektion",
    ["cast.augmentor.spike"] = "Korrumpierungsstachel",

    ["timer.next_bath"] = "Nächstes Bad",
    ["alert.switch_soon"] = "Wechsel bald!",
    ["alert.corruption"] = "%u STACKS VOR KORRUMPIERUNG",

    ["label.safespot"] = "Safespot",
    ["label.boss_markers"] = "Boss Markierungen",
    ["label.boss"] = "Bosse",
    ["label.incubation"] = "Inkubation (DoT)",
    ["label.corruption"] = "Stacks vor Korrumpierung",

    ["label.N"] = "N",
    ["label.E"] = "O",
    ["label.S"] = "S",
    ["label.W"] = "W",
  },
  ["frFR"] = {
    -- Unit names.
    ["unit.augmentor.inactive"] = "Opérateur de la Primo Évolution",
    ["unit.augmentor.active"] = "Distributeur de Primo Phage",
    ["unit.add"] = "Exploseur sternum",
    ["unit.incinerator"] = "Incinérateur organique",
    -- Datachron messages.
    ["datachron.irradiated"] = "([^%s]+%s[^%s]+) est irradiée.",
    ["datachron.transmission"] = "ENCLENCHEMENT DE LA TRANSMISSION DU TECHNOPHAGE",
    ["datachron.corrupted"] = "Un Primo purificateur a été corrompu !",
    --["chron.decontamination"] = "TODO",
    -- Casts.
    ["cast.incinerator.disintegrate"] = "Désintégration",
    ["cast.augmentor.digitize"] = "Numérisation",
    ["cast.augmentor.injection"] = "Injection de la Souillure",
    ["cast.augmentor.spike"] = "Pointe de corruption",
    -- Bars messages.
    ["timer.next_bath"] = "~Prochaine irradiation",
    ["alert.switch_soon"] = "Switch soon!",
    ["alert.corruption"] = "%u STACKS AVANT CORRUPTION",

    ["label.safespot"] = "Safespot",
    ["label.boss_markers"] = "Boss Markers",
    ["label.boss"] = "Bosses",
    ["label.incubation"] = "Incubation (DoT)",
    ["label.corruption"] = "Stacks before corruption",

    ["label.N"] = "N",
    ["label.E"] = "E",
    ["label.S"] = "S",
    ["label.W"] = "W",
  },
}, function(tConfig) --luacheck: no unused args
  -- luacheck: no global

  Boss{
    label = "label.boss",
    unit{},
    line{thickness = 8, color = "ff00ff00"},
  }
  Safespot{
    label = "label.safespot",
    icon{sprite = "LUIBM_stop", size = 60, color = "ffff0000"},
  }
  BossMarkers{
    label = "label.boss_markers",
    line{thickness = 3, color = "xkcdAmber"}
  }
  Bath{
    label = "timer.next_bath",
    timer{color = "xkcdLightRed"},
    line{thickness = 3, color = "cyan"},
  }
  Incubation{
    label = "label.incubation",
    icon{sprite = "LUIBM_biohazard", size = 40},
    line{thickness = 5, color = "ffff00ff"},
  }
  Incinerator{ --the HM-Mechanic.
    label = "unit.incinerator",
    line{color = false, thickness = 9},
  }
  Switch{
    label = "alert.switch_soon",
    alert{},
    sound{file = "long"},
    text{},
  }
  Corruption{
    label = "label.corruption",
    alert{color = false},
  }

end, function(_arg, _env, _var) --luacheck: no unused args
  -- luacheck: no global

  Enable{
    Timer("Bath", 27, "timer.next_bath"),
  }

  local v3Center = Vector3.New(1268.13, -800.51, 877.117)
  UnitCreated{ strUnitName = {"unit.augmentor.inactive", "unit.augmentor.active"},
    If{ [_arg.tUnit:GetPosition().z] = {"<", v3Center.z}, --The North Augmentor.
      Unit("Boss", _arg.tUnit) % ("North") + {strMark = "label.N", position = 1},
    }/{ [_arg.tUnit:GetPosition().x] = {"<", v3Center.x}, --The West Augmentor.
      Unit("Boss", _arg.tUnit) % ("West") + {strMark = "label.W", position = 2},
    }/{ --The East Augmentor.
      Unit("Boss", _arg.tUnit) % ("East") + {strMark = "label.E", position = 3},
    },

    Line("Boss", _arg.tUnit, 15) % (_arg.nUnitId),
    {"DrawBossMarkers", _arg.tUnit, _arg.nUnitId},
  }


  local tSafespots = {
    Vector3.New(1238.03, -800.51, 894.45), --West
    Vector3.New(1268.17, -800.51, 842.32), --North
    Vector3.New(1298.20, -800.51, 894.57), --East
  }
  local fClosestSafespot = _arg(function(tUnit)
    local v3Pos = Vector3.New(tUnit:GetPosition())
    local nDist = (tSafespots[1] - v3Pos):Length()
    local tClosest = tSafespots[1]
    for i = 2, #tSafespots, 1 do
      local nNewDist = (tSafespots[i] - v3Pos):Length()
      if nNewDist < nDist then
        nDist = nNewDist
        tClosest = tSafespots[i]
      end
    end
    return tClosest
  end)

  local DEBUFF_INCUBATION = 49303 --STRAIN_INCUBATION
  BuffAdded{ nSpellId = DEBUFF_INCUBATION,
    Icon("Incubation", _arg.tUnit) % (_arg.nUnitId),
    LineBetween("Incubation", _arg.tUnit, fClosestSafespot(_arg.tUnit)) % (_arg.nUnitId) + {
      min = 5, --min 5m distance to show.
      max = 40, --max 40m distance to show.
    },
  }
  BuffRemoved{ nSpellId = DEBUFF_INCUBATION,
    -Icon("Incubation") % (_arg.nUnitId),
    -LineBetween("Incubation") % (_arg.nUnitId),
  }

  UnitCreated{ strUnitName = "unit.incinerator",
    TrackUnit("Boss", _arg.tUnit) % ("Incinerator"), --we need the casts from this unit.
  }
  CastStart{ strSpellName = "cast.incinerator.disintegrate",
    Line("Incinerator", _arg.tUnit, 65) + {color = "ffff0000"},
  }
  local DEBUFF_INTERRUPTED_INCINERATOR = 81783 -- PAIN_SUPPRESSORS HM debuff, Laser interrupt
  _var.nLastInterrupt = 0 --the Debuff lasts 7 Sec. RaidCore does something similar.
  BuffAdded{ nSpellId = DEBUFF_INTERRUPTED_INCINERATOR, [_var.nLastInterrupt] = {"<", GameTime()-7},
    _var.nLastInterrupt(GameTime()),
    Line("Incinerator", _arg.self:GetUnit("Boss", "Incinerator"), 65) + {color = "6000ff00"},
    Delay("IncineratorAboutRestored", 4,
      Line("Incinerator", _arg.self:GetUnit("Boss", "Incinerator"), 65) + {color = "A0ff8000"}
    ),
    Delay("IncineratorRestored", 5,
      Line("Incinerator", _arg.self:GetUnit("Boss", "Incinerator"), 65) + {color = "ffff0000"}
    ),
  }

  HealthReached{ strUniTname = "unit.augmentor.active", nHealthReached = {63, 23},
    Alert("Switch", "alert.switch_soon"),
    Sound("Switch"),
  }

  local BUFF_BOSS_ACTIVE = 48735 --COMPROMISED_CIRCUITRY =
  BuffAdded{ nSpellId = BUFF_BOSS_ACTIVE,
    If{ [_arg.self:GetHealth(_arg.tUnit)] = {">", 60}, --next target: 60%!
      HealthText("Switch", _arg.tUnit, 60),
    }/{ [_arg.self:GetHealth(_arg.tUnit)] = {">", 20}, --next target: 20%!
      HealthText("Switch", _arg.tUnit, 20),
    }/{
      -HealthText("Switch"),
    }
  }

  local BUFF_CORRUPTION_EASY = 50075 --NANOSTRAIN_INFUSION
  local BUFF_CORRUPTION_HARD = 80483
  BuffUpdated{ nSpellId = {BUFF_CORRUPTION_EASY, BUFF_CORRUPTION_HARD}, nStack = {">=", 13},
    If{ nStack = 13,
      Alert("Corruption", {"alert.corruption", 2}) + {color = "blue"}
    }/{ nStack = 14,
      Alert("Corruption", {"alert.corruption", 1}) + {color = "red"}
    }
  }

  Datachron{ strMessage = "datachron.corrupted", --20% Phase (Last Phase)
    -HealthText("Switch"),
    HealthText("Switch", _arg.self:GetUnit("Boss", "North"), 0), -- the 0 is not nessecary (target health)
    HealthText("Switch", _arg.self:GetUnit("Boss", "South"), 0),
    HealthText("Switch", _arg.self:GetUnit("Boss", "West"), 0),
  }

  Datachron{ strMessage = "datachron.transmission",
    Timer("Bath", 40, "timer.next_bath"),
  }

  local DEBUFF_IN_BATH = 71188 -- RADIATION_BATH Cleansing debuff in bath
  _var.nLastBath = 0
  BuffAdded{ nSpellId = DEBUFF_IN_BATH, [_var.nLastBath] = {"<", GameTime()-12},
    LineBetween("Bath", _arg.tUnit:GetPosition(), nil) + {min = 10, nDuration = 10}
  }

  Datachron{ strMessage = {"match", "datachron.irradiated"},
    Timer("Bath", 26, "timer.next_bath"),
    LineBetween("Bath", _env.GameLib.GetPlayerUnitByName(_arg.strMatch), nil) + {min = 10}
  }

end)


local v3Center = Vector3.New(1268.13, -800.51, 877.117)
local tDrawConstants = {
  ["Offset.L"] = Vector3.New(11.21, 0, 6.68),
  ["Offset.R"] = Vector3.New(-10.63, 0, 6.87),
  ["Offset.F"] = Vector3.New(0, 0, -6.48),
  ["Offset.B"] = Vector3.New(0, 0, 13.01),

  ["Rotation.L"] = -90,
  ["Rotation.R"] =  90,
  ["Rotation.BL"] = -30,
  ["Rotation.BR"] = 30,

  ["Distance.Safespot"] = 12,
}
function Mod:DrawBossMarkers(tUnit, strKeyExt)
  strKeyExt = strKeyExt or ""

  local v3Pos = Vector3.New(tUnit:GetPosition())
  local v3Facing = (v3Pos-v3Center):Normal()

  self:Line("BossMarkers", v3Pos, 35, strKeyExt.."Front towards Left", {
    tVectorFacing = v3Facing,
    tVectorOffsets = tDrawConstants["Offset.F"],
    nRotation = tDrawConstants["Rotation.L"],
  })
  self:Line("BossMarkers", v3Pos, 35, strKeyExt.."Front towards Right", {
    tVectorFacing = v3Facing,
    tVectorOffsets = tDrawConstants["Offset.F"],
    nRotation = tDrawConstants["Rotation.R"]
  })

  self:Line("BossMarkers", v3Pos, 25, strKeyExt.."Left towards Left", {
    tVectorFacing = v3Facing,
    tVectorOffsets = tDrawConstants["Offset.L"],
    nRotation = tDrawConstants["Rotation.L"]
  })
  self:Line("BossMarkers", v3Pos, 25, strKeyExt.."Right towards Right", {
    tVectorFacing = v3Facing,
    tVectorOffsets = tDrawConstants["Offset.R"],
    nRotation = tDrawConstants["Rotation.R"]
  })

  self:Line("BossMarkers", v3Pos, 25, strKeyExt.."Left towards Back-Left", {
    tVectorFacing = v3Facing,
    tVectorOffsets = tDrawConstants["Offset.L"],
    nRotation = tDrawConstants["Rotation.BL"]
  })
  self:Line("BossMarkers", v3Pos, 25, strKeyExt.."Right towards Back-Right", {
    tVectorFacing = v3Facing,
    tVectorOffsets = tDrawConstants["Offset.R"],
    nRotation = tDrawConstants["Rotation.BR"]
  })

  self:Line("BossMarkers", v3Pos, 25, strKeyExt.."Back towards Back-Left", {
    tVectorFacing = v3Facing,
    tVectorOffsets = tDrawConstants["Offset.B"],
    nRotation = tDrawConstants["Rotation.BL"]
  })
  self:Line("BossMarkers", v3Pos, 25, strKeyExt.."Back towards Back-Right", {
    tVectorFacing = v3Facing,
    tVectorOffsets = tDrawConstants["Offset.B"],
    nRotation = tDrawConstants["Rotation.BR"]
  })
  self:Icon("Safespot", v3Pos - (v3Facing*tDrawConstants["Distance.Safespot"]), strKeyExt)
end