local Apollo = require "Apollo"
local GameLib = require "GameLib"
local tAddon = Apollo.GetAddon("MischhBossModules")
local Vector3 = require "Vector3"

local Encounter = tAddon.Encounter


--[[
  LUI_BossMods is not perfect. There are errors in it, and scince i dont want to distribute a new LUI_BossMods,
  i will try to identify these addons here and modify the calls i do to it, to circumvent these Problems.
]]

local function HookLUI(strHook, fCreator)
  tAddon.Injector:RegisterCallback(function(tLUI)
    while type(tLUI) == "table" do
      if rawget(tLUI, strHook) then break end

      local tMeta = getmetatable(tLUI)
      tLUI = tMeta and tMeta.__index or nil
    end
    if type(tLUI) ~= "table" then return end


    local fOrig = tLUI[strHook]
    tLUI[strHook] = fCreator(fOrig)
  end)
end

do
--[[Fix1: Lines throw errors, if supplied with nDuration]]
--[[Fix2: Lines tVectorFacing doesn't work with units as Origin]]

  local fOrigLine = Encounter.Line

  function Encounter:Line(strKey, tOrigin, nLength, strKeyExt, tOverrides, ...)

    --[[Fix2:]]
    local fOrigType = _G.type
    if tOverrides and tOverrides.tVectorFacing and
      (type(tOrigin) == "number" or type(tOrigin) == "userdata" and not Vector3.Is(tOrigin)) then

      local tUnit = tOrigin
      local tFacing = tOverrides.tVectorFacing

      if type(tOrigin) == "number" then
        tUnit = GameLib.GetUnitById(tOrigin)
      end

      local tFakeUnit = setmetatable({
        GetFacing = function() return tFacing end
      }, {__index = function(tFake, strMethod)
        tFake[strMethod] = function(_, ...)
          return tUnit[strMethod](tUnit, ...)
        end
        return tFake[strMethod]
      end})

      tOrigin = tFakeUnit

      _G.type = function(...)
        if (...) == tFakeUnit then
          return "userdata"
        else
          return fOrigType(...)
        end
      end
    end


    local ret = {fOrigLine(self, strKey, tOrigin, nLength, strKeyExt, tOverrides, ...)}
    _G.type = fOrigType

    --[[Fix1:]]
    if strKeyExt then
      strKey = strKey..strKeyExt
    end
    strKey = "Line"..strKey
    local tDraw = self.core.tDraws and self.core.tDraws[strKey]
    if tDraw then
      tDraw.nTick = Apollo.GetTickCount()
    end



    return unpack(ret)
  end

end

do
  --[[Fix1: OnUnitCreated dosnt fire properly, if you haven't reloaded yet.]]
  HookLUI("OnDocLoaded", function(fOrig)
    return function(...)
      local self = (...)
      self.tPreloadUnits = self.tPreloadUnits or {}
      return fOrig(...)
    end
  end)
end

do
  --[[Fix1: A removed unit's Cast is not tracked properly. It should probably be removed.]]
  HookLUI("RemoveUnit", function(fOrig)
    return function(...)
      local self, nId = ...
      local bSuccess = fOrig(...)

      if bSuccess and self.runtime and self.runtime.cast and self.runtime.cast.nUnitId == nId then
        self:HideCast()
      end

      return bSuccess
    end
  end)
end

do
  --[[Fix1: A Unit that has not gotten a Buff/Debuff-bar Applied to itself does not throw Buff/Debuff Updates]]

  local wndBuff = nil
  local arUnitCache = {}

  tAddon:OnceDocLoaded(function()
    wndBuff = tAddon:LoadForm("BuffWindow", nil, nil)
    for _, tUnit in ipairs(arUnitCache) do
      if tUnit:IsValid() then
        wndBuff:SetUnit(tUnit)
      end
    end
    wndBuff:SetUnit(nil)
  end)


  HookLUI("AddUnit", function(fOrig)
    return function(...)
      -- self:AddUnit(nId,sName,tUnit,tConfig,sMark)
      local self, nId, sName, tUnit = ... --luacheck: no unused

      if wndBuff then
        wndBuff:SetUnit(tUnit)
        wndBuff:SetUnit(nil) --this probably requires less resources than letting it display further.
      else
        table.insert(arUnitCache, tUnit)
      end

      return fOrig(...)
    end
  end)
end