local Apollo = require "Apollo"
local tAddon = Apollo.GetAddon("MischhBossModules")

local Injector = {
  bIsReady = false,
  tInjections = {},
  tReplaced = {},
  tLUI = nil,

  tCallbacks = {},
}
tAddon.Injector = Injector

function Injector:Inject(strEncounter, tEncounter)
  self.tInjections[strEncounter] = tEncounter

  if self.bIsReady then
    self:InjectThis(strEncounter,tEncounter)
  end

end

function Injector:InjectThis(strEncounter, tEncounter)
  if not self.tReplaced[strEncounter] then
    self.tReplaced[strEncounter] = self.tLUI.modules[strEncounter] --might be nil.
  end
  self.tLUI.modules[strEncounter] = tEncounter
end

function Injector:InjectAll()
  for strEncounter, tEncounter in pairs(self.tInjections) do
    self:InjectThis(strEncounter, tEncounter)
  end
end

function Injector:HasInjected()
  return self.bIsReady
end

function Injector:GetInjections()
  return self.tInjections
end

function Injector:GetLUI()
  return self.tLUI
end

function Injector:RevertInjectionsFor(fFunc, ...)
  if self.bIsReady then
    for strEncounter, tEncounter in pairs(self.tReplaced) do
      self.tLUI.modules[strEncounter] = tEncounter
    end
    fFunc(...)
    for strEncounter, _ in pairs(self.tReplaced) do
      self.tLUI.modules[strEncounter] = self.tInjections[strEncounter]
    end
  else
    fFunc(...) --nothing to revert, scince we have not Injected (yet?)
  end
end

function Injector:RegisterCallback(oFunc, oHandler, ...)
  local fCallback = tAddon.Utils.CreateCallback(1, oFunc, oHandler, ...)

  if self.bIsReady then
    self:FireCallback(fCallback)
  else
    table.insert(self.tCallbacks, fCallback)
  end
end

function Injector:FireCallback(fCallback)
  fCallback(self:GetLUI())
end

function Injector:FireCallbacks()
  for _, fCallback in ipairs(self.tCallbacks) do
    self:FireCallback(fCallback)
  end
  self.tCallbacks = {}
end

tAddon:OnceLoaded("CheckReady", Injector)
function Injector:CheckReady()
  local tLUI = Apollo.GetAddon("LUI_BossMods")

  if tLUI then
    if not self.bIsReady then
      self.bIsReady = true
      self.tLUI = tLUI

      self:InjectAll()
      self:FireCallbacks()
    end
  end
end

tAddon:OnceDocLoaded("ReportMissing", Injector)
function Injector:ReportMissing()
  if self.bIsReady then return end

  error("LUI_BossMods not found! This Addon requires LUI_BossMods to function!")
end

Injector:CheckReady()