local Apollo = require "Apollo"
local GameLib = require "GameLib"
local ApolloTimer = require "ApolloTimer"
local tAddon = Apollo.GetAddon("MischhBossModules")

local Savior = {
  tSaveData = nil, --we save our restored data here.
  tLateRestoreTimer = nil,
}
tAddon.Savior = Savior

tAddon:OnceSaving("OnSave", Savior)
function Savior:OnSave(eSaveLevel, tSaveData)
  if eSaveLevel ~= GameLib.CodeEnumAddonSaveLevel.General then return end

  if tAddon.Injector and tAddon.Injector:HasInjected() then --retrieve data from the modules
    tSaveData.Savior = {}

    for strEncounter, tEncounter in pairs(tAddon.Injector:GetInjections() or {}) do
      tSaveData.Savior[strEncounter] = tEncounter.config
    end

    return tSaveData
  else -- re-save the previously restored data.
    tSaveData.Savior = self.tSaveData
    return tSaveData
  end
end

tAddon:OnceRestored("OnRestore", Savior)
function Savior:OnRestore(eSaveLevel, tSaveData)
  if eSaveLevel ~= GameLib.CodeEnumAddonSaveLevel.General then return end
  self.tSaveData = tSaveData and tSaveData.Savior

  self.tLateRestoreTimer = ApolloTimer.Create(1, false, "OnLateRestore", self)
end

function Savior:OnLateRestore() --we want to force our Save-Data after LUI_BossMods applies theirs.
  self.tLateRestoreTimer = nil
  if not self.tSaveData then return end
  if not tAddon.Injector or not tAddon.Injector:HasInjected() then return end

  local tSaveData = self.tSaveData
  local tLUI = tAddon.Injector:GetLUI()

  for strEncounter, tEncounter in pairs(tAddon.Injector:GetInjections() or {}) do
    if tSaveData[strEncounter] then
      local tCheckedConfig = tLUI:CheckSavedData(tSaveData[strEncounter], tEncounter.config)
      local tDefultedConfig = tLUI:InsertDefaults(tCheckedConfig, tEncounter.config)
      tEncounter.config = tDefultedConfig
    end
  end
end

--[[
  Okay. we successfully intruded LUI_BossMods, replacing Encounters, managing the savefiles of them,
  but what happens, if you disable this addon? Well.. LUI_BossMods will load with settings not meant for
  the original Encounters, scince it saved our Encounters config aswell.
  This is where we try to prevent this: Let LUI_BossMods Restore/Save their own Encounters instead.
]]
function Savior:HookLUI(tLUI) --luacheck: no unused args (self)
  local fSave = tLUI.OnSave --these are the original functions from LUI
  local fRestore = tLUI.OnRestore --we will replace these with our own.

  function tLUI.OnSave(...)
    tAddon.Injector:RevertInjectionsFor(fSave, ...)
  end

  function tLUI.OnRestore(...)
    tAddon.Injector:RevertInjectionsFor(fRestore, ...)
  end

end
tAddon.Injector:RegisterCallback("HookLUI", Savior)